@@ -1,4 +1,4 @@
-// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2007, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -71,7 +71,7 @@ SQLColAttribute(SQLHSTMT hstmt, SQLUSMALLINT column,
 #endif
                )
 {
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
   return SQLColAttributeImpl(hstmt, column, field, char_attr, char_attr_max,
                              char_attr_len, num_attr);
 }
@@ -115,7 +115,7 @@ SQLColAttributes(SQLHSTMT hstmt, SQLUSMALLINT column, SQLUSMALLINT field,
                  SQLPOINTER char_attr, SQLSMALLINT char_attr_max,
                  SQLSMALLINT *char_attr_len, SQLLEN *num_attr)
 {
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
   return SQLColAttributeImpl(hstmt, column, field, char_attr, char_attr_max,
                              char_attr_len, num_attr);
 }
@@ -131,7 +131,7 @@ SQLColumnPrivileges(SQLHSTMT hstmt,
   SQLRETURN rc;
   DBC *dbc;
 
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   dbc= ((STMT *)hstmt)->dbc;
 
@@ -151,7 +151,7 @@ SQLColumns(SQLHSTMT hstmt, SQLCHAR *catalog, SQLSMALLINT catalog_len,
   SQLRETURN rc;
   DBC *dbc;
 
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   dbc= ((STMT *)hstmt)->dbc;
 
@@ -206,7 +206,7 @@ SQLDescribeCol(SQLHSTMT hstmt, SQLUSMALLINT column,
 
   SQLRETURN rc;
 
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   rc= MySQLDescribeCol(hstmt, column, &value, &free_value, type,
                                  size, scale, nullable);
@@ -291,12 +291,13 @@ SQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd, SQLCHAR *in, SQLSMALLINT in_len,
   {
     uint errors;
     /* Now we have to convert SQLWCHAR back into a SQLCHAR. */
-    *out_len= (SQLSMALLINT)sqlwchar_as_sqlchar_buf(default_charset_info, out, out_max,
-                                      outw, *out_len, &errors);
-    /* TODO what to do with errors? */
-
+    sqlwchar_as_sqlchar_buf(default_charset_info, out, out_max,
+                            outw, *out_len, &errors);
     if (*out_len > out_max - 1)
-      rc= ((DBC*)hdbc)->set_error("01004", NULL, 0);
+    {
+      ((DBC*)hdbc)->set_error("01004", "String data, right truncated.", 0);
+      rc = SQL_SUCCESS_WITH_INFO;
+    }
   }
 
 error:
@@ -342,7 +343,7 @@ SQLExecDirect(SQLHSTMT hstmt, SQLCHAR *str, SQLINTEGER str_len)
 {
   int error;
 
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   if ((error= SQLPrepareImpl(hstmt, str, str_len, false)))
     return error;
@@ -364,7 +365,7 @@ SQLForeignKeys(SQLHSTMT hstmt,
   SQLRETURN rc;
   DBC *dbc;
 
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   dbc= ((STMT *)hstmt)->dbc;
 
@@ -415,7 +416,7 @@ SQLGetConnectAttrImpl(SQLHDBC hdbc, SQLINTEGER attribute, SQLPOINTER value,
       see: "if (char_value)"
     */
     if (len > value_max - 1)
-      rc= set_conn_error(dbc, MYERR_01004, NULL, 0);
+      rc = dbc->set_error(MYERR_01004, NULL, 0);
 
     if (value && value_max > 1)
       strmake((char *)value, (char *)char_value, value_max - 1);
@@ -449,7 +450,7 @@ SQLGetCursorName(SQLHSTMT hstmt, SQLCHAR *cursor, SQLSMALLINT cursor_max,
   SQLINTEGER len;
   uint errors;
 
-  CHECK_HANDLE(stmt);
+  LOCK_STMT(stmt);
   CLEAR_STMT_ERROR(stmt);
 
   if (cursor_max < 0)
@@ -511,7 +512,7 @@ SQLGetDiagField(SQLSMALLINT handle_type, SQLHANDLE handle,
 
     /* We set the error only when the result is intented to be returned */
     if (info && len > info_max - 1)
-      rc= set_conn_error(dbc, MYERR_01004, NULL, 0);
+      rc = dbc->set_error(MYERR_01004, NULL, 0);
 
     if (info_len)
       *info_len= (SQLSMALLINT)len;
@@ -590,7 +591,7 @@ SQLGetDiagRecImpl(SQLSMALLINT handle_type, SQLHANDLE handle,
       and message_max is greaater than 0
     */
     if (message && message_max && len > message_max - 1)
-      rc= set_conn_error(dbc, MYERR_01004, NULL, 0);
+      rc = dbc->set_error(MYERR_01004, NULL, 0);
 
     if (message_len)
       *message_len= (SQLSMALLINT)len;
@@ -633,7 +634,7 @@ SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT type, SQLPOINTER value,
       value is not NULL and value_max is 0
      */
     if (value && value_max && len > value_max - 1)
-      rc= set_conn_error(dbc, MYERR_01004, NULL, 0);
+      rc = dbc->set_error(MYERR_01004, NULL, 0);
 
     if (value && value_max > 1)
       strmake((char *)value, (char *)char_value, value_max - 1);
@@ -651,7 +652,7 @@ SQLRETURN SQL_API
 SQLGetStmtAttr(SQLHSTMT hstmt, SQLINTEGER attribute, SQLPOINTER value,
                 SQLINTEGER value_max, SQLINTEGER *value_len)
 {
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   /* Nothing special to do, since we don't have any string stmt attribs */
   return MySQLGetStmtAttr(hstmt, attribute, value, value_max, value_len);
@@ -661,7 +662,7 @@ SQLGetStmtAttr(SQLHSTMT hstmt, SQLINTEGER attribute, SQLPOINTER value,
 SQLRETURN SQL_API
 SQLGetTypeInfo(SQLHSTMT hstmt, SQLSMALLINT type)
 {
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   return MySQLGetTypeInfo(hstmt, type);
 }
@@ -673,7 +674,7 @@ SQLNativeSql(SQLHDBC hdbc, SQLCHAR *in, SQLINTEGER in_len,
 {
   SQLRETURN rc= SQL_SUCCESS;
 
-  CHECK_HANDLE(hdbc);
+  LOCK_DBC(hdbc);
 
   if (in_len == SQL_NTS)
   {
@@ -687,7 +688,7 @@ SQLNativeSql(SQLHDBC hdbc, SQLCHAR *in, SQLINTEGER in_len,
 
   if (out && in_len >= out_max)
   {
-    rc= set_conn_error((DBC *)hdbc, MYERR_01004, NULL, 0);
+    rc = ((DBC *)hdbc)->set_error(MYERR_01004, NULL, 0);
   }
 
   if(out_max > 0)
@@ -708,7 +709,7 @@ SQLNativeSql(SQLHDBC hdbc, SQLCHAR *in, SQLINTEGER in_len,
 SQLRETURN SQL_API
 SQLPrepare(SQLHSTMT hstmt, SQLCHAR *str, SQLINTEGER str_len)
 {
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   return SQLPrepareImpl(hstmt, str, str_len, true);
 }
@@ -738,7 +739,7 @@ SQLPrimaryKeys(SQLHSTMT hstmt,
   SQLRETURN rc;
   DBC *dbc;
 
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   dbc= ((STMT *)hstmt)->dbc;
 
@@ -759,7 +760,7 @@ SQLProcedureColumns(SQLHSTMT hstmt,
   SQLRETURN rc;
   DBC *dbc;
 
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   dbc= ((STMT *)hstmt)->dbc;
 
@@ -780,7 +781,7 @@ SQLProcedures(SQLHSTMT hstmt,
   SQLRETURN rc;
   DBC *dbc;
 
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   dbc= ((STMT *)hstmt)->dbc;
 
@@ -833,7 +834,7 @@ SQLSetCursorName(SQLHSTMT hstmt, SQLCHAR *name, SQLSMALLINT name_len)
   SQLINTEGER len= name_len;
   uint errors= 0;
 
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
   return MySQLSetCursorName(hstmt, name, name_len);
 }
 
@@ -842,7 +843,7 @@ SQLRETURN SQL_API
 SQLSetStmtAttr(SQLHSTMT hstmt, SQLINTEGER attribute,
                SQLPOINTER value, SQLINTEGER value_len)
 {
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   /* Nothing special to do, since we don't have any string stmt attribs */
   return MySQLSetStmtAttr(hstmt, attribute, value, value_len);
@@ -859,7 +860,7 @@ SQLSpecialColumns(SQLHSTMT hstmt, SQLUSMALLINT type,
   SQLRETURN rc;
   DBC *dbc;
 
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   dbc= ((STMT *)hstmt)->dbc;
 
@@ -880,7 +881,7 @@ SQLStatistics(SQLHSTMT hstmt,
   SQLRETURN rc;
   DBC *dbc;
 
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   dbc= ((STMT *)hstmt)->dbc;
   rc= MySQLStatistics(hstmt, catalog, catalog_len, schema, schema_len,
@@ -898,7 +899,7 @@ SQLTablePrivileges(SQLHSTMT hstmt,
   SQLRETURN rc;
   DBC *dbc;
 
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   dbc= ((STMT *)hstmt)->dbc;
 
@@ -919,7 +920,7 @@ SQLTables(SQLHSTMT hstmt,
   SQLRETURN rc;
   DBC *dbc;
 
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   dbc= ((STMT *)hstmt)->dbc;
 
@@ -984,8 +985,8 @@ SQLBrowseConnect(SQLHDBC hdbc, SQLCHAR *in, SQLSMALLINT in_len,
 {
   CHECK_HANDLE(hdbc);
 
-  return set_conn_error((DBC*)hdbc,MYERR_S1000,
-                        "Driver does not support this API", 0);
+  return ((DBC*)hdbc)->set_error(MYERR_S1000,
+    "Driver does not support this API", 0);
 }
 
 
