@@ -1,4 +1,4 @@
-// Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -58,6 +58,10 @@ typedef BOOL (*PromptFunc)(SQLHWND, SQLWCHAR *, SQLUSMALLINT,
 
 const char *my_os_charset_to_mysql_charset(const char *csname);
 
+bool fido_callback_is_set = false;
+fido_callback_func global_fido_callback = nullptr;
+std::mutex global_fido_mutex;
+
 /**
   Get the connection flags based on the driver options.
 
@@ -86,75 +90,103 @@ unsigned long get_client_flags(DataSource *ds)
 }
 
 
+void DBC::set_charset(std::string charset)
+{
+  // Use SET NAMES instead of mysql_set_character_set()
+  // because running odbc_stmt() is thread safe.
+  std::string query = "SET NAMES " + charset;
+  if (execute_query(query.c_str(), query.length(), true))
+  {
+    throw MYERROR("HY000", mysql);
+  }
+}
+
 /**
- If it was specified, set the character set for the connection.
+ If it was specified, set the character set for the connection and
+ other internal charset properties.
 
  @param[in]  dbc      Database connection
  @param[in]  charset  Character set name
 */
-SQLRETURN myodbc_set_initial_character_set(DBC *dbc, const char *charset)
+SQLRETURN DBC::set_charset_options(const char *charset)
+try
 {
-  if (dbc->unicode)
+  if (unicode)
   {
     if (charset && charset[0])
     {
-      dbc->ansi_charset_info= get_charset_by_csname(charset,
-                                                    MYF(MY_CS_PRIMARY),
-                                                    MYF(0));
-      if (!dbc->ansi_charset_info)
+      ansi_charset_info= get_charset_by_csname(charset,
+                                               MYF(MY_CS_PRIMARY),
+                                               MYF(0));
+      if (!ansi_charset_info)
       {
-        char errmsg[NAME_LEN + 32*SYSTEM_CHARSET_MBMAXLEN];
+        std::string errmsg = "Wrong character set name ";
+        errmsg.append(charset);
         /* This message should help the user to identify the error */
-        sprintf(errmsg, "Wrong character set name %.*s", NAME_LEN, charset);
-        dbc->set_error("HY000", errmsg, 0);
-
-        return SQL_ERROR;
+        throw MYERROR("HY000", errmsg);
       }
     }
 
-    charset= "utf8";
+    charset = transport_charset;
   }
 
   if (charset && charset[0])
-  {
-    if (mysql_set_character_set(dbc->mysql, charset))
-    {
-      dbc->set_error("HY000", mysql_error(dbc->mysql),
-                     mysql_errno(dbc->mysql));
-      return SQL_ERROR;
-    }
-  }
+    set_charset(charset);
   else
-  {
-    if (mysql_set_character_set(dbc->mysql, dbc->ansi_charset_info->csname))
-    {
-      dbc->set_error("HY000", mysql_error(dbc->mysql),
-                     mysql_errno(dbc->mysql));
-      return SQL_ERROR;
-    }
-  }
+    set_charset(ansi_charset_info->csname);
 
   {
     MY_CHARSET_INFO my_charset;
-    mysql_get_character_set_info(dbc->mysql, &my_charset);
-    dbc->cxn_charset_info= get_charset(my_charset.number, MYF(0));
+    mysql_get_character_set_info(mysql, &my_charset);
+    cxn_charset_info = get_charset(my_charset.number, MYF(0));
   }
 
-  if (!dbc->unicode)
-    dbc->ansi_charset_info= dbc->cxn_charset_info;
+  if (!unicode)
+    ansi_charset_info = cxn_charset_info;
 
   /*
     We always set character_set_results to NULL so we can do our own
     conversion to the ANSI character set or Unicode.
   */
-  if (is_minimum_version(dbc->mysql->server_version, "4.1.1")
-      && odbc_stmt(dbc, "SET character_set_results = NULL", SQL_NTS, TRUE) != SQL_SUCCESS)
+  if (execute_query("SET character_set_results = NULL", SQL_NTS, true)
+    != SQL_SUCCESS)
   {
-    return SQL_ERROR;
+    throw error;
   }
 
   return SQL_SUCCESS;
 }
+catch(const MYERROR &e)
+{
+  error = e;
+  return e.retcode;
+}
+
+
+SQLRETURN run_initstmt(DBC* dbc, DataSource* dsrc)
+try
+{
+  if (dsrc->initstmt && dsrc->initstmt[0])
+  {
+    /* Check for SET NAMES */
+    if (is_set_names_statement(ds_get_utf8attr(dsrc->initstmt,
+      &dsrc->initstmt8)))
+    {
+      throw MYERROR("HY000", "SET NAMES not allowed by driver");
+    }
+
+    if (dbc->execute_query((char*)dsrc->initstmt8, SQL_NTS, true) != SQL_SUCCESS)
+    {
+      return SQL_ERROR;
+    }
+  }
+  return SQL_SUCCESS;
+}
+catch (const MYERROR& e)
+{
+  dbc->error = e;
+  return e.retcode;
+}
 
 /*
   Retrieve DNS+SRV list.
@@ -270,13 +302,14 @@ class dbc_guard
 */
 SQLRETURN DBC::connect(DataSource *dsrc)
 {
-  SQLRETURN rc= SQL_SUCCESS;
+  SQLRETURN rc = SQL_SUCCESS;
   unsigned long flags;
   /* Use 'int' and fill all bits to avoid alignment Bug#25920 */
   unsigned int opt_ssl_verify_server_cert = ~0;
-  const my_bool on= 1;
+  const my_bool on = 1;
   unsigned int on_int = 1;
   unsigned long max_long = ~0L;
+  bool initstmt_executed = false;
 
   dbc_guard guard(this);
 
@@ -286,21 +319,21 @@ SQLRETURN DBC::connect(DataSource *dsrc)
    FLAG_COLUMN_SIZE_S32 option if we are.
   */
   if (GetModuleHandle("msado15.dll") != NULL)
-    dsrc->limit_column_size= 1;
+    dsrc->limit_column_size = 1;
 
   /* Detect another problem specific to MS Access */
   if (GetModuleHandle("msaccess.exe") != NULL)
-    dsrc->default_bigint_bind_str= 1;
+    dsrc->default_bigint_bind_str = 1;
 
   /* MS SQL Likes when the CHAR columns are padded */
   if (GetModuleHandle("sqlservr.exe") != NULL)
-    dsrc->pad_char_to_full_length= 1;
+    dsrc->pad_char_to_full_length = 1;
 
 #endif
 
   mysql = mysql_init(nullptr);
 
-  flags= get_client_flags(dsrc);
+  flags = get_client_flags(dsrc);
 
   /* Set other connection options */
 
@@ -309,7 +342,7 @@ SQLRETURN DBC::connect(DataSource *dsrc)
     mysql_options(mysql, MYSQL_OPT_MAX_ALLOWED_PACKET, &max_long);
 #else
     /* max_allowed_packet is a magical mysql macro. */
-    max_allowed_packet= ~0L;
+    max_allowed_packet = ~0L;
 #endif
 
   if (dsrc->force_use_of_named_pipes)
@@ -318,17 +351,6 @@ SQLRETURN DBC::connect(DataSource *dsrc)
   if (dsrc->read_options_from_mycnf)
     mysql_options(mysql, MYSQL_READ_DEFAULT_GROUP, "odbc");
 
-  if (dsrc->initstmt && dsrc->initstmt[0])
-  {
-    /* Check for SET NAMES */
-    if (is_set_names_statement(ds_get_utf8attr(dsrc->initstmt,
-                                              &dsrc->initstmt8)))
-    {
-      return set_error("HY000", "SET NAMES not allowed by driver", 0);
-    }
-    mysql_options(mysql, MYSQL_INIT_COMMAND, dsrc->initstmt8);
-  }
-
   if (login_timeout)
     mysql_options(mysql, MYSQL_OPT_CONNECT_TIMEOUT, (char *)&login_timeout);
 
@@ -367,6 +389,44 @@ SQLRETURN DBC::connect(DataSource *dsrc)
                   ds_get_utf8attr(dsrc->default_auth, &dsrc->default_auth8));
   }
 
+  /*
+   * If fido callback is used the lock must remain till the end of
+   * the connection process.
+   */
+  std::unique_lock<std::mutex> fido_lock(global_fido_mutex);
+  /*
+  * Set callback even if the value is NULL, but only to reset the previously
+  * installed callback.
+  */
+  fido_callback_func fido_func = fido_callback;
+
+  if(!fido_func && global_fido_callback)
+    fido_func = global_fido_callback;
+
+  if (fido_func || fido_callback_is_set)
+  {
+    struct st_mysql_client_plugin* plugin =
+      mysql_client_find_plugin(mysql,
+        "authentication_fido_client",
+        MYSQL_CLIENT_AUTHENTICATION_PLUGIN);
+
+    if (!plugin)
+    {
+      return set_error("HY000", "Couldn't load plugin authentication_fido_client", 0);
+    }
+
+    if (mysql_plugin_options(plugin, "fido_messages_callback", (const void*)fido_func))
+    {
+      return set_error("HY000", "Failed to set a FIDO authentication callback function", 0);
+    }
+    fido_callback_is_set = fido_func;
+  }
+  else
+  {
+    // No need to keep the lock if callback is not used.
+    fido_lock.unlock();
+  }
+
   if(dsrc->oci_config_file && dsrc->oci_config_file[0])
   {
     /* load client authentication plugin if required */
@@ -388,6 +448,40 @@ SQLRETURN DBC::connect(DataSource *dsrc)
     }
   }
 
+  if (dsrc->authentication_kerberos_mode &&
+      dsrc->authentication_kerberos_mode[0])
+  {
+#ifdef WIN32
+    /* load client authentication plugin if required */
+    struct st_mysql_client_plugin* plugin =
+      mysql_client_find_plugin(mysql,
+        "authentication_kerberos_client",
+        MYSQL_CLIENT_AUTHENTICATION_PLUGIN);
+
+    if (!plugin)
+    {
+      return set_error("HY000", mysql_error(mysql), 0);
+    }
+
+    if (mysql_plugin_options(plugin, "plugin_authentication_kerberos_client_mode",
+      ds_get_utf8attr(dsrc->authentication_kerberos_mode,
+        &dsrc->authentication_kerberos_mode8)))
+    {
+      return set_error("HY000",
+        "Failed to set mode for authentication_kerberos_client plugin", 0);
+    }
+#else
+    if (myodbc_strcasecmp("GSSAPI",
+      ds_get_utf8attr(dsrc->authentication_kerberos_mode,
+        &dsrc->authentication_kerberos_mode8)))
+    {
+      return set_error("HY000",
+        "Invalid value for authentication-kerberos-mode. "
+        "Only GSSAPI is supported.", 0);
+    }
+#endif
+  }
+
 #endif
 
   /* set SSL parameters */
@@ -416,29 +510,56 @@ SQLRETURN DBC::connect(DataSource *dsrc)
   {
     std::string tls_options;
 
-    std::map<std::string, bool> opts = {
-      { "TLSv1.2", !dsrc->no_tls_1_2 },
-      { "TLSv1.3", !dsrc->no_tls_1_3 },
-    };
-
-    for (auto &opt : opts)
+    if (dsrc->tls_versions)
     {
-      if (!opt.second)
-        continue;
+      // If tls-versions is used the NO_TLS_X options are deactivated
+      tls_options = ds_get_utf8attr(dsrc->tls_versions, &dsrc->tls_versions8);
+    }
+    else
+    {
+      std::map<std::string, bool> opts = {
+        { "TLSv1.2", !dsrc->no_tls_1_2 },
+        { "TLSv1.3", !dsrc->no_tls_1_3 },
+      };
 
-      if (!tls_options.empty())
-         tls_options.append(",");
-      tls_options.append(opt.first);
+      for (auto &opt : opts)
+      {
+        if (!opt.second)
+          continue;
+
+        if (!tls_options.empty())
+           tls_options.append(",");
+        tls_options.append(opt.first);
+      }
     }
 
-    if (tls_options.length())
-      mysql_options(mysql, MYSQL_OPT_TLS_VERSION, tls_options.c_str());
-    else
+    if (!tls_options.length() ||
+        mysql_options(mysql, MYSQL_OPT_TLS_VERSION, tls_options.c_str()))
+    {
       return set_error("HY000",
         "SSL connection error: No valid TLS version available", 0);
+    }
   }
 #endif
 
+  if (dsrc->ssl_crl)
+  {
+    if (mysql_options(mysql, MYSQL_OPT_SSL_CRL,
+      ds_get_utf8attr(dsrc->ssl_crl, &dsrc->ssl_crl8)))
+    {
+      return set_error("HY000", "Failed to set the certificate revocation list file", 0);
+    }
+  }
+
+  if (dsrc->ssl_crlpath)
+  {
+    if (mysql_options(mysql, MYSQL_OPT_SSL_CRLPATH,
+      ds_get_utf8attr(dsrc->ssl_crlpath, &dsrc->ssl_crlpath8)))
+    {
+      return set_error("HY000", "Failed to set the certificate revocation list path", 0);
+    }
+  }
+
 #if MYSQL_VERSION_ID >= 80004
   if (dsrc->get_server_public_key)
   {
@@ -458,7 +579,7 @@ SQLRETURN DBC::connect(DataSource *dsrc)
     /*
       We always use utf8 for the connection, and change it afterwards if needed.
     */
-    mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "utf8");
+    mysql_options(mysql, MYSQL_SET_CHARSET_NAME, transport_charset);
     cxn_charset_info= utf8_charset_info;
   }
   else
@@ -628,9 +749,9 @@ SQLRETURN DBC::connect(DataSource *dsrc)
     //Setting server and port to the dsrc->server8 and dsrc->port
     //TODO: IS THERE A FUNCTION TO DO THIS USING myodbc_malloc?
     //COPY OF sqlwchardup
-    size_t chars= strlen(host);
+    size_t chars = strlen(host);
     x_free(dsrc->server8);
-    dsrc->server8= (SQLCHAR *)myodbc_malloc(( chars + 1) , MYF(0));
+    dsrc->server8 = (SQLCHAR *)myodbc_malloc(( chars + 1) , MYF(0));
     memcpy(dsrc->server8, host, chars);
     dsrc->port = port;
 
@@ -678,7 +799,7 @@ SQLRETURN DBC::connect(DataSource *dsrc)
 #endif
       set_error("HY000", mysql_error(mysql), native_error);
 
-      translate_error(error.sqlstate, MYERR_S1000, native_error);
+      translate_error((char*)error.sqlstate.c_str(), MYERR_S1000, native_error);
 
       return SQL_ERROR;
     }
@@ -734,16 +855,15 @@ SQLRETURN DBC::connect(DataSource *dsrc)
 
     if(!connected)
     {
-      std::stringstream err;
       if(dsrc->enable_dns_srv)
       {
-        err << "Unable to connect to any of the hosts of "
-            << dsrc->server8 << " SRV";
-        set_error("HY000", err.str().c_str(), 0);
+        std::string err =
+          std::string("Unable to connect to any of the hosts of ") +
+          (const char*)dsrc->server8 + " SRV";
+        set_error("HY000", err.c_str(), 0);
       }
       else if (dsrc->multi_host && hosts.size() > 1) {
-        err << "Unable to connect to any of the hosts";
-        set_error("HY000", err.str().c_str(), 0);
+        set_error("HY000", "Unable to connect to any of the hosts", 0);
       }
       //The others will retrieve the error from connect
       return SQL_ERROR;
@@ -758,11 +878,26 @@ SQLRETURN DBC::connect(DataSource *dsrc)
     return set_error("08001", "Driver does not support server versions under 4.1.1", 0);
   }
 
-  rc= myodbc_set_initial_character_set(this, ds_get_utf8attr(dsrc->charset,
-                                                            &dsrc->charset8));
+  rc = set_charset_options(ds_get_utf8attr(dsrc->charset, &dsrc->charset8));
+
+  // It could be an error with expired password in which case we
+  // still try to execute init statements and retry below.
+  if (rc == SQL_ERROR && error.native_error != ER_MUST_CHANGE_PASSWORD)
+    goto error;
+
+  // Try running INITSTMT.
+  if (!SQL_SUCCEEDED(run_initstmt(this, dsrc)))
+    return error.retcode;
+
+  // If we had expired password error at the beginning
+  // try setting charset options again.
+  // NOTE: charset name is converted to a single-byte
+  //       charset by previous call to ds_get_utf8attr().
+  if (rc == SQL_ERROR)
+    rc = set_charset_options((const char*)dsrc->charset8);
+
   if (!SQL_SUCCEEDED(rc))
   {
-    /** @todo set failure reason */
     goto error;
   }
 
@@ -773,13 +908,12 @@ SQLRETURN DBC::connect(DataSource *dsrc)
     other problems.
   */
   if (!dsrc->auto_increment_null_search &&
-      odbc_stmt(this, "SET SQL_AUTO_IS_NULL = 0", SQL_NTS, TRUE) != SQL_SUCCESS)
+      execute_query("SET SQL_AUTO_IS_NULL = 0", SQL_NTS, true) != SQL_SUCCESS)
   {
-    /** @todo set error reason */
     goto error;
   }
 
-  ds= dsrc;
+  ds = dsrc;
   /* init all needed UTF-8 strings */
   ds_get_utf8attr(ds->name, &ds->name8);
   ds_get_utf8attr(ds->server, &ds->server8);
@@ -788,12 +922,11 @@ SQLRETURN DBC::connect(DataSource *dsrc)
   ds_get_utf8attr(ds->socket, &ds->socket8);
   if (ds->database)
   {
-
     database= ds_get_utf8attr(ds->database, &ds->database8);
   }
 
   if (ds->save_queries && !query_log)
-    query_log= init_query_log();
+    query_log = init_query_log();
 
   /* Set the statement error prefix based on the server version. */
   strxmov(st_error_prefix, MYODBC_ERROR_PREFIX, "[mysqld-",
@@ -810,11 +943,11 @@ SQLRETURN DBC::connect(DataSource *dsrc)
   {
     if (!transactions_supported() || ds->disable_transactions)
     {
-      rc= SQL_SUCCESS_WITH_INFO;
-      commit_flag= CHECK_AUTOCOMMIT_ON;
-      set_conn_error((DBC*)this, MYERR_01S02,
-                     "Transactions are not enabled, option value "
-                     "SQL_AUTOCOMMIT_OFF changed to SQL_AUTOCOMMIT_ON", 0);
+      commit_flag = CHECK_AUTOCOMMIT_ON;
+      rc = set_error(MYERR_01S02,
+             "Transactions are not enabled, option value "
+             "SQL_AUTOCOMMIT_OFF changed to SQL_AUTOCOMMIT_ON",
+             SQL_SUCCESS_WITH_INFO);
     }
     else if (autocommit_is_on() && mysql_autocommit(mysql, FALSE))
     {
@@ -850,32 +983,22 @@ SQLRETURN DBC::connect(DataSource *dsrc)
     if (transactions_supported())
     {
       sprintf(buff, "SET SESSION TRANSACTION ISOLATION LEVEL %s", level);
-      if (odbc_stmt(this, buff, SQL_NTS, TRUE) != SQL_SUCCESS)
+      if (execute_query(buff, SQL_NTS, true) != SQL_SUCCESS)
       {
-        /** @todo set error reason */
         goto error;
       }
     }
     else
     {
-      txn_isolation= SQL_TXN_READ_UNCOMMITTED;
-      rc= SQL_SUCCESS_WITH_INFO;
-      set_conn_error((DBC*)this, MYERR_01S02,
-                     "Transactions are not enabled, so transaction isolation "
-                     "was ignored.", 0);
+      txn_isolation = SQL_TXN_READ_UNCOMMITTED;
+      rc = set_error(MYERR_01S02,
+             "Transactions are not enabled, so transaction isolation "
+             "was ignored.", SQL_SUCCESS_WITH_INFO);
     }
   }
 
   mysql_get_option(mysql, MYSQL_OPT_NET_BUFFER_LENGTH, &net_buffer_len);
 
-  if (ds->no_information_schema)
-  {
-    set_error("01000", "The connection option NO_I_S is now "
-                       "deprecated and will be removed in future "
-                       "releases of MySQL Connector/ODBC", 0);
-    rc= SQL_SUCCESS_WITH_INFO;
-  }
-
   guard.set_success(rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO);
   return rc;
 
@@ -916,15 +1039,15 @@ SQLRETURN SQL_API MySQLConnect(SQLHDBC   hdbc,
 
   /* Can't connect if we're already connected. */
   if (is_connected(dbc))
-    return set_conn_error((DBC*)hdbc, MYERR_08002, NULL, 0);
+    return ((DBC*)hdbc)->set_error(MYERR_08002, NULL, 0);
 
   /* Reset error state */
   CLEAR_DBC_ERROR(dbc);
 
   if (szDSN && !szDSN[0])
   {
-    return set_conn_error((DBC*)hdbc, MYERR_S1000,
-                          "Invalid connection parameters", 0);
+    return ((DBC*)hdbc)->set_error(MYERR_S1000,
+      "Invalid connection parameters", 0);
   }
 
   ds= ds_new();
@@ -985,18 +1108,20 @@ SQLRETURN SQL_API MySQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd,
   DataSource *ds= ds_new();
   /* We may have to read driver info to find the setup library. */
   Driver *pDriver= driver_new();
-  SQLWCHAR *prompt_instr= NULL;
   /* We never know how many new parameters might come out of the prompt */
   SQLWCHAR prompt_outstr[4096];
-  size_t prompt_inlen;
   BOOL bPrompt= FALSE;
   HMODULE hModule= NULL;
+  SQLWSTRING conn_str_in, conn_str_out;
+  SQLWSTRING prompt_instr;
 
   if (cbConnStrIn != SQL_NTS)
-    szConnStrIn= sqlwchardup(szConnStrIn, cbConnStrIn);
+    conn_str_in = SQLWSTRING(szConnStrIn, cbConnStrIn);
+  else
+    conn_str_in = szConnStrIn;
 
   /* Parse the incoming string */
-  if (ds_from_kvpair(ds, szConnStrIn, (SQLWCHAR)';'))
+  if (ds_from_kvpair(ds, conn_str_in.c_str(), (SQLWCHAR)';'))
   {
     rc= dbc->set_error( "HY000",
                       "Failed to parse the incoming connect string.", 0);
@@ -1020,7 +1145,7 @@ SQLRETURN SQL_API MySQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd,
       1 - we want the connection string options to override DSN options
       2 - no need to check for parsing erros as it was done before
     */
-    ds_from_kvpair(ds, szConnStrIn, (SQLWCHAR)';');
+    ds_from_kvpair(ds, conn_str_in.c_str(), (SQLWCHAR)';');
   }
 #endif
 
@@ -1173,25 +1298,21 @@ SQLRETURN SQL_API MySQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd,
     }
 
     /* create a string for prompting, and add driver manually */
-    prompt_inlen= ds_to_kvpair_len(ds) + sqlwcharlen(W_DRIVER_PARAM) +
-                  sqlwcharlen(ds->driver) + 1;
-    prompt_instr= (SQLWCHAR *) myodbc_malloc(prompt_inlen * sizeof(SQLWCHAR),
-                                         MYF(0));
-    if (ds_to_kvpair(ds, prompt_instr, prompt_inlen, ';') == -1)
+    if (ds_to_kvpair(ds, prompt_instr, ';') == -1)
     {
       rc= dbc->set_error( "HY000",
                         "Failed to prepare prompt input.", 0);
       goto error;
     }
-    prompt_inlen-= sqlwcharlen(prompt_instr);
-    sqlwcharncat2(prompt_instr, W_DRIVER_PARAM, &prompt_inlen);
-    sqlwcharncat2(prompt_instr, ds->driver, &prompt_inlen);
+
+    prompt_instr.append(W_DRIVER_PARAM);
+    prompt_instr.append(ds->driver);
 
     /*
       In case the client app did not provide the out string we use our
       inner buffer prompt_outstr
     */
-    if (!pFunc(hwnd, prompt_instr, fDriverCompletion,
+    if (!pFunc(hwnd, (SQLWCHAR*)prompt_instr.c_str(), fDriverCompletion,
                prompt_outstr, sizeof(prompt_outstr), pcbConnStrOut))
     {
       dbc->set_error("HY000", "User cancelled.", 0);
@@ -1238,11 +1359,10 @@ SQLRETURN SQL_API MySQLDriverConnect(SQLHDBC hdbc, SQLHWND hwnd,
 connected:
 
   /* copy input to output if connected without prompting */
-  if (!bPrompt && szConnStrOut && cbConnStrOutMax)
+  if (!bPrompt)
   {
-    size_t inlen, copylen;
-    SQLWCHAR *internal_out_str= szConnStrIn;
-
+    size_t copylen;
+    conn_str_out = conn_str_in;
     if (ds->savefile)
     {
       SQLWCHAR *pwd_temp= ds->pwd;
@@ -1252,26 +1372,38 @@ connected:
       ds->pwd= NULL;
       ds->pwd8= NULL;
 
-      ds_to_kvpair(ds, prompt_outstr, sizeof(prompt_outstr)/sizeof(SQLWCHAR), ';');
-      internal_out_str= (SQLWCHAR*)prompt_outstr;
+      ds_to_kvpair(ds, conn_str_out, ';');
 
       /* restore old values */
       ds->pwd= pwd_temp;
       ds->pwd8= pwd8_temp;
     }
 
-    inlen= (sqlwcharlen(internal_out_str) + 1) * sizeof(SQLWCHAR);
-    copylen= myodbc_min((size_t)cbConnStrOutMax, inlen);
-    memcpy(szConnStrOut, internal_out_str, copylen);
-    /* term needed if possibly truncated */
-    szConnStrOut[(copylen / sizeof(SQLWCHAR)) - 1] = 0;
+    size_t inlen = conn_str_out.length();
+    copylen = myodbc_min((size_t)cbConnStrOutMax, inlen + 1) * sizeof(SQLWCHAR);
+
+    if (szConnStrOut && copylen)
+    {
+      memcpy(szConnStrOut, conn_str_out.c_str(), copylen);
+      /* term needed if possibly truncated */
+      szConnStrOut[(copylen / sizeof(SQLWCHAR)) - 1] = 0;
+    }
+
+    /*
+      Even if the buffer for out connection string is NULL
+      it will still return the total number of characters
+      (excluding the null-termination character for character data)
+      available to return in the buffer pointed to by
+      OutConnectionString.
+      https://learn.microsoft.com/en-us/sql/odbc/reference/syntax/sqldriverconnect-function?view=sql-server-ver16
+    */
     if (pcbConnStrOut)
-      *pcbConnStrOut= (copylen / sizeof(SQLWCHAR)) - 1;
+      *pcbConnStrOut = inlen;
   }
 
   /* return SQL_SUCCESS_WITH_INFO if truncated output string */
-  if (pcbConnStrOut &&
-      cbConnStrOutMax - sizeof(SQLWCHAR) == *pcbConnStrOut * sizeof(SQLWCHAR))
+  if (pcbConnStrOut && cbConnStrOutMax &&
+      cbConnStrOutMax - sizeof(SQLWCHAR) <= *pcbConnStrOut * sizeof(SQLWCHAR))
   {
     dbc->set_error("01004", "String data, right truncated.", 0);
     rc= SQL_SUCCESS_WITH_INFO;
@@ -1280,14 +1412,11 @@ connected:
 error:
   if (hModule)
     FreeLibrary(hModule);
-  if (cbConnStrIn != SQL_NTS)
-    x_free(szConnStrIn);
 
   driver_delete(pDriver);
   /* delete data source unless connected */
   if (!dbc->ds)
     ds_delete(ds);
-  x_free(prompt_instr);
 
   return rc;
 }
@@ -1365,3 +1494,30 @@ void DBC::execute_prep_stmt(MYSQL_STMT *pstmt, std::string &query,
   }
 
 }
+
+SQLRETURN DBC::execute_query(const char* query,
+  SQLULEN query_length, my_bool req_lock)
+{
+  SQLRETURN result = SQL_SUCCESS;
+  LOCK_DBC_DEFER(this);
+
+  if (req_lock)
+  {
+    DO_LOCK_DBC();
+  }
+
+  if (query_length == SQL_NTS)
+  {
+    query_length = strlen(query);
+  }
+
+  if (check_if_server_is_alive(this) ||
+    mysql_real_query(mysql, query, query_length))
+  {
+    result = set_error(MYERR_S1000, mysql_error(mysql),
+      mysql_errno(mysql));
+  }
+
+  return result;
+
+}
