@@ -1,4 +1,4 @@
-// Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -105,7 +105,7 @@ void myodbc_init(void)
 
     __LOCALE_RESTORE()
 
-    utf8_charset_info= get_charset_by_csname("utf8", MYF(MY_CS_PRIMARY),
+    utf8_charset_info= get_charset_by_csname(transport_charset, MYF(MY_CS_PRIMARY),
                                              MYF(0));
 
 #ifdef IS_BIG_ENDIAN
@@ -195,7 +195,12 @@ int APIENTRY LibMain(HANDLE inst, DWORD ul_reason_being_called,
     break;
   case DLL_PROCESS_DETACH:  /* case of wep call in win 3.x */
     if (!--inited)
+    {
+      // Process is about to detach. All has to be deinited to avoid
+      // memory leaks even if initialized multiple times (myodbc_inited > 1).
+      myodbc_inited = 1;
       myodbc_end();
+    }
     break;
 
   /*
