@@ -1,4 +1,4 @@
-// Copyright (c) 2001, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2001, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -70,16 +70,24 @@
 #include <list>
 #include <mutex>
 
-#define LOCK_STMT(S) std::unique_lock<std::recursive_mutex> slock(S->dbc->lock)
-#define LOCK_STMT_DEFER(S) std::unique_lock<std::recursive_mutex> slock(S->dbc->lock, std::defer_lock)
+#define LOCK_STMT(S) CHECK_HANDLE(S); \
+  std::unique_lock<std::recursive_mutex> slock(((STMT*)S)->lock)
+#define LOCK_STMT_DEFER(S) CHECK_HANDLE(S); \
+  std::unique_lock<std::recursive_mutex> slock(((STMT*)S)->lock, std::defer_lock)
 #define DO_LOCK_STMT() slock.lock();
 
-#define LOCK_DBC(D) std::unique_lock<std::recursive_mutex> dlock(D->lock)
-#define LOCK_DBC_DEFER(D) std::unique_lock<std::recursive_mutex> dlock(D->lock, std::defer_lock)
+#define LOCK_DBC(D) std::unique_lock<std::recursive_mutex> dlock(((DBC*)D)->lock)
+#define LOCK_DBC_DEFER(D) std::unique_lock<std::recursive_mutex> dlock(((DBC*)D)->lock, std::defer_lock)
 #define DO_LOCK_DBC() dlock.lock();
 
 #define LOCK_ENV(E) std::unique_lock<std::mutex> elock(E->lock)
 
+// SQL_DRIVER_CONNECT_ATTR_BASE is not defined in all driver managers.
+// Therefore use a custom constant until it becomes a standard.
+#define MYSQL_DRIVER_CONNECT_ATTR_BASE 0x00004000
+
+#define CB_FIDO_GLOBAL MYSQL_DRIVER_CONNECT_ATTR_BASE + 0x00001000
+#define CB_FIDO_CONNECTION MYSQL_DRIVER_CONNECT_ATTR_BASE + 0x00001001
 
 #if defined(_WIN32) || defined(WIN32)
 # define INTFUNC  __stdcall
@@ -118,8 +126,13 @@
 /*
    Internal driver definitions
 */
-#define MYSQL_RESET_BUFFERS 1000  /* param to SQLFreeStmt */
-#define MYSQL_RESET 1001	  /* param to SQLFreeStmt */
+
+/* Options for SQLFreeStmt */
+#define FREE_STMT_RESET_BUFFERS 1000
+#define FREE_STMT_RESET 1001
+#define FREE_STMT_CLEAR_RESULT 1
+#define FREE_STMT_DO_LOCK 2
+
 #define MYSQL_3_21_PROTOCOL 10	  /* OLD protocol */
 #define CHECK_IF_ALIVE	    1800  /* Seconds between queries for ping */
 
@@ -165,6 +178,10 @@ typedef enum { DESC_PARAM, DESC_ROW, DESC_UNKNOWN } desc_desc_type;
 /* header or record field? (location in descriptor) */
 typedef enum { DESC_HDR, DESC_REC } fld_loc;
 
+typedef void (*fido_callback_func)(const char*);
+extern fido_callback_func global_fido_callback;
+extern std::mutex global_fido_mutex;
+
 /* permissions - header, and base for record */
 #define P_RI 1 /* imp */
 #define P_WI 2
@@ -445,9 +462,9 @@ struct DBC;
 
 struct DESC {
   /* header fields */
-  SQLSMALLINT   alloc_type;
-  SQLULEN       array_size;
-  SQLUSMALLINT *array_status_ptr;
+  SQLSMALLINT   alloc_type = 0;
+  SQLULEN       array_size = 0;
+  SQLUSMALLINT *array_status_ptr = nullptr;
   /* NOTE: This field is defined as SQLINTEGER* in the descriptor
    * documentation, but corresponds to SQL_ATTR_ROW_BIND_OFFSET_PTR or
    * SQL_ATTR_PARAM_BIND_OFFSET_PTR when set via SQLSetStmtAttr(). The
@@ -455,23 +472,23 @@ struct DESC {
    * is now a 64-bit value. These two are conflicting, so we opt for
    * the 64-bit value.
    */
-  SQLULEN      *bind_offset_ptr;
-  SQLINTEGER    bind_type;
-  SQLLEN        count;  // Only used for SQLGetDescField()
-  SQLLEN        bookmark_count;
+  SQLULEN      *bind_offset_ptr = nullptr;
+  SQLINTEGER    bind_type = 0;
+  SQLLEN        count = 0;  // Only used for SQLGetDescField()
+  SQLLEN        bookmark_count = 0;
   /* Everywhere(http://msdn.microsoft.com/en-us/library/ms713560(VS.85).aspx
      http://msdn.microsoft.com/en-us/library/ms712631(VS.85).aspx) I found
      it's referred as SQLULEN* */
-  SQLULEN      *rows_processed_ptr;
+  SQLULEN      *rows_processed_ptr = nullptr;
 
   /* internal fields */
-  desc_desc_type  desc_type;
-  desc_ref_type   ref_type;
+  desc_desc_type  desc_type = DESC_PARAM;
+  desc_ref_type   ref_type = DESC_IMP;
 
   std::vector<DESCREC> bookmark2;
   std::vector<DESCREC> records2;
 
-  MYERROR         error;
+  MYERROR error;
   STMT *stmt;
   DBC *dbc;
 
@@ -557,11 +574,10 @@ struct	ENV
 {
   SQLINTEGER   odbc_ver;
   std::list<DBC*> conn_list;
-  LIST	       *connections;
   MYERROR      error;
   std::mutex lock;
 
-  ENV(SQLINTEGER ver) : odbc_ver(ver), connections(nullptr)
+  ENV(SQLINTEGER ver) : odbc_ver(ver)
   {}
 
   void add_dbc(DBC* dbc);
@@ -596,14 +612,21 @@ struct DBC
   bool          has_query_attrs = false;
   std::recursive_mutex lock;
 
-  bool          unicode = false;            /* Whether SQL*ConnectW was used */
-  CHARSET_INFO  *ansi_charset_info = nullptr, /* 'ANSI' charset (SQL_C_CHAR) */
-                *cxn_charset_info = nullptr;  /* Connection charset ('ANSI' or utf-8) */
+  // Whether SQL*ConnectW was used
+  bool          unicode = false;
+  // 'ANSI' charset (SQL_C_CHAR)
+  CHARSET_INFO  *ansi_charset_info = nullptr,
+  // Connection charset ('ANSI' or utf-8)
+                *cxn_charset_info = nullptr;
   MY_SYNTAX_MARKERS *syntax = nullptr;
-  DataSource    *ds = nullptr;                /* data source used to connect (parsed or stored) */
-  SQLULEN       sql_select_limit = -1;/* value of the sql_select_limit currently set for a session
-                                       (SQLULEN)(-1) if wasn't set */
-  int           need_to_wakeup = 0;      /* Connection have been put to the pool */
+  // data source used to connect (parsed or stored)
+  DataSource    *ds = nullptr;
+  // value of the sql_select_limit currently set for a session
+  //   (SQLULEN)(-1) if wasn't set
+  SQLULEN       sql_select_limit = -1;
+  // Connection have been put to the pool
+  int           need_to_wakeup = 0;
+  fido_callback_func fido_callback = nullptr;
 
   DBC(ENV *p_env);
   void free_explicit_descriptors();
@@ -625,6 +648,12 @@ struct DBC
   void close();
   ~DBC();
 
+  void set_charset(std::string charset);
+  SQLRETURN set_charset_options(const char* charset);
+  SQLRETURN set_error(myodbc_errid errid, const char* errtext,
+    SQLINTEGER errcode);
+  SQLRETURN execute_query(const char *query,
+    SQLULEN query_length, my_bool req_lock);
 };
 
 
@@ -887,7 +916,11 @@ struct ROW_STORAGE
       *(bind[i].is_null) = data.is_null();
       *(bind[i].length) = data.is_null() ? -1 : data.length();
       if (!data.is_null())
-        memcpy(bind[i].buffer, data.data(), *(bind[i].length));                                                                                                                                                              }
+      {
+        size_t copy_zero = bind[i].buffer_length > *(bind[i].length) ? 1 : 0;
+        memcpy(bind[i].buffer, data.data(), *(bind[i].length) + copy_zero);
+      }
+    }
     // Set EOF if the last row was filled
     m_eof = (m_rnum <= (m_cur_row + 1));
     // Increment row counter only if not EOF
@@ -911,10 +944,15 @@ enum class EXCEPTION_TYPE {EMPTY_SET, CONN_ERR, GENERAL};
 struct ODBCEXCEPTION
 {
   EXCEPTION_TYPE m_type = EXCEPTION_TYPE::GENERAL;
+  std::string m_msg;
 
   ODBCEXCEPTION(EXCEPTION_TYPE t = EXCEPTION_TYPE::GENERAL) : m_type(t)
   {}
 
+  ODBCEXCEPTION(std::string msg, EXCEPTION_TYPE t = EXCEPTION_TYPE::GENERAL) :
+    m_type(t), m_msg(msg)
+  {}
+
 };
 
 struct ODBC_STMT
@@ -947,7 +985,7 @@ struct STMT
   std::string       table_name;
 
   MY_PARSED_QUERY	query, orig_query;
-  DYNAMIC_ARRAY     *param_bind;
+  std::vector<MYSQL_BIND> param_bind;
   std::vector<MYSQL_BIND> query_attr_bind;
   std::vector<char*>      query_attr_names;
 
@@ -991,6 +1029,8 @@ struct STMT
   DESC *imp_ard;
   DESC *imp_apd;
 
+  std::recursive_mutex lock;
+
   int ssps_bind_result();
 
   char* extend_buffer(char *to, size_t len);
@@ -1039,8 +1079,7 @@ struct STMT
   STMT(DBC *d) : dbc(d), result(NULL), array(NULL), result_array(NULL),
     current_values(NULL), fields(NULL), end_of_set(NULL),
     tempbuf(),
-    stmt_options(dbc->stmt_options),
-    param_bind(NULL), lengths(nullptr), affected_rows(0),
+    stmt_options(dbc->stmt_options), lengths(nullptr), affected_rows(0),
     current_row(0), cursor_row(0), dae_type(0),
     order(NULL), order_count(0), param_count(0), current_param(0),
     rows_found_in_set(0),
@@ -1071,6 +1110,99 @@ struct STMT
   ~STMT();
 };
 
+namespace myodbc {
+  struct HENV
+  {
+    SQLHENV henv = nullptr;
+
+    HENV(unsigned long v)
+    {
+      size_t ver = v;
+      SQLAllocHandle(SQL_HANDLE_ENV, nullptr, &henv);
+
+      if (SQLSetEnvAttr(henv, SQL_ATTR_ODBC_VERSION, (SQLPOINTER)ver, 0) != SQL_SUCCESS)
+      {
+        throw MYERROR(SQL_HANDLE_ENV, henv, SQL_ERROR);
+      }
+    }
+
+    HENV() : HENV(SQL_OV_ODBC3)
+    {}
+
+    operator SQLHENV() const
+    {
+      return henv;
+    }
+
+    ~HENV()
+    {
+      SQLFreeHandle(SQL_HANDLE_ENV, henv);
+    }
+  };
+
+  struct HDBC
+  {
+    SQLHDBC hdbc = nullptr;
+    SQLHENV henv;
+    xstring connout = nullptr;
+    SQLCHAR ch_out[512] = { 0 };
+
+    HDBC(SQLHENV env, DataSource *params) : henv(env)
+    {
+      SQLWSTRING  string_connect_in;
+
+      assert(params->driver && *params->driver);
+      ds_set_strattr(&params->name, NULL);
+      ds_to_kvpair(params, string_connect_in, ';');
+      if (SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc) != SQL_SUCCESS)
+      {
+        throw MYERROR(SQL_HANDLE_ENV, henv, SQL_ERROR);
+      }
+
+      if (SQLDriverConnectW(hdbc, NULL, (SQLWCHAR*)string_connect_in.c_str(),
+        SQL_NTS, NULL, 0, NULL, SQL_DRIVER_NOPROMPT) != SQL_SUCCESS)
+      {
+        throw MYERROR(SQL_HANDLE_DBC, hdbc, SQL_ERROR);
+      }
+
+    }
+
+    operator SQLHDBC() const
+    {
+      return hdbc;
+    }
+
+    ~HDBC()
+    {
+      SQLDisconnect(hdbc);
+      SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
+    }
+  };
+
+
+  struct HSTMT
+  {
+    SQLHDBC hdbc;
+    SQLHSTMT hstmt = nullptr;
+    HSTMT(SQLHDBC dbc) : hdbc(dbc)
+    {
+      if (SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt) != SQL_SUCCESS)
+      {
+        throw MYERROR(SQL_HANDLE_STMT, hstmt, SQL_ERROR);
+      }
+    }
+
+    operator SQLHSTMT() const
+    {
+      return hstmt;
+    }
+
+    ~HSTMT()
+    {
+      SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
+    }
+  };
+};
 
 extern char *default_locale, *decimal_point, *thousands_sep;
 extern uint decimal_point_length,thousands_sep_length;
