@@ -1,4 +1,4 @@
-// Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -43,6 +43,7 @@
 SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
 {
     int error= SQL_ERROR, native_error= 0;
+    assert(stmt);
     LOCK_STMT_DEFER(stmt);
 
     if (!query)
@@ -55,8 +56,8 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
                       stmt->stmt_options.max_rows, TRUE)))
     {
       /* The error is set for DBC, copy it into STMT */
-      stmt->set_error(stmt->dbc->error.sqlstate,
-                     stmt->dbc->error.message,
+      stmt->set_error(stmt->dbc->error.sqlstate.c_str(),
+                     stmt->dbc->error.message.c_str(),
                      stmt->dbc->error.native_error);
 
       /* if setting sql_select_limit fails, the query will probably fail anyway too */
@@ -70,14 +71,13 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
 
     MYLOG_QUERY(stmt, query);
     DO_LOCK_STMT();
-
     if ( check_if_server_is_alive( stmt->dbc ) )
     {
       stmt->set_error("08S01" /* "HYT00" */,
                       mysql_error(stmt->dbc->mysql),
                       mysql_errno(stmt->dbc->mysql));
 
-      translate_error(stmt->error.sqlstate, MYERR_08S01 /* S1000 */,
+      translate_error((char*)stmt->error.sqlstate.c_str(), MYERR_08S01 /* S1000 */,
                       mysql_errno(stmt->dbc->mysql));
       goto exit;
     }
@@ -115,8 +115,10 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
        this is a batch of queries */
     else if (ssps_used(stmt))
     {
-      native_error= mysql_stmt_bind_param(stmt->ssps,
-                                        (MYSQL_BIND*)stmt->param_bind->buffer);
+      native_error = 0;
+      if (stmt->param_bind.size() && stmt->param_count)
+        native_error = mysql_stmt_bind_param(stmt->ssps, &stmt->param_bind[0]);
+
       if (native_error == 0)
       {
         native_error= mysql_stmt_execute(stmt->ssps);
@@ -128,7 +130,7 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
                        mysql_stmt_errno(stmt->ssps));
 
         /* For some errors - translating to more appropriate status */
-        translate_error(stmt->error.sqlstate, MYERR_S1000,
+        translate_error((char*)stmt->error.sqlstate.c_str(), MYERR_S1000,
                         mysql_stmt_errno(stmt->ssps));
         goto exit;
       }
@@ -159,7 +161,7 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
       stmt->set_error("HY000");
 
       /* For some errors - translating to more appropriate status */
-      translate_error(stmt->error.sqlstate, MYERR_S1000,
+      translate_error((char*)stmt->error.sqlstate.c_str(), MYERR_S1000,
                       mysql_errno(stmt->dbc->mysql));
       goto exit;
     }
@@ -243,24 +245,15 @@ exit:
 SQLRETURN insert_params(STMT *stmt, SQLULEN row, char **finalquery,
                         SQLULEN *finalquery_length)
 {
+  assert(stmt);
   char *query= GET_QUERY(&stmt->query);
   uint i,length, had_info= 0;
   SQLRETURN rc= SQL_SUCCESS;
   DECLARE_LOCALE_HANDLE
 
-  LOCK_STMT(stmt);
+  LOCK_DBC(stmt->dbc);
 
-  //to = stmt->buf() + (finalquery_length != NULL ? *finalquery_length : 0);
-
-  if (!stmt->dbc->ds->dont_use_set_locale)
-  {
-    __LOCALE_SET("C"); /* force use of '.' as decimal point */
-  }
-
-  if (adjust_param_bind_array(stmt) )
-  {
-    goto memerror;
-  }
+  adjust_param_bind_array(stmt);
 
   for ( i= 0; i < stmt->param_count; ++i )
   {
@@ -349,21 +342,12 @@ SQLRETURN insert_params(STMT *stmt, SQLULEN row, char **finalquery,
     }
   }
 
-  if (!stmt->dbc->ds->dont_use_set_locale)
-  {
-    __LOCALE_RESTORE();
-  }
-
-
-
   return rc;
 
 memerror:      /* Too much data */
   rc= stmt->set_error(MYERR_S1001,NULL,4001);
 
 error:
-  if (!stmt->dbc->ds->dont_use_set_locale)
-    __LOCALE_RESTORE();
   return rc;
 }
 
@@ -594,6 +578,7 @@ SQLRETURN convert_c_type2str(STMT *stmt, SQLSMALLINT ctype, DESCREC *iprec,
         /* We should perpare this data for string comparison */
         sprintf(buff, "%.15e", *((float*) *res));
       }
+      delocalize_radix(buff);
       *length= strlen(*res= buff);
       break;
     case SQL_C_DOUBLE:
@@ -606,6 +591,7 @@ SQLRETURN convert_c_type2str(STMT *stmt, SQLSMALLINT ctype, DESCREC *iprec,
         /* We should perpare this data for string comparison */
         sprintf(buff,"%.15e",*((double*) *res));
       }
+      delocalize_radix(buff);
       *length= strlen(*res= buff);
       break;
     case SQL_C_DATE:
@@ -1032,7 +1018,7 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
             }
             else
             {
-              stmt->add_to_buffer("_utf8", 5);
+              stmt->add_to_buffer("_utf8mb4", 8);
             }
           }
           else if (aprec->concise_type != SQL_C_WCHAR &&
@@ -1244,7 +1230,8 @@ SQLRETURN insert_param(STMT *stmt, MYSQL_BIND *bind, DESC* apd,
           goto memerror;
         }
 
-        copy_binhex_result(stmt, (SQLCHAR*)stmt->endbuf(), length * 2 + 1, &transformed_len, 0, data, length);
+        copy_binhex_result(stmt, (SQLCHAR*)stmt->endbuf(),
+          length * 2 + 1, &transformed_len, data, length);
         stmt->buf_add_pos(transformed_len);
       }
       else
@@ -1330,7 +1317,7 @@ SQLRETURN do_my_pos_cursor_std( STMT *pStmt, STMT *pStmtCursor )
 
 SQLRETURN SQL_API SQLExecute(SQLHSTMT hstmt)
 {
-  CHECK_HANDLE(hstmt);
+  LOCK_STMT(hstmt);
 
   return my_SQLExecute((STMT *)hstmt);
 }
@@ -1418,7 +1405,7 @@ SQLRETURN my_SQLExecute( STMT *pStmt )
     return do_my_pos_cursor_std(pStmt, pStmtCursor);
   }
 
-  my_SQLFreeStmt((SQLHSTMT)pStmt,MYSQL_RESET_BUFFERS);
+  my_SQLFreeStmt((SQLHSTMT)pStmt,FREE_STMT_RESET_BUFFERS);
 
   query= GET_QUERY(&pStmt->query);
 
@@ -1437,7 +1424,7 @@ SQLRETURN my_SQLExecute( STMT *pStmt )
     *pStmt->ipd->rows_processed_ptr= (SQLULEN)0;
   }
 
-  LOCK_STMT(pStmt);
+  LOCK_DBC(pStmt->dbc);
 
   for (row= 0; row < pStmt->apd->array_size; ++row)
   {
@@ -1906,11 +1893,9 @@ SQLRETURN SQL_API SQLCancel(SQLHSTMT hstmt)
 {
   MYSQL *second= NULL;
   DBC *dbc;
+  STMT *stmt = (STMT *)hstmt;
 
-
-  CHECK_HANDLE(hstmt);
-
-  dbc= ((STMT *)hstmt)->dbc;
+  dbc= stmt->dbc;
 
   LOCK_DBC_DEFER(dbc); // implicitly declares dlock variable without locking
 
@@ -1918,10 +1903,12 @@ SQLRETURN SQL_API SQLCancel(SQLHSTMT hstmt)
   if (dlock.try_lock())
   {
     /*
-      Lock is acquired. STMT can be closed safely.
-      Lock will be released automatically
+      DBC lock can be released.
+      STMT lock will be acquired inside my_SQLFreeStmtExtended.
     */
-    return my_SQLFreeStmt(hstmt, SQL_CLOSE);
+    dlock.unlock();
+    return my_SQLFreeStmtExtended(hstmt, SQL_CLOSE,
+      FREE_STMT_CLEAR_RESULT | FREE_STMT_DO_LOCK);
   }
 
   /*
