@@ -1,4 +1,4 @@
-// Copyright (c) 2001, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2001, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -122,8 +122,8 @@ DBC::~DBC()
 
 SQLRETURN DBC::set_error(char * state, const char * message, uint errcode)
 {
-  myodbc_stpmov(error.sqlstate, state);
-  strxmov(error.message, MYODBC_ERROR_PREFIX, message, NullS);
+  error.sqlstate = state ? state : "";
+  error.message = std::string(MYODBC_ERROR_PREFIX) + message;
   error.native_error= errcode;
   return SQL_ERROR;
 }
@@ -135,6 +135,14 @@ SQLRETURN DBC::set_error(char * state)
 }
 
 
+SQLRETURN DBC::set_error(myodbc_errid errid, const char* errtext,
+  SQLINTEGER errcode)
+{
+  error = MYERROR(errid, errtext, errcode, MYODBC_ERROR_PREFIX);
+  return error.retcode;
+}
+
+
 /*
   @type    : myodbc3 internal
   @purpose : to allocate the environment handle and to maintain
@@ -373,69 +381,33 @@ SQLRETURN SQL_API SQLFreeConnect(SQLHDBC hdbc)
 }
 
 
-/* allocates dynamic array for param bind.
-   Throws on allocation errors */
+/* Allocates memory for parameter binds in vector.
+ */
 void STMT::allocate_param_bind(uint elements)
 {
   if (dbc->ds->no_ssps)
     return;
 
-  if (param_bind == NULL)
+  if (param_bind.capacity() < elements)
   {
-    param_bind= (DYNAMIC_ARRAY*)myodbc_malloc(sizeof(DYNAMIC_ARRAY), MYF(0));
-
-    if (param_bind == NULL)
-    {
-      throw;
-    }
+    param_bind.reserve(elements);
+    while(elements > param_bind.size())
+      param_bind.push_back(MYSQL_BIND{});
   }
-
-  myodbc_init_dynamic_array(param_bind, sizeof(MYSQL_BIND), elements, 10);
-  memset(param_bind->buffer, 0, sizeof(MYSQL_BIND) *
-         param_bind->max_element);
 }
 
 
 int adjust_param_bind_array(STMT *stmt)
 {
-  if (ssps_used(stmt) && stmt->param_count > stmt->param_bind->max_element)
-  {
-    uint prev_max_elements= stmt->param_bind->max_element;
-
-    if (myodbc_allocate_dynamic(stmt->param_bind, stmt->param_count))
-    {
-      return 1;
-    }
-
-    /* Need to init newly allocated area with 0s */
-    memset(stmt->param_bind->buffer + sizeof(MYSQL_BIND)*prev_max_elements, 0,
-      sizeof(MYSQL_BIND) * (stmt->param_bind->max_element - prev_max_elements));
-  }
+  stmt->allocate_param_bind(stmt->param_count);
+  /* Need to init newly allocated area with 0s */
+  //  memset(stmt->param_bind->buffer + sizeof(MYSQL_BIND)*prev_max_elements, 0,
+  //    sizeof(MYSQL_BIND) * (stmt->param_bind->max_element - prev_max_elements));
 
   return 0;
 }
 
 
-void delete_param_bind(DYNAMIC_ARRAY *param_bind)
-{
-  if (param_bind != NULL)
-  {
-    uint i;
-    for (i=0; i < param_bind->max_element; ++i)
-    {
-      MYSQL_BIND *bind= (MYSQL_BIND *)param_bind->buffer + i;
-
-      if (bind != NULL)
-      {
-        x_free(bind->buffer);
-      }
-    }
-    delete_dynamic(param_bind);
-    x_free(param_bind);
-  }
-}
-
-
 /*
   @type    : myodbc3 internal
   @purpose : allocates the statement handle
@@ -488,11 +460,10 @@ SQLRETURN SQL_API SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT *phstmt)
        resources associated with the statement handle
 */
 
-SQLRETURN SQL_API SQLFreeStmt(SQLHSTMT hstmt,SQLUSMALLINT fOption)
+SQLRETURN SQL_API SQLFreeStmt(SQLHSTMT hstmt, SQLUSMALLINT fOption)
 {
-    CHECK_HANDLE(hstmt);
-
-    return my_SQLFreeStmt(hstmt,fOption);
+  CHECK_HANDLE(hstmt);
+  return my_SQLFreeStmt(hstmt, fOption);
 }
 
 
@@ -504,9 +475,10 @@ SQLRETURN SQL_API SQLFreeStmt(SQLHSTMT hstmt,SQLUSMALLINT fOption)
        resources associated with the statement handle
 */
 
-SQLRETURN SQL_API my_SQLFreeStmt(SQLHSTMT hstmt,SQLUSMALLINT fOption)
+SQLRETURN SQL_API my_SQLFreeStmt(SQLHSTMT hstmt, SQLUSMALLINT f_option)
 {
-  return my_SQLFreeStmtExtended(hstmt,fOption,1);
+  return my_SQLFreeStmtExtended(hstmt, f_option,
+    FREE_STMT_CLEAR_RESULT | FREE_STMT_DO_LOCK);
 }
 
 /*
@@ -517,15 +489,20 @@ SQLRETURN SQL_API my_SQLFreeStmt(SQLHSTMT hstmt,SQLUSMALLINT fOption)
        resources associated with the statement handle
 */
 
-SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
-                                         uint clearAllResults)
+SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt, SQLUSMALLINT f_option,
+  SQLUSMALLINT f_extra)
 {
     STMT *stmt= (STMT *) hstmt;
     uint i;
+    LOCK_STMT_DEFER(stmt);
+    if (f_extra & FREE_STMT_DO_LOCK)
+    {
+      DO_LOCK_STMT();
+    }
 
     stmt->reset();
 
-    if (fOption == SQL_UNBIND)
+    if (f_option == SQL_UNBIND)
     {
       stmt->free_unbind();
       return SQL_SUCCESS;
@@ -533,13 +510,13 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
 
     stmt->free_reset_out_params();
 
-    if (fOption == SQL_RESET_PARAMS)
+    if (f_option == SQL_RESET_PARAMS)
     {
       stmt->free_reset_params();
       return SQL_SUCCESS;
     }
 
-    stmt->free_fake_result(clearAllResults);
+    stmt->free_fake_result((bool)(f_extra & FREE_STMT_CLEAR_RESULT));
 
     x_free(stmt->fields);   // TODO: Looks like STMT::fields is not used anywhere
     x_free(stmt->result_array);
@@ -556,7 +533,7 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
     stmt->dae_type= 0;
     stmt->ird->reset();
 
-    if (fOption == MYSQL_RESET_BUFFERS)
+    if (f_option == FREE_STMT_RESET_BUFFERS)
     {
       free_result_bind(stmt);
       x_free(stmt->array);
@@ -578,10 +555,10 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
     }
     stmt->cursor.pk_count= 0;
 
-    if (fOption == SQL_CLOSE)
+    if (f_option == SQL_CLOSE)
         return SQL_SUCCESS;
 
-    if (clearAllResults)
+    if (f_extra & FREE_STMT_CLEAR_RESULT)
     {
       x_free(stmt->array);
       stmt->array= 0;
@@ -592,15 +569,10 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
       }
     }
 
-    /* At this point, only MYSQL_RESET and SQL_DROP left out */
+    /* At this point, only FREE_STMT_RESET and SQL_DROP left out */
     reset_parsed_query(&stmt->orig_query, NULL, NULL, NULL);
     reset_parsed_query(&stmt->query, NULL, NULL, NULL);
 
-    if (stmt->param_bind != NULL)
-    {
-      reset_dynamic(stmt->param_bind);
-    }
-
     stmt->param_count= 0;
 
     reset_ptr(stmt->apd->rows_processed_ptr);
@@ -611,7 +583,7 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
     reset_ptr(stmt->ard->array_status_ptr);
     reset_ptr(stmt->stmt_options.rowStatusPtr_ex);
 
-    if (fOption == MYSQL_RESET)
+    if (f_option == FREE_STMT_RESET)
     {
       return SQL_SUCCESS;
     }
@@ -620,6 +592,13 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
     stmt->apd->stmt_list_remove(stmt);
     stmt->ard->stmt_list_remove(stmt);
 
+    // Unlock before destroying STMT to prevent unowned mutex.
+    // See inside ~STMT()
+    if (slock)
+    {
+      slock.unlock();
+    }
+
     delete stmt;
     return SQL_SUCCESS;
 }
@@ -720,7 +699,7 @@ SQLRETURN SQL_API SQLAllocHandle(SQLSMALLINT HandleType,
             break;
 
         default:
-            return set_conn_error((DBC*)InputHandle,MYERR_S1C00,NULL,0);
+            return ((DBC*)InputHandle)->set_error(MYERR_S1C00, NULL, 0);
     }
 
     return error;
