@@ -1,4 +1,4 @@
-// Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -911,7 +911,7 @@ MySQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
   {
     char buff[80];
     sprintf(buff, "Unsupported option: %d to SQLGetInfo", fInfoType);
-    return set_conn_error((DBC*)hdbc, MYERR_S1C00, buff, 4000);
+    return ((DBC*)hdbc)->set_error(MYERR_S1C00, buff, 4000);
   }
   }
 
@@ -1077,7 +1077,7 @@ SQLRETURN SQL_API MySQLGetTypeInfo(SQLHSTMT hstmt, SQLSMALLINT fSqlType)
   STMT *stmt = (STMT *)hstmt;
   uint i;
 
-  my_SQLFreeStmt(hstmt, MYSQL_RESET);
+  my_SQLFreeStmt(hstmt, FREE_STMT_RESET);
 
   /* use ODBC2 types if called with ODBC3 types on an ODBC2 handle */
   if (stmt->dbc->env->odbc_ver == SQL_OV_ODBC2)
