@@ -1,4 +1,4 @@
-// Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2012, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -317,11 +317,24 @@ int get_int(STMT *stmt, ulong column_number, char *value, ulong length)
 {
   if (ssps_used(stmt))
   {
-     return (int)ssps_get_int64(stmt, column_number, value, length);
+     return (int)ssps_get_int64<long long>(stmt, column_number, value, length);
   }
   else
   {
-    return atoi(value);
+    return (int)strtol(value, NULL, 10);
+  }
+}
+
+
+unsigned int get_uint(STMT* stmt, ulong column_number, char* value, ulong length)
+{
+  if (ssps_used(stmt))
+  {
+    return (int)ssps_get_int64<unsigned long long>(stmt, column_number, value, length);
+  }
+  else
+  {
+    return (unsigned int)strtoul(value, NULL, 10);
   }
 }
 
@@ -330,7 +343,7 @@ long long get_int64(STMT *stmt, ulong column_number, char *value, ulong length)
 {
   if (ssps_used(stmt))
   {
-     return ssps_get_int64(stmt, column_number, value, length);
+     return ssps_get_int64<long long>(stmt, column_number, value, length);
   }
   else
   {
@@ -339,6 +352,19 @@ long long get_int64(STMT *stmt, ulong column_number, char *value, ulong length)
 }
 
 
+unsigned long long get_uint64(STMT* stmt, ulong column_number, char* value, ulong length)
+{
+  if (ssps_used(stmt))
+  {
+    return ssps_get_int64<unsigned long long>(stmt, column_number, value, length);
+  }
+  else
+  {
+    return strtoull(value, NULL, 10);
+  }
+}
+
+
 char * get_string(STMT *stmt, ulong column_number, char *value, ulong *length,
                   char * buffer)
 {
@@ -385,6 +411,7 @@ BOOL is_null(STMT *stmt, ulong column_number, char *value)
 SQLRETURN prepare(STMT *stmt, char * query, SQLINTEGER query_length,
           bool reset_sql_limit, bool force_prepare)
 {
+  assert(stmt);
   /* TODO: I guess we always have to have query length here */
   if (query_length <= 0)
   {
@@ -402,10 +429,10 @@ SQLRETURN prepare(STMT *stmt, char * query, SQLINTEGER query_length,
   }
 
   ssps_close(stmt);
-  stmt->param_count= PARAM_COUNT(&stmt->query);
+  stmt->param_count= PARAM_COUNT(stmt->query);
   /* Trusting our parsing we are not using prepared statments unsless there are
      actually parameter markers in it */
-  if (!stmt->dbc->ds->no_ssps && (PARAM_COUNT(&stmt->query) || force_prepare)
+  if (!stmt->dbc->ds->no_ssps && (PARAM_COUNT(stmt->query) || force_prepare)
     && !IS_BATCH(&stmt->query)
     && preparable_on_server(&stmt->query, stmt->dbc->mysql->server_version))
   {
@@ -416,7 +443,7 @@ SQLRETURN prepare(STMT *stmt, char * query, SQLINTEGER query_length,
        it at the moment */
     if (!get_cursor_name(&stmt->query))
     {
-     LOCK_STMT(stmt);
+     LOCK_DBC(stmt->dbc);
 
      if (reset_sql_limit)
         set_sql_select_limit(stmt->dbc, 0, false);
@@ -428,7 +455,7 @@ SQLRETURN prepare(STMT *stmt, char * query, SQLINTEGER query_length,
         MYLOG_QUERY(stmt, mysql_error(stmt->dbc->mysql));
 
         stmt->set_error("HY000");
-        translate_error(stmt->error.sqlstate,MYERR_S1000,
+        translate_error((char*)stmt->error.sqlstate.c_str(), MYERR_S1000,
                         mysql_errno(stmt->dbc->mysql));
 
         return SQL_ERROR;
@@ -452,7 +479,6 @@ SQLRETURN prepare(STMT *stmt, char * query, SQLINTEGER query_length,
         fix_result_types(stmt);
        /*Should we reset stmt->result?*/
       }
-    /*assert(stmt->param_count==PARAM_COUNT(&stmt->query));*/
     }
   }
 
@@ -622,6 +648,7 @@ unsigned long long scroller_move(STMT * stmt)
 
 SQLRETURN scroller_prefetch(STMT * stmt)
 {
+  assert(stmt);
   if (stmt->scroller.total_rows > 0
       && stmt->scroller.next_offset >= (stmt->scroller.total_rows + stmt->scroller.start_offset))
   {
@@ -645,7 +672,7 @@ SQLRETURN scroller_prefetch(STMT * stmt)
 
   MYLOG_QUERY(stmt, stmt->scroller.query);
 
-  LOCK_STMT(stmt);
+  LOCK_DBC(stmt->dbc);
 
   if (exec_stmt_query(stmt, stmt->scroller.query,
                         (unsigned long)stmt->scroller.query_len, FALSE))
