@@ -1,4 +1,4 @@
-// Copyright (c) 2001, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2001, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -130,14 +130,12 @@ SQLRETURN         my_SQLPrepare (SQLHSTMT hstmt, SQLCHAR *szSqlStr,
                                  bool force_prepare);
 SQLRETURN         my_SQLExecute         (STMT * stmt);
 SQLRETURN SQL_API my_SQLFreeStmt        (SQLHSTMT hstmt,SQLUSMALLINT fOption);
-SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,
-                                        SQLUSMALLINT fOption, uint clearAllResults);
+SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt, SQLUSMALLINT fOption,
+                                         SQLUSMALLINT fExtra);
 SQLRETURN SQL_API my_SQLAllocStmt       (SQLHDBC hdbc,SQLHSTMT *phstmt);
 SQLRETURN         do_query              (STMT *stmt,char *query, SQLULEN query_length);
 SQLRETURN         insert_params         (STMT *stmt, SQLULEN row, char **finalquery,
                                         SQLULEN *length);
-SQLRETURN odbc_stmt(DBC *dbc, const char *query, SQLULEN query_length,
-                    my_bool reqLock);
 void      myodbc_link_fields (STMT *stmt,MYSQL_FIELD *fields,uint field_count);
 void      fix_row_lengths   (STMT *stmt, const long* fix_rules, uint row, uint field_count);
 void      fix_result_types  (STMT *stmt);
@@ -169,9 +167,10 @@ SQLRETURN
 copy_binary_result(STMT *stmt,
                    SQLCHAR *result, SQLLEN result_bytes, SQLLEN *used_bytes,
                    MYSQL_FIELD *field, char *src, unsigned long src_bytes);
+template <typename T>
 SQLRETURN copy_binhex_result(STMT *stmt,
-           SQLCHAR *rgbValue, SQLINTEGER cbValueMax,
-           SQLLEN *pcbValue, MYSQL_FIELD *field, char *src,
+           T *rgbValue, SQLINTEGER cbValueMax,
+           SQLLEN *pcbValue, char *src,
            ulong src_length);
 SQLRETURN copy_bit_result(STMT *stmt,
                           SQLCHAR *result, SQLLEN result_bytes, SQLLEN *used_bytes,
@@ -272,8 +271,6 @@ bool   myodbc_append_quoted_name_std(std::string &str, const char *name);
 
 SQLRETURN set_handle_error          (SQLSMALLINT HandleType, SQLHANDLE handle,
                                     myodbc_errid errid, const char *errtext, SQLINTEGER errcode);
-SQLRETURN set_conn_error(DBC *dbc,myodbc_errid errid, const char *errtext,
-                        SQLINTEGER errcode);
 SQLRETURN set_env_error (ENV * env,myodbc_errid errid, const char *errtext,
                         SQLINTEGER errcode);
 SQLRETURN copy_str_data (SQLSMALLINT HandleType, SQLHANDLE Handle,
@@ -327,8 +324,6 @@ void query_print          (FILE *log_file,char *query);
 FILE *init_query_log      (void);
 void end_query_log        (FILE *query_log);
 
-LIST *list_delete_forward (LIST *elem);
-
 enum enum_field_types map_sql2mysql_type(SQLSMALLINT sql_type);
 
 /* proc_* functions - used to parse prcedures headers in SQLProcedureColumns */
@@ -378,9 +373,10 @@ int           reset_connection        (DBC *dbc);
 int           wakeup_connection       (DBC *dbc);
 #define WAKEUP_CONN_IF_NEEDED(dbc) if (dbc->need_to_wakeup && wakeup_connection(dbc)) return SQL_ERROR
 
-/*results.c*/
-long long     binary2numeric        (long long *dst, char *src, uint srcLen);
-void          fill_ird_data_lengths (DESC *ird, ulong *lengths, uint fields);
+long long binary2ll(char* src, uint64 srcLen);
+unsigned long long binary2ull(char* src, uint64 srcLen);
+
+void fill_ird_data_lengths (DESC *ird, ulong *lengths, uint fields);
 
 /* Functions to work with prepared and regular statements  */
 #define IS_PS_OUT_PARAMS(_stmt) ((_stmt)->dbc->mysql->server_status & SERVER_PS_OUT_PARAMS)
@@ -406,8 +402,12 @@ SQLRETURN         send_long_data      (STMT *stmt, unsigned int param_num, DESCR
 
 int           get_int     (STMT *stmt, ulong column_number, char *value,
                           ulong length);
+unsigned int  get_uint    (STMT *stmt, ulong column_number, char *value,
+                          ulong length);
 long long     get_int64   (STMT *stmt, ulong column_number, char *value,
                           ulong length);
+unsigned long long get_uint64(STMT * stmt, ulong column_number, char* value,
+                          ulong length);
 char *        get_string  (STMT *stmt, ulong column_number, char *value,
                           ulong *length, char * buffer);
 long double   get_double  (STMT *stmt, ulong column_number, char *value,
@@ -455,8 +455,9 @@ SQLRETURN   ssps_fetch_chunk      (STMT *stmt, char *dest, unsigned long dest_by
                                   unsigned long *avail_bytes);
 void        free_result_bind      (STMT *stmt);
 BOOL        ssps_buffers_need_extending(STMT *stmt);
-long long   ssps_get_int64        (STMT *stmt, ulong column_number, char *value,
-                                  ulong length);
+
+template <typename T>
+T ssps_get_int64 (STMT *stmt, ulong column_number, char *value, ulong length);
 long double ssps_get_double       (STMT *stmt, ulong column_number, char *value,
                                   ulong length);
 char *      ssps_get_string       (STMT *stmt, ulong column_number, char *value,
@@ -537,7 +538,7 @@ void free_connection_stmts(DBC *dbc);
 
 #define CHECK_DBC_OUTPUT(e, d) if(d == NULL) return set_env_error((ENV *)e, MYERR_S1009, NULL, 0)
 
-#define CHECK_STMT_OUTPUT(d, s) if(s == NULL) return set_conn_error((DBC *)d, MYERR_S1009, NULL, 0)
+#define CHECK_STMT_OUTPUT(d, s) if(s == NULL) return ((DBC *)d)->set_error(MYERR_S1009, NULL, 0)
 
 #define CHECK_DESC_OUTPUT(d, s) CHECK_STMT_OUTPUT(d, s)
 
