@@ -1,30 +1,24 @@
-// Copyright (c) 2017, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of MySQL Server, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+/* Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License, version 2.0, for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /**
   This defines built-in functions for use by logging services.
@@ -40,12 +34,14 @@
 #include <mysql/components/component_implementation.h>
 #include <mysql/components/my_service.h>
 #include <mysql/components/service_implementation.h>
+#include <mysql/components/services/log_service.h>
 #include <mysql/components/services/log_shared.h>
 #if defined(MYSQL_DYNAMIC_PLUGIN)
 #include <mysql/service_plugin_registry.h>
 #endif
 #include <stdarg.h>
 #include <stdio.h>
+#include <string.h>
 
 #include <my_compiler.h>
 #if defined(MYSQL_SERVER) && !defined(MYSQL_DYNAMIC_PLUGIN)
@@ -164,7 +160,7 @@ DECLARE_METHOD(bool, item_numeric_class, (log_item_class c));
 
 /**
   Set an integer value on a log_item.
-  Fails gracefully if not log_item_data is supplied, so it can safely
+  Fails gracefully if no log_item_data is supplied, so it can safely
   wrap log_line_item_set[_with_key]().
 
   @param  lid    log_item_data struct to set the value on
@@ -176,7 +172,7 @@ DECLARE_METHOD(bool, item_numeric_class, (log_item_class c));
 DECLARE_METHOD(bool, item_set_int, (log_item_data * lid, longlong i));
 /**
   Set a floating point value on a log_item.
-  Fails gracefully if not log_item_data is supplied, so it can safely
+  Fails gracefully if no log_item_data is supplied, so it can safely
   wrap log_line_item_set[_with_key]().
 
   @param  lid    log_item_data struct to set the value on
@@ -189,7 +185,7 @@ DECLARE_METHOD(bool, item_set_float, (log_item_data * lid, double f));
 
 /**
   Set a string value on a log_item.
-  Fails gracefully if not log_item_data is supplied, so it can safely
+  Fails gracefully if no log_item_data is supplied, so it can safely
   wrap log_line_item_set[_with_key]().
 
   @param  lid    log_item_data struct to set the value on
@@ -204,7 +200,7 @@ DECLARE_METHOD(bool, item_set_lexstring,
 
 /**
   Set a string value on a log_item.
-  Fails gracefully if not log_item_data is supplied, so it can safely
+  Fails gracefully if no log_item_data is supplied, so it can safely
   wrap log_line_item_set[_with_key]().
 
   @param  lid    log_item_data struct to set the value on
@@ -385,13 +381,27 @@ DECLARE_METHOD(int, line_item_count, (log_line * ll));
 DECLARE_METHOD(log_item_type_mask, line_item_types_seen,
                (log_line * ll, log_item_type_mask m));
 
+/**
+  Get log-line's output buffer.
+  If the logger core provides this buffer, the log-service may use it
+  to assemble its output therein and implicitly return it to the core.
+  Participation is required for services that support populating
+  performance_schema.error_log, and optional for all others.
+
+  @param  ll  the log_line to examine
+
+  @retval  nullptr    success, an output buffer is available
+  @retval  otherwise  failure, no output buffer is available
+*/
+DECLARE_METHOD(log_item *, line_get_output_buffer, (log_line * ll));
+
 /**
   Get an iterator for the items in a log_line.
   For now, only one iterator may exist per log_line.
 
   @param  ll  the log_line to examine
 
-  @retval     a log_iterm_iter, or nullptr on failure
+  @retval     a log_iter_iter, or nullptr on failure
 */
 DECLARE_METHOD(log_item_iter *, line_item_iter_acquire, (log_line * ll));
 
@@ -530,10 +540,24 @@ DECLARE_METHOD(longlong, errcode_by_errsymbol, (const char *sym));
 */
 DECLARE_METHOD(const char *, label_from_prio, (int prio));
 
+/**
+  Parse a ISO8601 timestamp and return the number of microseconds
+  since the epoch. Heeds +/- timezone info if present.
+
+  @see make_iso8601_timestamp()
+
+  @param timestamp  an ASCII string containing an ISO8601 timestamp
+  @param len        Length in bytes of the aforementioned string
+
+  @return microseconds since the epoch
+*/
+DECLARE_METHOD(ulonglong, parse_iso8601_timestamp,
+               (const char *timestamp, size_t len));
+
 /**
   open an error log file
 
-  @param       file          if beginning with '.':
+  @param       name_or_ext   if beginning with '.':
                                @@global.log_error, except with this extension
                              otherwise:
                                use this as file name in the same location as
@@ -543,10 +567,15 @@ DECLARE_METHOD(const char *, label_from_prio, (int prio));
 
   @param[out]  my_errstream  an error log handle, or nullptr on failure
 
-  @retval      0             success
-  @retval     !0             failure
+  @returns LOG_SERVICE_SUCCESS                  success
+  @returns LOG_SERVICE_INVALID_ARGUMENT         no my_errstream, or bad log name
+  @returns LOG_SERVICE_OUT_OF_MEMORY            could not allocate file handle
+  @returns LOG_SERVICE_LOCK_ERROR               couldn't lock lock
+  @returns LOG_SERVICE_UNABLE_TO_WRITE          couldn't write to given location
+  @returns LOG_SERVICE_COULD_NOT_MAKE_LOG_NAME  could not make log name
 */
-DECLARE_METHOD(int, open_errstream, (const char *file, void **my_errstream));
+DECLARE_METHOD(log_service_error, open_errstream,
+               (const char *name_or_ext, void **my_errstream));
 
 /**
   write to an error log file previously opened with open_errstream()
@@ -555,10 +584,10 @@ DECLARE_METHOD(int, open_errstream, (const char *file, void **my_errstream));
   @param       buffer        pointer to the string to write
   @param       length        length of the string to write
 
-  @retval  0                 success
-  @retval !0                 failure
+  @returns  LOG_SERVICE_SUCCESS                 success
+  @returns  otherwise                           failure
 */
-DECLARE_METHOD(int, write_errstream,
+DECLARE_METHOD(log_service_error, write_errstream,
                (void *my_errstream, const char *buffer, size_t length));
 
 /**
@@ -576,10 +605,37 @@ DECLARE_METHOD(int, dedicated_errstream, (void *my_errstream));
 
   @param       my_stream  a handle describing the log file
 
-  @retval      0          success
-  @retval     !0          failure
+  @returns    LOG_SERVICE_SUCCESS on success
 */
-DECLARE_METHOD(int, close_errstream, (void **my_errstream));
+DECLARE_METHOD(log_service_error, close_errstream, (void **my_errstream));
+
+/**
+  re-open an error log file
+  (primarily to facilitate flush/log-rotation)
+
+  If the new file can be opened, update the my_errstream descriptor to
+  use it and close the old file. Otherwise, keep using the old file.
+
+  @param       name_or_ext   if beginning with '.':
+                               @@global.log_error, except with this extension
+                             otherwise:
+                               use this as file name in the same location as
+                               @@global.log_error
+
+                             Value may not contain folder separators!
+
+                             In the general case, the caller will be a
+                             log-writer, the log-writer will just pass
+                             its preferred file extension, and the resulting
+                             file name and path will therefore be the same
+                             as for the original log file.
+
+  @param[in,out]  my_errstream  an error log handle
+
+  @returns LOG_SERVICE_INVALID_ARGUMENT, or the result of open_errstream()
+*/
+DECLARE_METHOD(log_service_error, reopen_errstream,
+               (const char *file, void **my_errstream));
 
 END_SERVICE_DEFINITION(log_builtins)
 
@@ -634,7 +690,6 @@ END_SERVICE_DEFINITION(log_builtins_string)
 */
 BEGIN_SERVICE_DEFINITION(log_builtins_tmp)
 // Are we shutting down yet?  Windows EventLog needs to know.
-DECLARE_METHOD(bool, connection_loop_aborted, (void));
 DECLARE_METHOD(size_t, notify_client,
                (void *thd, uint severity, uint code, char *to, size_t n,
                 const char *format, ...))
@@ -645,9 +700,11 @@ END_SERVICE_DEFINITION(log_builtins_tmp)
   Syslog/Eventlog functions for logging services.
 */
 BEGIN_SERVICE_DEFINITION(log_builtins_syseventlog)
-DECLARE_METHOD(int, open, (const char *name, int option, int facility));
-DECLARE_METHOD(int, write, (enum loglevel level, const char *msg));
-DECLARE_METHOD(int, close, (void));
+DECLARE_METHOD(log_service_error, open,
+               (const char *name, int option, int facility));
+DECLARE_METHOD(log_service_error, write,
+               (enum loglevel level, const char *msg));
+DECLARE_METHOD(log_service_error, close, (void));
 END_SERVICE_DEFINITION(log_builtins_syseventlog)
 
 #ifdef __cplusplus
@@ -685,10 +742,88 @@ extern SERVICE_TYPE(log_builtins_string) * log_bs;
 #define log_set_float log_item_set_float
 #define log_set_lexstring log_item_set_lexstring
 #define log_set_cstring log_item_set_cstring
+
+/**
+  Very long-running functions during server start-up can use this
+  function to check whether the time-out for buffered logging has
+  been reached. If so and we have urgent information, all buffered
+  log events will be flushed to the log using built-in default-logger
+  for the time being.  The information will be kept until start-up
+  completes in case it later turns out the user configured a loadable
+  logger, in which case we'll also flush the buffered information to
+  that logger later once the logger becomes available.
+
+  This function should only be used during start-up; once external
+  components are loaded by the component framework, this function
+  should no longer be called (as log events are no longer buffered,
+  but logged immediately).
+*/
+void log_sink_buffer_check_timeout(void);
 #endif  // LOG_H
 
 #ifndef DISABLE_ERROR_LOGGING
 
+#if defined(LOG_COMPONENT_TAG)
+
+#define LogErr(severity, ecode, ...) \
+  LogEvent()                         \
+      .prio(severity)                \
+      .errcode(ecode)                \
+      .subsys(LOG_SUBSYSTEM_TAG)     \
+      .component(LOG_COMPONENT_TAG)  \
+      .source_line(__LINE__)         \
+      .source_file(MY_BASENAME)      \
+      .function(__FUNCTION__)        \
+      .lookup(ecode, ##__VA_ARGS__)
+
+#define LogComponentErr(severity, ecode, ...)                           \
+  LogEvent()                                                            \
+      .prio(severity)                                                   \
+      .errcode(ecode)                                                   \
+      .subsys(LOG_SUBSYSTEM_TAG)                                        \
+      .component("component:" LOG_COMPONENT_TAG)                        \
+      .source_line(__LINE__)                                            \
+      .source_file(MY_BASENAME)                                         \
+      .function(__FUNCTION__)                                           \
+      .lookup_quoted(ecode, "Component " LOG_COMPONENT_TAG " reported", \
+                     ##__VA_ARGS__)
+
+#define LogPluginErr(severity, ecode, ...)                           \
+  LogEvent()                                                         \
+      .prio(severity)                                                \
+      .errcode(ecode)                                                \
+      .subsys(LOG_SUBSYSTEM_TAG)                                     \
+      .component("plugin:" LOG_COMPONENT_TAG)                        \
+      .source_line(__LINE__)                                         \
+      .source_file(MY_BASENAME)                                      \
+      .function(__FUNCTION__)                                        \
+      .lookup_quoted(ecode, "Plugin " LOG_COMPONENT_TAG " reported", \
+                     ##__VA_ARGS__)
+
+#define LogPluginErrV(severity, ecode, vl)    \
+  LogEvent()                                  \
+      .prio(severity)                         \
+      .errcode(ecode)                         \
+      .subsys(LOG_SUBSYSTEM_TAG)              \
+      .component("plugin:" LOG_COMPONENT_TAG) \
+      .source_line(__LINE__)                  \
+      .source_file(MY_BASENAME)               \
+      .function(__FUNCTION__)                 \
+      .lookup_quotedv(ecode, "Plugin " LOG_COMPONENT_TAG " reported", vl)
+
+#define LogPluginErrMsg(severity, ecode, ...) \
+  LogEvent()                                  \
+      .prio(severity)                         \
+      .errcode(ecode)                         \
+      .subsys(LOG_SUBSYSTEM_TAG)              \
+      .component("plugin:" LOG_COMPONENT_TAG) \
+      .source_line(__LINE__)                  \
+      .source_file(MY_BASENAME)               \
+      .function(__FUNCTION__)                 \
+      .message_quoted("Plugin " LOG_COMPONENT_TAG " reported", ##__VA_ARGS__)
+
+#else
+
 #define LogErr(severity, ecode, ...) \
   LogEvent()                         \
       .prio(severity)                \
@@ -699,46 +834,18 @@ extern SERVICE_TYPE(log_builtins_string) * log_bs;
       .function(__FUNCTION__)        \
       .lookup(ecode, ##__VA_ARGS__)
 
-#define LogPluginErr(severity, ecode, ...)                           \
-  LogEvent()                                                         \
-      .prio(severity)                                                \
-      .errcode(ecode)                                                \
-      .subsys(LOG_SUBSYSTEM_TAG)                                     \
-      .source_line(__LINE__)                                         \
-      .source_file(MY_BASENAME)                                      \
-      .function(__FUNCTION__)                                        \
-      .lookup_quoted(ecode, "Plugin " LOG_SUBSYSTEM_TAG " reported", \
-                     ##__VA_ARGS__)
-
-#define LogPluginErrV(severity, ecode, vl) \
-  LogEvent()                               \
-      .prio(severity)                      \
-      .errcode(ecode)                      \
-      .subsys(LOG_SUBSYSTEM_TAG)           \
-      .source_line(__LINE__)               \
-      .source_file(MY_BASENAME)            \
-      .function(__FUNCTION__)              \
-      .lookup_quotedv(ecode, "Plugin " LOG_SUBSYSTEM_TAG " reported", vl)
-
-#define LogPluginErrMsg(severity, ecode, ...) \
-  LogEvent()                                  \
-      .prio(severity)                         \
-      .errcode(ecode)                         \
-      .subsys(LOG_SUBSYSTEM_TAG)              \
-      .source_line(__LINE__)                  \
-      .source_file(MY_BASENAME)               \
-      .function(__FUNCTION__)                 \
-      .message_quoted("Plugin " LOG_SUBSYSTEM_TAG " reported", ##__VA_ARGS__)
+#endif
 
 #else
 
-inline void dummy_log_message(longlong severity MY_ATTRIBUTE((unused)),
-                              longlong ecode MY_ATTRIBUTE((unused)), ...) {
+inline void dummy_log_message(longlong severity [[maybe_unused]],
+                              longlong ecode [[maybe_unused]], ...) {
   return;
 }
 
 #define LogErr(severity, ecode, ...) \
   dummy_log_message(severity, ecode, ##__VA_ARGS__)
+
 #define LogPluginErr(severity, ecode, ...) \
   dummy_log_message(severity, ecode, ##__VA_ARGS__)
 #define LogPluginErrV(severity, ecode, ...) \
@@ -943,14 +1050,15 @@ class LogEvent {
 
   /**
     Which subsystem in the source was the problem detected in?
-    ("rpl" etc.)
+    ("Repl"/"InnoDB"/"Server")
 
     @param  val  the subsystem. NTBS.
 
     @retval      the LogEvent, for easy fluent-style chaining.
   */
   LogEvent &subsys(const char *val) {
-    log_set_cstring(log_line_item_set(this->ll, LOG_ITEM_SRV_SUBSYS), val);
+    if (val != nullptr)
+      log_set_cstring(log_line_item_set(this->ll, LOG_ITEM_SRV_SUBSYS), val);
     return *this;
   }
 
@@ -964,7 +1072,8 @@ class LogEvent {
     @retval      the LogEvent, for easy fluent-style chaining.
   */
   LogEvent &component(const char *val) {
-    log_set_cstring(log_line_item_set(this->ll, LOG_ITEM_SRV_COMPONENT), val);
+    if (val != nullptr)
+      log_set_cstring(log_line_item_set(this->ll, LOG_ITEM_SRV_COMPONENT), val);
     return *this;
   }
 
@@ -1112,6 +1221,24 @@ class LogEvent {
     return *this;
   }
 
+  /**
+    Fill in a format string by substituting the % with the given
+    arguments, then add the result as the event's message.
+    This should be used very sparingly; use registered messages
+    and their error codes wherever possible!
+
+    @param  fmt  message (treated as a printf-style format-string,
+                 so % substitution will happen)
+    @param  ap   valist to satisfy any % in the message
+
+    @retval      the LogEvent, for easy fluent-style chaining.
+  */
+  LogEvent &messagev(const char *fmt, va_list ap)
+      MY_ATTRIBUTE((format(printf, 2, 0))) {
+    set_message(fmt, ap);
+    return *this;
+  }
+
   /**
     Fill in a format string by substituting the % with the given
     arguments, then add the result as the event's message.
@@ -1172,6 +1299,23 @@ class LogEvent {
     return *this;
   }
 
+  /**
+    Find an error message by its MySQL error code. Substitute the % in that
+    message with the given arguments list, then add the result as the event's
+    message.
+
+    @param  errcode  MySQL error code for the message in question,
+                     e.g. ER_STARTUP
+    @param  args     varargs to satisfy any % in the message
+
+    @retval          the LogEvent, for easy fluent-style chaining.
+  */
+  LogEvent &lookupv(longlong errcode, va_list args) {
+    set_message_by_errcode(errcode, args);
+
+    return *this;
+  }
+
   LogEvent &lookup_quoted(longlong errcode, const char *tag, ...) {
     msg_tag = tag;
 
@@ -1267,10 +1411,15 @@ inline void LogEvent::set_message(const char *fmt, va_list ap) {
   if ((ll != nullptr) && (msg != nullptr)) {
     char buf[LOG_BUFF_MAX];
     if (msg_tag != nullptr) {
-      snprintf(buf, LOG_BUFF_MAX - 1, "%s: \'%s\'", msg_tag, fmt);
+      snprintf(buf, LOG_BUFF_MAX, "%s: \'%s\'", msg_tag, fmt);
       fmt = buf;
     }
-    size_t len = log_msg(msg, LOG_BUFF_MAX - 1, fmt, ap);
+    size_t len = log_msg(msg, LOG_BUFF_MAX, fmt, ap);
+    if (len >= LOG_BUFF_MAX) {
+      const char ellipsis[] = " <...>";
+      len = LOG_BUFF_MAX - 1;
+      strcpy(&msg[LOG_BUFF_MAX - sizeof(ellipsis)], ellipsis);
+    }
     log_set_lexstring(log_line_item_set(this->ll, LOG_ITEM_LOG_MESSAGE), msg,
                       len);
   }
@@ -1298,8 +1447,14 @@ inline LogEvent &LogEvent::message(const char *fmt, ...) {
 inline void deinit_logging_service_for_plugin(
     SERVICE_TYPE(registry) * *reg_srv, SERVICE_TYPE(log_builtins) * *log_bi,
     SERVICE_TYPE(log_builtins_string) * *log_bs) {
-  if (*log_bi) (*reg_srv)->release((my_h_service)(*log_bi));
-  if (*log_bs) (*reg_srv)->release((my_h_service)(*log_bs));
+  using log_builtins_t = SERVICE_TYPE_NO_CONST(log_builtins);
+  using log_builtins_string_t = SERVICE_TYPE_NO_CONST(log_builtins_string);
+  if (*log_bi)
+    (*reg_srv)->release(
+        reinterpret_cast<my_h_service>(const_cast<log_builtins_t *>(*log_bi)));
+  if (*log_bs)
+    (*reg_srv)->release(reinterpret_cast<my_h_service>(
+        const_cast<log_builtins_string_t *>(*log_bs)));
   mysql_plugin_registry_release(*reg_srv);
   *log_bi = nullptr;
   *log_bs = nullptr;
@@ -1347,9 +1502,9 @@ inline bool init_logging_service_for_plugin(
   @retval     true   Failed.
 */
 inline bool init_logging_service_for_plugin(
-    SERVICE_TYPE(registry) * *reg_srv MY_ATTRIBUTE((unused)),
-    SERVICE_TYPE(log_builtins) * *log_bi MY_ATTRIBUTE((unused)),
-    SERVICE_TYPE(log_builtins_string) * *log_bs MY_ATTRIBUTE((unused)))
+    SERVICE_TYPE(registry) * *reg_srv [[maybe_unused]],
+    SERVICE_TYPE(log_builtins) * *log_bi [[maybe_unused]],
+    SERVICE_TYPE(log_builtins_string) * *log_bs [[maybe_unused]])
 
 {
   return false;
@@ -1363,9 +1518,9 @@ inline bool init_logging_service_for_plugin(
   param[in,out]  log_bs     String service for error logging.
 */
 inline void deinit_logging_service_for_plugin(
-    SERVICE_TYPE(registry) * *reg_srv MY_ATTRIBUTE((unused)),
-    SERVICE_TYPE(log_builtins) * *log_bi MY_ATTRIBUTE((unused)),
-    SERVICE_TYPE(log_builtins_string) * *log_bs MY_ATTRIBUTE((unused))) {}
+    SERVICE_TYPE(registry) * *reg_srv [[maybe_unused]],
+    SERVICE_TYPE(log_builtins) * *log_bi [[maybe_unused]],
+    SERVICE_TYPE(log_builtins_string) * *log_bs [[maybe_unused]]) {}
 
 #endif  // MYSQL_DYNAMIC_PLUGIN
 
