@@ -1,30 +1,24 @@
-// Copyright (c) 2005, 2017, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of MySQL Server, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+/* Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License, version 2.0, for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #ifndef _my_plugin_h
 #define _my_plugin_h
@@ -102,7 +96,7 @@ struct MYSQL_XID {
   Plugin API. Common for all plugin types.
 */
 
-#define MYSQL_PLUGIN_INTERFACE_VERSION 0x0109
+#define MYSQL_PLUGIN_INTERFACE_VERSION 0x010A
 
 /*
   The allowable types of plugins
@@ -165,6 +159,7 @@ struct MYSQL_XID {
 
 #define PLUGIN_OPT_NO_INSTALL 1UL   /* Not dynamically loadable */
 #define PLUGIN_OPT_NO_UNINSTALL 2UL /* Not dynamically unloadable */
+#define PLUGIN_OPT_ALLOW_EARLY 4UL  /* allow --early-plugin-load */
 
 /*
   declarations for server variables and command line options
@@ -181,7 +176,7 @@ struct MYSQL_XID {
 #define PLUGIN_VAR_UNSIGNED 0x0080
 #define PLUGIN_VAR_THDLOCAL 0x0100  /* Variable is per-connection */
 #define PLUGIN_VAR_READONLY 0x0200  /* Server variable is read only */
-#define PLUGIN_VAR_NOSYSVAR 0x0400  /* Not a server variable */
+#define PLUGIN_VAR_NOSYSVAR 0x0400  /* Configurable only by cmd-line */
 #define PLUGIN_VAR_NOCMDOPT 0x0800  /* Not a command line option */
 #define PLUGIN_VAR_NOCMDARG 0x1000  /* No argument for cmd line */
 #define PLUGIN_VAR_RQCMDARG 0x0000  /* Argument required for cmd line */
@@ -191,12 +186,14 @@ struct MYSQL_XID {
 #define PLUGIN_VAR_NOPERSIST                \
   0x10000 /* SET PERSIST_ONLY is prohibited \
              for read only variables */
+
 /**
   There can be some variables which needs to be set before plugin is loaded but
   not after plugin is loaded. ex: GR specific variables. Below flag must be set
   for these kind of variables.
 */
 #define PLUGIN_VAR_PERSIST_AS_READ_ONLY 0x20000
+#define PLUGIN_VAR_INVISIBLE 0x40000 /* Variable should not be shown */
 
 struct SYS_VAR;
 struct st_mysql_value;
@@ -246,7 +243,7 @@ typedef void (*mysql_var_update_func)(MYSQL_THD thd, SYS_VAR *var,
   (PLUGIN_VAR_READONLY | PLUGIN_VAR_NOSYSVAR | PLUGIN_VAR_NOCMDOPT |   \
    PLUGIN_VAR_NOCMDARG | PLUGIN_VAR_OPCMDARG | PLUGIN_VAR_RQCMDARG |   \
    PLUGIN_VAR_MEMALLOC | PLUGIN_VAR_NODEFAULT | PLUGIN_VAR_NOPERSIST | \
-   PLUGIN_VAR_PERSIST_AS_READ_ONLY)
+   PLUGIN_VAR_PERSIST_AS_READ_ONLY | PLUGIN_VAR_INVISIBLE)
 
 #define MYSQL_PLUGIN_VAR_HEADER \
   int flags;                    \
@@ -467,7 +464,7 @@ typedef void (*mysql_var_update_func)(MYSQL_THD thd, SYS_VAR *var,
       blk}
 
 #define MYSQL_THDVAR_BOOL(name, opt, comment, check, update, def)      \
-  DECLARE_MYSQL_THDVAR_BASIC(name, char) = {                           \
+  DECLARE_MYSQL_THDVAR_BASIC(name, bool) = {                           \
       PLUGIN_VAR_BOOL | PLUGIN_VAR_THDLOCAL | ((opt)&PLUGIN_VAR_MASK), \
       #name,                                                           \
       comment,                                                         \
@@ -819,11 +816,11 @@ int mysql_tmpfile(const char *prefix);
   time-consuming loops, and gracefully abort the operation if it is
   non-zero.
 
-  @param thd  user thread connection handle
+  @param v_thd  user thread connection handle
   @retval 0  the connection is active
   @retval 1  the connection has been killed
 */
-int thd_killed(const MYSQL_THD thd);
+int thd_killed(const void *v_thd);
 
 /**
   Set the killed status of the current statement.
@@ -897,6 +894,11 @@ void thd_set_ha_data(MYSQL_THD thd, const struct handlerton *hton,
 */
 
 void remove_ssl_err_thread_state();
+
+/**
+  Interface to get the number of VCPUs.
+*/
+unsigned int thd_get_num_vcpus();
 #ifdef __cplusplus
 }
 #endif
