@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2016, 2021, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -34,7 +34,9 @@
 
 #include "my_inttypes.h"
 #include "my_sharedlib.h"
-#include "psi_base.h"
+
+/* HAVE_PSI_*_INTERFACE */
+#include "my_psi_config.h"  // IWYU pragma: keep
 
 #ifdef HAVE_PSI_DATA_LOCK_INTERFACE
 
@@ -104,8 +106,8 @@ typedef struct PSI_data_lock_bootstrap PSI_data_lock_bootstrap;
 */
 class PSI_server_data_lock_container {
  public:
-  PSI_server_data_lock_container() {}
-  virtual ~PSI_server_data_lock_container() {}
+  PSI_server_data_lock_container() = default;
+  virtual ~PSI_server_data_lock_container() = default;
 
   /**
     Add a string to the container cache.
@@ -190,8 +192,8 @@ class PSI_server_data_lock_container {
 
 class PSI_server_data_lock_wait_container {
  public:
-  PSI_server_data_lock_wait_container() {}
-  virtual ~PSI_server_data_lock_wait_container() {}
+  PSI_server_data_lock_wait_container() = default;
+  virtual ~PSI_server_data_lock_wait_container() = default;
 
   /** @sa PSI_server_data_lock_container::cache_string. */
   virtual const char *cache_string(const char *string) = 0;
@@ -287,8 +289,8 @@ class PSI_server_data_lock_wait_container {
 */
 class PSI_engine_data_lock_iterator {
  public:
-  PSI_engine_data_lock_iterator() {}
-  virtual ~PSI_engine_data_lock_iterator() {}
+  PSI_engine_data_lock_iterator() = default;
+  virtual ~PSI_engine_data_lock_iterator() = default;
 
   /**
     Scan for more data locks.
@@ -314,8 +316,8 @@ class PSI_engine_data_lock_iterator {
 
 class PSI_engine_data_lock_wait_iterator {
  public:
-  PSI_engine_data_lock_wait_iterator() {}
-  virtual ~PSI_engine_data_lock_wait_iterator() {}
+  PSI_engine_data_lock_wait_iterator() = default;
+  virtual ~PSI_engine_data_lock_wait_iterator() = default;
 
   /**
     Scan for more data lock waits.
@@ -356,8 +358,8 @@ class PSI_engine_data_lock_wait_iterator {
 */
 class PSI_engine_data_lock_inspector {
  public:
-  PSI_engine_data_lock_inspector() {}
-  virtual ~PSI_engine_data_lock_inspector() {}
+  PSI_engine_data_lock_inspector() = default;
+  virtual ~PSI_engine_data_lock_inspector() = default;
 
   /**
     Create a data lock iterator.
