@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -34,10 +34,12 @@
 
 #include "my_inttypes.h"
 #include "my_macros.h"
-#include "my_psi_config.h"
+
+/* HAVE_PSI_*_INTERFACE */
+#include "my_psi_config.h"  // IWYU pragma: keep
+
 #include "my_sharedlib.h"
 #include "mysql/components/services/psi_stage_bits.h"
-#include "psi_base.h"
 
 /** Entry point for the performance schema interface. */
 struct PSI_stage_bootstrap {
