@@ -1,7 +1,7 @@
 #ifndef SQL_STRING_INCLUDED
 #define SQL_STRING_INCLUDED
 
-/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,6 +29,7 @@
   See in particular the comment on String before you use anything from here.
 */
 
+#include <assert.h>
 #include <stdint.h>
 #include <string.h>
 #include <sys/types.h>
@@ -41,10 +42,10 @@
 #include "memory_debugging.h"
 #include "my_alloc.h"
 #include "my_compiler.h"
-#include "my_dbug.h"
+
 #include "my_inttypes.h"
+#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/mysql_lex_string.h"  // LEX_STRING
-#include "mysql/psi/psi_base.h"
 #include "mysql/psi/psi_memory.h"
 #include "mysql/service_mysql_alloc.h"  // my_free
 
@@ -79,19 +80,19 @@ class Simple_cstring {
   */
   void set(const char *str_arg, size_t length_arg) {
     // NULL is allowed only with length==0
-    DBUG_ASSERT(str_arg || length_arg == 0);
+    assert(str_arg || length_arg == 0);
     // For non-NULL, make sure length_arg is in sync with '\0' terminator.
-    DBUG_ASSERT(!str_arg || str_arg[length_arg] == '\0');
+    assert(!str_arg || str_arg[length_arg] == '\0');
     m_str = str_arg;
     m_length = length_arg;
   }
-  Simple_cstring() { set(NULL, 0); }
+  Simple_cstring() { set(nullptr, 0); }
   Simple_cstring(const char *str_arg, size_t length_arg) {
     set(str_arg, length_arg);
   }
   Simple_cstring(const LEX_STRING arg) { set(arg.str, arg.length); }
   Simple_cstring(const LEX_CSTRING arg) { set(arg.str, arg.length); }
-  void reset() { set(NULL, 0); }
+  void reset() { set(nullptr, 0); }
   /**
     Set to a null-terminated string.
   */
@@ -103,7 +104,7 @@ class Simple_cstring {
   /**
     Check if m_ptr is set.
   */
-  bool is_set() const { return m_str != NULL; }
+  bool is_set() const { return m_str != nullptr; }
   /**
     Return name length.
   */
@@ -171,13 +172,13 @@ class String {
 
  public:
   String()
-      : m_ptr(NULL),
+      : m_ptr(nullptr),
         m_length(0),
         m_charset(&my_charset_bin),
         m_alloced_length(0),
         m_is_alloced(false) {}
   explicit String(size_t length_arg)
-      : m_ptr(NULL),
+      : m_ptr(nullptr),
         m_length(0),
         m_charset(&my_charset_bin),
         m_alloced_length(0),
@@ -217,8 +218,8 @@ class String {
     str.m_is_alloced = false;
   }
   static void *operator new(size_t size, MEM_ROOT *mem_root,
-                            const std::nothrow_t &arg MY_ATTRIBUTE((unused)) =
-                                std::nothrow) noexcept {
+                            const std::nothrow_t &arg
+                            [[maybe_unused]] = std::nothrow) noexcept {
     return mem_root->Alloc(size);
   }
   static void operator delete(void *ptr_arg, size_t size) {
@@ -228,7 +229,7 @@ class String {
   }
 
   static void operator delete(
-      void *, MEM_ROOT *, const std::nothrow_t &)noexcept { /* never called */
+      void *, MEM_ROOT *, const std::nothrow_t &) noexcept { /* never called */
   }
 
   ~String() { mem_free(); }
@@ -246,8 +247,8 @@ class String {
   const char *ptr() const { return m_ptr; }
   char *ptr() { return m_ptr; }
   char *c_ptr() {
-    DBUG_ASSERT(!m_is_alloced || !m_ptr || !m_alloced_length ||
-                (m_alloced_length >= (m_length + 1)));
+    assert(!m_is_alloced || !m_ptr || !m_alloced_length ||
+           (m_alloced_length >= (m_length + 1)));
 
     /*
       Should be safe, but in case valgrind complains on this line, it means
@@ -275,7 +276,7 @@ class String {
   }
 
   void set(String &str, size_t offset, size_t arg_length) {
-    DBUG_ASSERT(&str != this);
+    assert(&str != this);
     mem_free();
     m_ptr = str.ptr() + offset;
     m_length = arg_length;
@@ -366,9 +367,9 @@ class String {
     m_ptr[m_length] = '\0';
   }
 
-  void mem_claim() {
+  void mem_claim(bool claim) {
     if (m_is_alloced) {
-      my_claim(m_ptr);
+      my_claim(m_ptr, claim);
     }
   }
 
@@ -377,7 +378,7 @@ class String {
       m_is_alloced = false;
       m_alloced_length = 0;
       my_free(m_ptr);
-      m_ptr = NULL;
+      m_ptr = nullptr;
       m_length = 0; /* Safety */
     }
   }
@@ -416,7 +417,7 @@ class String {
         It is forbidden to do assignments like
         some_string = substring_of_that_string
        */
-      DBUG_ASSERT(!s.uses_buffer_owned_by(this));
+      assert(!s.uses_buffer_owned_by(this));
       mem_free();
       m_ptr = s.m_ptr;
       m_length = s.m_length;
@@ -432,7 +433,7 @@ class String {
         It is forbidden to do assignments like
         some_string = substring_of_that_string
        */
-      DBUG_ASSERT(!s.uses_buffer_owned_by(this));
+      assert(!s.uses_buffer_owned_by(this));
       mem_free();
       m_ptr = s.m_ptr;
       m_length = s.m_length;
@@ -446,23 +447,23 @@ class String {
   }
   /**
     Takeover the buffer owned by another string.
-    "this" becames the owner of the buffer and
+    "this" becomes the owner of the buffer and
     is further responsible to free it.
-    The string "s" is detouched from the buffer (cleared).
+    The string "s" is detached from the buffer (cleared).
 
     @param s - a String object to steal buffer from.
   */
   void takeover(String &s) {
-    DBUG_ASSERT(this != &s);
+    assert(this != &s);
     // Make sure buffers of the two Strings do not overlap
-    DBUG_ASSERT(!s.uses_buffer_owned_by(this));
+    assert(!s.uses_buffer_owned_by(this));
     mem_free();
     m_ptr = s.m_ptr;
     m_length = s.m_length;
     m_alloced_length = s.m_alloced_length;
     m_is_alloced = s.m_is_alloced;
     m_charset = s.m_charset;
-    s.m_ptr = NULL;
+    s.m_ptr = nullptr;
     s.m_alloced_length = 0;
     s.m_length = 0;
     s.m_is_alloced = false;
@@ -551,7 +552,10 @@ class String {
   size_t charpos(size_t i, size_t offset = 0) const;
 
   bool reserve(size_t space_needed) {
-    return mem_realloc(m_length + space_needed);
+    if (m_alloced_length < m_length + space_needed) {
+      return mem_realloc(m_length + space_needed);
+    }
+    return false;
   }
   bool reserve(size_t space_needed, size_t grow_by);
 
@@ -560,7 +564,7 @@ class String {
   char *prep_append(size_t arg_length, size_t step_alloc) {
     size_t new_length = arg_length + m_length;
     if (new_length > m_alloced_length) {
-      if (mem_realloc(new_length + step_alloc)) return NULL;
+      if (mem_realloc(new_length + step_alloc)) return nullptr;
     }
     size_t old_length = m_length;
     m_length += arg_length;
@@ -651,7 +655,7 @@ inline LEX_STRING to_lex_string(const LEX_CSTRING &s) {
 }
 
 inline LEX_CSTRING to_lex_cstring(const char *s) {
-  LEX_CSTRING cstr = {s, s != NULL ? strlen(s) : 0};
+  LEX_CSTRING cstr = {s, s != nullptr ? strlen(s) : 0};
   return cstr;
 }
 
