@@ -1,4 +1,4 @@
-// Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -509,12 +509,18 @@ int list_datasource_details(DataSource *ds)
   if (ds->sslcapath  ) printf("SSL CA path:         %s\n", ds_get_utf8attr(ds->sslcapath, &ds->sslcapath8));
   if (ds->sslcipher  ) printf("SSL cipher:          %s\n", ds_get_utf8attr(ds->sslcipher, &ds->sslcipher8));
   if (ds->sslmode   ) printf("SSL Mode:             %s\n", ds_get_utf8attr(ds->sslmode, &ds->sslmode8));
-  if (ds->sslverify) printf("Verify SSL cert      yes\n");
-  if (ds->rsakey)    printf("RSA public key       %s\n", ds_get_utf8attr(ds->rsakey, &ds->rsakey8));
+  if (ds->sslverify) printf("Verify SSL cert:      yes\n");
+  if (ds->rsakey)    printf("RSA public key:       %s\n", ds_get_utf8attr(ds->rsakey, &ds->rsakey8));
+  if (ds->tls_versions) printf("TLS Versions:        %s\n", ds_get_utf8attr(ds->tls_versions, &ds->tls_versions8));
+  if (ds->ssl_crl   ) printf("SSL CRL:             %s\n", ds_get_utf8attr(ds->ssl_crl, &ds->ssl_crl8));
+  if (ds->ssl_crlpath) printf("SSL CRL path:        %s\n", ds_get_utf8attr(ds->ssl_crlpath, &ds->ssl_crlpath8));
   if (ds->port && ds->has_port) printf("Port:                %d\n", ds->port);
   if (ds->plugin_dir  ) printf("Plugin directory:    %s\n", ds_get_utf8attr(ds->plugin_dir, &ds->plugin_dir8));
   if (ds->default_auth) printf("Default Authentication Library: %s\n", ds_get_utf8attr(ds->default_auth, &ds->default_auth8));
   if (ds->oci_config_file) printf("OCI Config File: %s\n", ds_get_utf8attr(ds->oci_config_file, &ds->oci_config_file8));
+  if (ds->authentication_kerberos_mode) printf("Kerberos Authentication Mode: %s\n",
+    ds_get_utf8attr(ds->authentication_kerberos_mode,
+    &ds->authentication_kerberos_mode8));
   printf("Options:\n");
   if (ds->return_matching_rows) printf("\tFOUND_ROWS\n");
   if (ds->allow_big_results) printf("\tBIG_PACKETS\n");
@@ -550,7 +556,6 @@ int list_datasource_details(DataSource *ds)
   if (ds->no_tls_1_2) printf("\tNO_TLS_1_2\n");
   if (ds->no_tls_1_3) printf("\tNO_TLS_1_3\n");
   if (ds->no_ssps) printf("\tNO_SSPS\n");
-  if (ds->no_information_schema) printf("\tNO_I_S\n");
   if (ds->cursor_prefetch_number) printf("\tPREFETCH=%d\n", ds->cursor_prefetch_number);
   if (ds->readtimeout) printf("\tREADTIMEOUT=%d\n", ds->readtimeout);
   if (ds->writetimeout) printf("\tWRITETIMEOUT=%d\n", ds->writetimeout);
@@ -860,7 +865,7 @@ int main(int argc, char **argv)
 
   /* init libmysqlclient */
   my_sys_init();
-  utf8_charset_info= get_charset_by_csname("utf8", MYF(MY_CS_PRIMARY),
+  utf8_charset_info= get_charset_by_csname(transport_charset, MYF(MY_CS_PRIMARY),
                                            MYF(0));
 
   /* convert to SQLWCHAR for installer API */
