@@ -1,4 +1,4 @@
-/* Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2002, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,13 +25,16 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
+#include <assert.h>
 #include <string.h>
 #include <sys/types.h>
 
+#include <algorithm>
+
 #include "m_ctype.h"
 #include "m_string.h"
 #include "my_compiler.h"
-#include "my_dbug.h"
+
 #include "my_inttypes.h"
 #include "my_macros.h"
 #include "str_uca_type.h"
@@ -74,23 +77,23 @@ size_t my_casedn_str_mb(const CHARSET_INFO *cs, char *str) {
 static inline const MY_UNICASE_CHARACTER *get_case_info_for_ch(
     const CHARSET_INFO *cs, uint page, uint offs) {
   const MY_UNICASE_CHARACTER *p;
-  return cs->caseinfo ? ((p = cs->caseinfo->page[page]) ? &p[offs] : NULL)
-                      : NULL;
+  return cs->caseinfo ? ((p = cs->caseinfo->page[page]) ? &p[offs] : nullptr)
+                      : nullptr;
 }
 
 /*
   For character sets which don't change octet length in case conversion.
 */
 size_t my_caseup_mb(const CHARSET_INFO *cs, char *src, size_t srclen,
-                    char *dst MY_ATTRIBUTE((unused)),
-                    size_t dstlen MY_ATTRIBUTE((unused))) {
+                    char *dst [[maybe_unused]],
+                    size_t dstlen [[maybe_unused]]) {
   uint32 l;
   char *srcend = src + srclen;
   const uchar *map = cs->to_upper;
 
-  DBUG_ASSERT(cs->caseup_multiply == 1);
-  DBUG_ASSERT(src == dst && srclen == dstlen);
-  DBUG_ASSERT(cs->mbmaxlen == 2);
+  assert(cs->caseup_multiply == 1);
+  assert(src == dst && srclen == dstlen);
+  assert(cs->mbmaxlen == 2);
 
   while (src < srcend) {
     if ((l = my_ismbchar(cs, src, srcend))) {
@@ -109,15 +112,15 @@ size_t my_caseup_mb(const CHARSET_INFO *cs, char *src, size_t srclen,
 }
 
 size_t my_casedn_mb(const CHARSET_INFO *cs, char *src, size_t srclen,
-                    char *dst MY_ATTRIBUTE((unused)),
-                    size_t dstlen MY_ATTRIBUTE((unused))) {
+                    char *dst [[maybe_unused]],
+                    size_t dstlen [[maybe_unused]]) {
   uint32 l;
   char *srcend = src + srclen;
   const uchar *map = cs->to_lower;
 
-  DBUG_ASSERT(cs->casedn_multiply == 1);
-  DBUG_ASSERT(src == dst && srclen == dstlen);
-  DBUG_ASSERT(cs->mbmaxlen == 2);
+  assert(cs->casedn_multiply == 1);
+  assert(src == dst && srclen == dstlen);
+  assert(cs->mbmaxlen == 2);
 
   while (src < srcend) {
     if ((l = my_ismbchar(cs, src, srcend))) {
@@ -146,11 +149,11 @@ size_t my_casedn_mb(const CHARSET_INFO *cs, char *src, size_t srclen,
 */
 static size_t my_casefold_mb_varlen(const CHARSET_INFO *cs, char *src,
                                     size_t srclen, char *dst,
-                                    size_t dstlen MY_ATTRIBUTE((unused)),
+                                    size_t dstlen [[maybe_unused]],
                                     const uchar *map, size_t is_upper) {
   char *srcend = src + srclen, *dst0 = dst;
 
-  DBUG_ASSERT(cs->mbmaxlen == 2);
+  assert(cs->mbmaxlen == 2);
 
   while (src < srcend) {
     size_t mblen = my_ismbchar(cs, src, srcend);
@@ -174,15 +177,15 @@ static size_t my_casefold_mb_varlen(const CHARSET_INFO *cs, char *src,
 
 size_t my_casedn_mb_varlen(const CHARSET_INFO *cs, char *src, size_t srclen,
                            char *dst, size_t dstlen) {
-  DBUG_ASSERT(dstlen >= srclen * cs->casedn_multiply);
-  DBUG_ASSERT(src != dst || cs->casedn_multiply == 1);
+  assert(dstlen >= srclen * cs->casedn_multiply);
+  assert(src != dst || cs->casedn_multiply == 1);
   return my_casefold_mb_varlen(cs, src, srclen, dst, dstlen, cs->to_lower, 0);
 }
 
 size_t my_caseup_mb_varlen(const CHARSET_INFO *cs, char *src, size_t srclen,
                            char *dst, size_t dstlen) {
-  DBUG_ASSERT(dstlen >= srclen * cs->caseup_multiply);
-  DBUG_ASSERT(src != dst || cs->caseup_multiply == 1);
+  assert(dstlen >= srclen * cs->caseup_multiply);
+  assert(src != dst || cs->caseup_multiply == 1);
   return my_casefold_mb_varlen(cs, src, srclen, dst, dstlen, cs->to_upper, 1);
 }
 
@@ -204,7 +207,7 @@ int my_strcasecmp_mb(const CHARSET_INFO *cs, const char *s, const char *t) {
       return 1;
   }
   /* At least one of '*s' and '*t' is zero here. */
-  DBUG_ASSERT(!*t || !*s);
+  assert(!*t || !*s);
   return (*t != *s);
 }
 
@@ -403,10 +406,10 @@ uint my_instr_mb(const CHARSET_INFO *cs, const char *b, size_t b_length,
 
 /* BINARY collations handlers for MB charsets */
 
-int my_strnncoll_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
-                        const uchar *s, size_t slen, const uchar *t,
-                        size_t tlen, bool t_is_prefix) {
-  size_t len = MY_MIN(slen, tlen);
+int my_strnncoll_mb_bin(const CHARSET_INFO *cs [[maybe_unused]], const uchar *s,
+                        size_t slen, const uchar *t, size_t tlen,
+                        bool t_is_prefix) {
+  size_t len = std::min(slen, tlen);
   int cmp = len == 0 ? 0 : memcmp(s, t, len);
   return cmp ? cmp : (int)((t_is_prefix ? len : slen) - tlen);
 }
@@ -433,14 +436,14 @@ int my_strnncoll_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
     0 if strings are equal
 */
 
-int my_strnncollsp_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
+int my_strnncollsp_mb_bin(const CHARSET_INFO *cs [[maybe_unused]],
                           const uchar *a, size_t a_length, const uchar *b,
                           size_t b_length) {
   const uchar *end;
   size_t length;
   int res;
 
-  end = a + (length = MY_MIN(a_length, b_length));
+  end = a + (length = std::min(a_length, b_length));
   while (a < end) {
     if (*a++ != *b++) return ((int)a[-1] - (int)b[-1]);
   }
@@ -478,13 +481,13 @@ int my_strnncollsp_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
     switch (cs->cset->ismbchar(cs, (const char *)src, (const char *)se)) {   \
       case 4:                                                                \
         *dst++ = *src++;                                                     \
-        /* fall through */                                                   \
+        [[fallthrough]];                                                     \
       case 3:                                                                \
         *dst++ = *src++;                                                     \
-        /* fall through */                                                   \
+        [[fallthrough]];                                                     \
       case 2:                                                                \
         *dst++ = *src++;                                                     \
-        /* fall through */                                                   \
+        [[fallthrough]];                                                     \
       case 0:                                                                \
         *dst++ = *src++; /* byte in range 0x80..0xFF which is not MB head */ \
     }                                                                        \
@@ -503,7 +506,7 @@ size_t my_strnxfrm_mb(const CHARSET_INFO *cs, uchar *dst, size_t dstlen,
   const uchar *se = src + srclen;
   const uchar *sort_order = cs->sort_order;
 
-  DBUG_ASSERT(cs->mbmaxlen <= 4);
+  assert(cs->mbmaxlen <= 4);
 
   /*
     If "srclen" is smaller than both "dstlen" and "nweights"
@@ -554,12 +557,12 @@ pad:
   return my_strxfrm_pad(cs, d0, dst, de, nweights, flags);
 }
 
-int my_strcasecmp_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
-                         const char *s, const char *t) {
+int my_strcasecmp_mb_bin(const CHARSET_INFO *cs [[maybe_unused]], const char *s,
+                         const char *t) {
   return strcmp(s, t);
 }
 
-void my_hash_sort_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
+void my_hash_sort_mb_bin(const CHARSET_INFO *cs [[maybe_unused]],
                          const uchar *key, size_t len, uint64 *nr1,
                          uint64 *nr2) {
   const uchar *pos = key;
@@ -609,7 +612,7 @@ static void pad_max_char(const CHARSET_INFO *cs, char *str, char *end) {
       buflen = 2;
     } else {
       /* Currently, it's only for GB18030, so it must be a 4-byte char */
-      DBUG_ASSERT(cs->max_sort_char > 0xFFFFFF);
+      assert(cs->max_sort_char > 0xFFFFFF);
       buf[0] = cs->max_sort_char >> 24 & 0xFF;
       buf[1] = cs->max_sort_char >> 16 & 0xFF;
       buf[2] = cs->max_sort_char >> 8 & 0xFF;
@@ -621,7 +624,7 @@ static void pad_max_char(const CHARSET_INFO *cs, char *str, char *end) {
                              (uchar *)buf + sizeof(buf));
   }
 
-  DBUG_ASSERT(buflen > 0);
+  assert(buflen > 0);
   do {
     if ((str + buflen) <= end) {
       /* Enough space for the characer */
@@ -831,11 +834,11 @@ bool my_like_range_generic(const CHARSET_INFO *cs, const char *ptr,
                                  pointer_cast<const uchar *>(end))) <= 0) {
         if (res == MY_CS_ILSEQ)
           return true; /* min_length and max_length are not important */
-        /*
-           End of the string: Escape is the last character.
-           Put escape as a normal character.
-           We'll will leave the loop on the next iteration.
-        */
+                       /*
+                          End of the string: Escape is the last character.
+                          Put escape as a normal character.
+                          We'll will leave the loop on the next iteration.
+                       */
       } else
         ptr += res;
 
@@ -1232,43 +1235,58 @@ static const struct {
   int page;
   const char *p;
 } utr11_data[256] = {
-    {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL},
-    {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL},
-    {0, NULL}, {0, NULL}, {0, NULL}, {0, pg11}, {0, NULL}, {0, NULL}, {0, NULL},
-    {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL},
-    {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL},
-    {0, pg23}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL},
-    {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, pg2E}, {0, pg2F}, {0, pg30},
-    {0, pg31}, {0, pg32}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {0, pg4D}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {0, pg9F}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {0, pgA4}, {0, NULL}, {0, NULL}, {0, NULL},
-    {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL},
-    {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {1, NULL}, {0, pgD7}, {0, NULL},
-    {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL},
-    {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL},
-    {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL},
-    {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL},
-    {0, NULL}, {0, NULL}, {0, NULL}, {0, NULL}, {1, NULL}, {0, pgFA}, {0, NULL},
-    {0, NULL}, {0, NULL}, {0, pgFE}, {0, pgFF}};
+    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, nullptr}, {0, pg11},    {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, pg23},    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, pg2E},    {0, pg2F},    {0, pg30},    {0, pg31},
+    {0, pg32},    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {0, pg4D},    {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {0, pg9F},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {0, pgA4},
+    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr}, {1, nullptr},
+    {0, pgD7},    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr},
+    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, nullptr}, {1, nullptr},
+    {0, pgFA},    {0, nullptr}, {0, nullptr}, {0, nullptr}, {0, pgFE},
+    {0, pgFF}};
 
 size_t my_numcells_mb(const CHARSET_INFO *cs, const char *b, const char *e) {
   my_wc_t wc;
