@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -92,7 +92,7 @@ static void map_coll_name_to_number(const char *name, int num) {
   lower_case_name[len] = '\0';
   my_casedn_str(&my_charset_latin1, lower_case_name);
 
-  DBUG_ASSERT(coll_name_num_map != nullptr);
+  assert(coll_name_num_map != nullptr);
   (*coll_name_num_map)[lower_case_name] = num;
 }
 
@@ -103,7 +103,7 @@ static void map_cs_name_to_number(const char *name, int num, int state) {
   lower_case_name[len] = '\0';
   my_casedn_str(&my_charset_latin1, lower_case_name);
 
-  DBUG_ASSERT(cs_name_pri_num_map != nullptr && cs_name_bin_num_map != nullptr);
+  assert(cs_name_pri_num_map != nullptr && cs_name_bin_num_map != nullptr);
   if ((state & MY_CS_PRIMARY)) (*cs_name_pri_num_map)[lower_case_name] = num;
   if ((state & MY_CS_BINSORT)) (*cs_name_bin_num_map)[lower_case_name] = num;
 }
@@ -115,7 +115,7 @@ static uint get_collation_number_internal(const char *name) {
   lower_case_name[len] = '\0';
   my_casedn_str(&my_charset_latin1, lower_case_name);
 
-  DBUG_ASSERT(coll_name_num_map != nullptr);
+  assert(coll_name_num_map != nullptr);
   auto name_num_map_it = coll_name_num_map->find(lower_case_name);
   if (name_num_map_it != coll_name_num_map->end())
     return name_num_map_it->second;
@@ -209,6 +209,15 @@ static void copy_uca_collation(CHARSET_INFO *to, CHARSET_INFO *from) {
   to->state |= MY_CS_AVAILABLE | MY_CS_LOADED | MY_CS_STRNXFRM | MY_CS_UNICODE;
 }
 
+static void clear_cs_info(CHARSET_INFO *cs) {
+  cs->number = 0;
+  cs->primary_number = 0;
+  cs->binary_number = 0;
+  cs->name = nullptr;
+  cs->state = 0;
+  cs->sort_order = nullptr;
+}
+
 static int add_collation(CHARSET_INFO *cs) {
   if (cs->name &&
       (cs->number || (cs->number = get_collation_number_internal(cs->name))) &&
@@ -218,6 +227,10 @@ static int add_collation(CHARSET_INFO *cs) {
                 (CHARSET_INFO *)my_once_alloc(sizeof(CHARSET_INFO), MYF(0))))
         return MY_XML_ERROR;
       memset(all_charsets[cs->number], 0, sizeof(CHARSET_INFO));
+    } else if (all_charsets[cs->number]->state & MY_CS_COMPILED) {
+      // Disallow overwriting compiled character sets
+      clear_cs_info(cs);
+      return MY_XML_OK;  // Just ignore it.
     }
 
     if (cs->primary_number == cs->number) cs->state |= MY_CS_PRIMARY;
@@ -303,13 +316,7 @@ static int add_collation(CHARSET_INFO *cs) {
         if (!(dst->name = my_once_strdup(cs->name, MYF(MY_WME))))
           return MY_XML_ERROR;
     }
-    cs->number = 0;
-    cs->primary_number = 0;
-    cs->binary_number = 0;
-    cs->name = NULL;
-    cs->state = 0;
-    cs->sort_order = NULL;
-    cs->state = 0;
+    clear_cs_info(cs);
   }
   return MY_XML_OK;
 }
@@ -318,8 +325,8 @@ static int add_collation(CHARSET_INFO *cs) {
   Report character set initialization errors and warnings.
   Be silent by default: no warnings on the client side.
 */
-static void default_reporter(enum loglevel level MY_ATTRIBUTE((unused)),
-                             uint ecode MY_ATTRIBUTE((unused)), ...) {}
+static void default_reporter(enum loglevel level [[maybe_unused]],
+                             uint ecode [[maybe_unused]], ...) {}
 my_error_reporter my_charset_error_reporter = default_reporter;
 
 /**
@@ -358,7 +365,7 @@ void my_charset_loader_init_mysys(MY_CHARSET_LOADER *loader) {
 #define MY_MAX_ALLOWED_BUF 1024 * 1024
 #define MY_CHARSET_INDEX "Index.xml"
 
-const char *charsets_dir = NULL;
+const char *charsets_dir = nullptr;
 
 static bool my_read_charset_file(MY_CHARSET_LOADER *loader,
                                  const char *filename, myf myflags) {
@@ -397,7 +404,7 @@ char *get_charsets_dir(char *buf) {
   char *res;
   DBUG_TRACE;
 
-  if (charsets_dir != NULL)
+  if (charsets_dir != nullptr)
     strmake(buf, charsets_dir, FN_REFLEN - 1);
   else {
     if (test_if_hard_path(sharedir) ||
@@ -412,11 +419,11 @@ char *get_charsets_dir(char *buf) {
   return res;
 }
 
-CHARSET_INFO *all_charsets[MY_ALL_CHARSETS_SIZE] = {NULL};
+CHARSET_INFO *all_charsets[MY_ALL_CHARSETS_SIZE] = {nullptr};
 CHARSET_INFO *default_charset_info = &my_charset_latin1;
 
 void add_compiled_collation(CHARSET_INFO *cs) {
-  DBUG_ASSERT(cs->number < array_elements(all_charsets));
+  assert(cs->number < array_elements(all_charsets));
   all_charsets[cs->number] = cs;
   map_coll_name_to_number(cs->name, cs->number);
   map_cs_name_to_number(cs->csname, cs->number, cs->state);
@@ -431,8 +438,8 @@ static void init_available_charsets(void) {
 
   memset(&all_charsets, 0, sizeof(all_charsets));
 
-  DBUG_ASSERT(coll_name_num_map == nullptr && cs_name_pri_num_map == nullptr &&
-              cs_name_bin_num_map == nullptr);
+  assert(coll_name_num_map == nullptr && cs_name_pri_num_map == nullptr &&
+         cs_name_bin_num_map == nullptr);
   coll_name_num_map = new std::unordered_map<std::string, int>(0);
   cs_name_pri_num_map = new std::unordered_map<std::string, int>(0);
   cs_name_bin_num_map = new std::unordered_map<std::string, int>(0);
@@ -456,7 +463,7 @@ static const char *get_collation_name_alias(const char *name, char *buf,
     snprintf(buf, bufsize, "utf8mb4_da_0900_%s", name + 16);
     return buf;
   }
-  return NULL;
+  return nullptr;
 }
 
 uint get_collation_number(const char *name) {
@@ -483,7 +490,7 @@ static uint get_charset_number_internal(const char *charset_name,
     This function might be called concurrently. C++ guarantees this read-only
     access to STL container is thread-safe.
   */
-  DBUG_ASSERT(cs_name_pri_num_map != nullptr && cs_name_bin_num_map != nullptr);
+  assert(cs_name_pri_num_map != nullptr && cs_name_bin_num_map != nullptr);
   if ((cs_flags & MY_CS_PRIMARY)) {
     auto name_num_map_it = cs_name_pri_num_map->find(lower_case_name);
     if (name_num_map_it != cs_name_pri_num_map->end()) {
@@ -501,13 +508,13 @@ static uint get_charset_number_internal(const char *charset_name,
     }
   }
 
-  DBUG_ASSERT(false);
+  assert(false);
   return 0;
 }
 
 static const char *get_charset_name_alias(const char *name) {
   if (!my_strcasecmp(&my_charset_latin1, name, "utf8mb3")) return "utf8";
-  return NULL;
+  return nullptr;
 }
 
 uint get_charset_number(const char *charset_name, uint cs_flags) {
@@ -536,7 +543,7 @@ static CHARSET_INFO *get_internal_charset(MY_CHARSET_LOADER *loader_arg,
   char buf[FN_REFLEN];
   CHARSET_INFO *cs;
 
-  DBUG_ASSERT(cs_number < array_elements(all_charsets));
+  assert(cs_number < array_elements(all_charsets));
 
   if ((cs = all_charsets[cs_number])) {
     if (cs->state & MY_CS_READY) /* if CS is already initialized */
@@ -561,12 +568,12 @@ static CHARSET_INFO *get_internal_charset(MY_CHARSET_LOADER *loader_arg,
       if (!(cs->state & MY_CS_READY)) {
         if ((cs->cset->init && cs->cset->init(cs, loader_arg)) ||
             (cs->coll->init && cs->coll->init(cs, loader_arg))) {
-          cs = NULL;
+          cs = nullptr;
         } else
           cs->state |= MY_CS_READY;
       }
     } else
-      cs = NULL;
+      cs = nullptr;
 
     mysql_mutex_unlock(&THR_LOCK_charset);
   }
@@ -581,7 +588,7 @@ CHARSET_INFO *get_charset(uint cs_number, myf flags) {
 
   std::call_once(charsets_initialized, init_available_charsets);
 
-  if (cs_number >= array_elements(all_charsets)) return NULL;
+  if (cs_number >= array_elements(all_charsets)) return nullptr;
 
   my_charset_loader_init_mysys(&loader);
   cs = get_internal_charset(&loader, cs_number, flags);
@@ -613,7 +620,7 @@ CHARSET_INFO *my_collation_get_by_name(MY_CHARSET_LOADER *loader,
 
   cs_number = get_collation_number(name);
   my_charset_loader_init_mysys(loader);
-  cs = cs_number ? get_internal_charset(loader, cs_number, flags) : NULL;
+  cs = cs_number ? get_internal_charset(loader, cs_number, flags) : nullptr;
 
   if (!cs && (flags & MY_WME)) {
     char index_file[FN_REFLEN + sizeof(MY_CHARSET_INDEX)];
@@ -649,7 +656,7 @@ CHARSET_INFO *my_charset_get_by_name(MY_CHARSET_LOADER *loader,
   std::call_once(charsets_initialized, init_available_charsets);
 
   cs_number = get_charset_number(cs_name, cs_flags);
-  cs = cs_number ? get_internal_charset(loader, cs_number, flags) : NULL;
+  cs = cs_number ? get_internal_charset(loader, cs_number, flags) : nullptr;
 
   if (!cs && (flags & MY_WME)) {
     char index_file[FN_REFLEN + sizeof(MY_CHARSET_INDEX)];
@@ -687,7 +694,7 @@ bool resolve_charset(const char *cs_name, const CHARSET_INFO *default_cs,
                      const CHARSET_INFO **cs) {
   *cs = get_charset_by_csname(cs_name, MY_CS_PRIMARY, MYF(0));
 
-  if (*cs == NULL) {
+  if (*cs == nullptr) {
     *cs = default_cs;
     return true;
   }
@@ -715,7 +722,7 @@ bool resolve_collation(const char *cl_name, const CHARSET_INFO *default_cl,
                        const CHARSET_INFO **cl) {
   *cl = get_charset_by_name(cl_name, MYF(0));
 
-  if (*cl == NULL) {
+  if (*cl == nullptr) {
     *cl = default_cl;
     return true;
   }
@@ -919,7 +926,7 @@ size_t escape_quotes_for_mysql(CHARSET_INFO *charset_info, char *to,
 
 void charset_uninit() {
   for (CHARSET_INFO *cs : all_charsets) {
-    if (cs && cs->coll->uninit) {
+    if (cs && cs->coll && cs->coll->uninit) {
       cs->coll->uninit(cs);
     }
   }
