@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -52,6 +52,7 @@
 #include "my_psi_config.h"
 #include "my_sys.h"
 #include "my_thread.h"
+#include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/psi/mysql_cond.h"
 #include "mysql/psi/mysql_file.h"
 #include "mysql/psi/mysql_memory.h"
@@ -59,7 +60,6 @@
 #include "mysql/psi/mysql_rwlock.h"
 #include "mysql/psi/mysql_stage.h"
 #include "mysql/psi/mysql_thread.h"
-#include "mysql/psi/psi_base.h"
 #include "mysql/psi/psi_cond.h"
 #include "mysql/psi/psi_file.h"
 #include "mysql/psi/psi_memory.h"
@@ -142,9 +142,10 @@ bool my_init() {
   my_umask_dir = 0750; /* Default umask for new directories */
 
   /* Default creation of new files */
-  if ((str = getenv("UMASK")) != 0) my_umask = (int)(atoi_octal(str) | 0600);
+  if ((str = getenv("UMASK")) != nullptr)
+    my_umask = (int)(atoi_octal(str) | 0600);
   /* Default creation of new dir's */
-  if ((str = getenv("UMASK_DIR")) != 0)
+  if ((str = getenv("UMASK_DIR")) != nullptr)
     my_umask_dir = (int)(atoi_octal(str) | 0700);
 
   if (my_thread_global_init()) return true;
@@ -152,7 +153,7 @@ bool my_init() {
   if (my_thread_init()) return true;
 
   /* $HOME is needed early to parse configuration files located in ~/ */
-  if ((home_dir = getenv("HOME")) != 0)
+  if ((home_dir = getenv("HOME")) != nullptr)
     home_dir = intern_filename(home_dir_buff, home_dir);
 
   {
@@ -161,6 +162,8 @@ bool my_init() {
 #ifdef _WIN32
     my_win_init();
 #endif
+    MyFileInit();
+
     DBUG_PRINT("exit", ("home: '%s'", home_dir));
     return false;
   }
@@ -177,6 +180,11 @@ void my_end(int infoflag) {
 
   if (!my_init_done) return;
 
+  MyFileEnd();
+#ifdef _WIN32
+  MyWinfileEnd();
+#endif /* WIN32 */
+
   if ((infoflag & MY_CHECK_ERROR) || (info_file != stderr))
 
   { /* Test if some file is left open */
@@ -186,7 +194,6 @@ void my_end(int infoflag) {
                my_stream_opened);
       my_message_stderr(EE_OPEN_WARNING, ebuff, MYF(0));
       DBUG_PRINT("error", ("%s", ebuff));
-      my_print_open_files();
     }
   }
   my_error_unregister_all();
@@ -248,14 +255,14 @@ Voluntary context switches %ld, Involuntary context switches %ld\n",
 
   Invalid parameter handler we will use instead of the one "baked"
   into the CRT for Visual Studio.
-  The DBUG_ASSERT will catch things typically *not* caught by sanitizers,
+  The assert will catch things typically *not* caught by sanitizers,
   e.g. iterator out-of-range, but pointing to valid memory.
 */
 
 void my_parameter_handler(const wchar_t *expression, const wchar_t *function,
                           const wchar_t *file, unsigned int line,
                           uintptr_t pReserved) {
-#ifndef DBUG_OFF
+#ifndef NDEBUG
   fprintf(stderr,
           "my_parameter_handler errno %d "
           "expression: %ws  function: %ws  file: %ws, line: %d\n",
@@ -265,7 +272,7 @@ void my_parameter_handler(const wchar_t *expression, const wchar_t *function,
   //   DBUG_EXECUTE_IF("ib_export_io_write_failure_1", close(fileno(file)););
   // So ignore EBADF
   if (errno != EBADF) {
-    DBUG_ASSERT(false);
+    assert(false);
   }
 #endif
 }
@@ -415,19 +422,16 @@ static void my_win_init() {
 
   win_init_registry();
   win32_init_tcp_ip();
+
+  MyWinfileInit();
 }
 #endif /* _WIN32 */
 
-/*
-  These initializers do not work under VS2015, which we still use to build
-  Con/ODBC. And they are not really needed for our code.
-*/
+PSI_stage_info stage_waiting_for_table_level_lock = {
+    0, "Waiting for table level lock", 0, PSI_DOCUMENT_ME};
 
-PSI_stage_info stage_waiting_for_table_level_lock /*  = {
-    0, "Waiting for table level lock", 0, PSI_DOCUMENT_ME}*/;
-
-PSI_stage_info stage_waiting_for_disk_space /* = {0, "Waiting for disk space", 0,
-                                               PSI_DOCUMENT_ME} */;
+PSI_stage_info stage_waiting_for_disk_space = {0, "Waiting for disk space", 0,
+                                               PSI_DOCUMENT_ME};
 
 PSI_mutex_key key_IO_CACHE_append_buffer_lock, key_IO_CACHE_SHARE_mutex,
     key_KEY_CACHE_cache_lock, key_THR_LOCK_charset, key_THR_LOCK_heap,
@@ -508,6 +512,8 @@ static PSI_memory_info all_mysys_memory[] = {
     {&key_memory_win_PACL, "win_PACL", 0, 0, PSI_DOCUMENT_ME},
     {&key_memory_win_IP_ADAPTER_ADDRESSES, "win_IP_ADAPTER_ADDRESSES", 0, 0,
      PSI_DOCUMENT_ME},
+    {&key_memory_win_handle_info, "win_handle_to_fd_mapping", 0, 0,
+     PSI_DOCUMENT_ME},
 #endif
 
     {&key_memory_max_alloca, "max_alloca", PSI_FLAG_ONLY_GLOBAL_STAT, 0,
@@ -543,14 +549,14 @@ static PSI_memory_info all_mysys_memory[] = {
 
 #ifdef HAVE_PSI_THREAD_INTERFACE
 static PSI_thread_info all_mysys_thread[] = {
-    {&key_thread_timer_notifier, "thread_timer_notifier", PSI_FLAG_SINGLETON, 0,
-     PSI_DOCUMENT_ME}};
+    {&key_thread_timer_notifier, "thread_timer_notifier", "timer_notifier",
+     PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME}};
 #endif /* HAVE_PSI_THREAD_INTERFACE */
 
 #ifdef HAVE_PSI_INTERFACE
 void my_init_mysys_psi_keys() {
-  const char *category MY_ATTRIBUTE((unused)) = "mysys";
-  int count MY_ATTRIBUTE((unused));
+  const char *category [[maybe_unused]] = "mysys";
+  int count [[maybe_unused]];
 
 #ifdef HAVE_PSI_MUTEX_INTERFACE
   count = sizeof(all_mysys_mutexes) / sizeof(all_mysys_mutexes[0]);
