@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -21,13 +21,17 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /**
-  @file mysys/mysys_priv.h
+  @file mysys_priv.h
 */
 
 #ifndef MYSYS_PRIV_INCLUDED
 #define MYSYS_PRIV_INCLUDED
 
+#include <memory>  // std::unique_ptr
+
+#include "my_inttypes.h"  // myf
 #include "my_macros.h"
+
 #include "my_psi_config.h"
 #include "mysql/components/services/mysql_mutex_bits.h"  // for mysql_mutex_t
 #include "mysql/components/services/psi_cond_bits.h"
@@ -37,6 +41,7 @@
 #include "mysql/components/services/psi_rwlock_bits.h"  // for PSI_rwlock_key
 #include "mysql/components/services/psi_stage_bits.h"   // for PSI_stage_info
 #include "mysql/components/services/psi_thread_bits.h"  // for PSI_thread_key
+#include "mysql/psi/mysql_mutex.h"                      // for mysql_mutex_lock
 
 extern PSI_mutex_key key_IO_CACHE_append_buffer_lock, key_IO_CACHE_SHARE_mutex,
     key_KEY_CACHE_cache_lock, key_THR_LOCK_charset, key_THR_LOCK_heap,
@@ -85,6 +90,7 @@ extern PSI_memory_key key_memory_defaults;
 extern PSI_memory_key key_memory_win_SECURITY_ATTRIBUTES;
 extern PSI_memory_key key_memory_win_PACL;
 extern PSI_memory_key key_memory_win_IP_ADAPTER_ADDRESSES;
+extern PSI_memory_key key_memory_win_handle_info;
 #endif
 
 extern PSI_thread_key key_thread_timer_notifier;
@@ -97,32 +103,69 @@ extern PSI_thread_key key_thread_timer_notifier;
 #define EDQUOT (-1)
 #endif
 
+namespace mysys_priv {
+template <class SYSC, class RET>
+inline RET RetryOnEintr(SYSC &&sysc, RET err) {
+  RET r;
+  do {
+    r = sysc();
+  } while (r == err && errno == EINTR);
+  return r;
+}
+}  // namespace mysys_priv
+
 void my_error_unregister_all();
 
 #ifdef _WIN32
 #include <stdint.h>  // int64_t
 #include <sys/stat.h>
-/* my_winfile.c exports, should not be used outside mysys */
-extern File my_win_open(const char *path, int oflag);
-extern int my_win_close(File fd);
-extern int64_t my_win_read(File fd, uchar *buffer, size_t count);
-extern int64_t my_win_write(File fd, const uchar *buffer, size_t count);
-extern int64_t my_win_pread(File fd, uchar *buffer, size_t count,
-                            int64_t offset);
-extern int64_t my_win_pwrite(File fd, const uchar *buffer, size_t count,
-                             int64_t offset);
-extern int64_t my_win_lseek(File fd, int64_t pos, int whence);
-extern int my_win_chsize(File fd, int64_t newlength);
-extern FILE *my_win_fopen(const char *filename, const char *type);
-extern File my_win_fclose(FILE *file);
-extern File my_win_fileno(FILE *file);
-extern FILE *my_win_fdopen(File Filedes, const char *type);
-extern int my_win_stat(const char *path, struct _stati64 *buf);
-extern int my_win_fstat(File fd, struct _stati64 *buf);
-extern int my_win_fsync(File fd);
-extern File my_win_dup(File fd);
-extern File my_win_sopen(const char *path, int oflag, int shflag, int perm);
-extern File my_open_osfhandle(HANDLE handle, int oflag);
-#endif
+// my_winfile.cc exports, should not be used outside mysys
+File my_win_open(const char *path, int oflag);
+int my_win_close(File fd);
+int64_t my_win_pread(File fd, uchar *buffer, size_t count, int64_t offset);
+int64_t my_win_pwrite(File fd, const uchar *buffer, size_t count,
+                      int64_t offset);
+int64_t my_win_lseek(File fd, int64_t pos, int whence);
+int64_t my_win_write(File fd, const uchar *buffer, size_t count);
+int my_win_chsize(File fd, int64_t newlength);
+File my_win_fileno(FILE *file);
+FILE *my_win_fopen(const char *filename, const char *mode);
+FILE *my_win_fdopen(File Filedes, const char *mode);
+File my_win_fclose(FILE *stream);
+FILE *my_win_freopen(const char *path, const char *mode, FILE *stream);
+int my_win_fstat(File fd, struct _stati64 *buf);
+int my_win_stat(const char *path, struct _stati64 *buf);
+int my_win_fsync(File fd);
+
+void MyWinfileInit();
+void MyWinfileEnd();
+
+#endif /* _WIN32 */
+
+namespace file_info {
+
+/**
+   How was this file opened (for debugging purposes).
+   The important part is whether it is UNOPEN or not.
+*/
+enum class OpenType : char {
+  UNOPEN = 0,
+  FILE_BY_OPEN,
+  FILE_BY_CREATE,
+  STREAM_BY_FOPEN,
+  STREAM_BY_FDOPEN,
+  FILE_BY_MKSTEMP,
+  FILE_BY_O_TMPFILE
+};
+
+void CountFileOpen(OpenType pt, OpenType ct);
+void CountFileClose(OpenType ft);
+
+void RegisterFilename(File fd, const char *FileName, OpenType type_of_file);
+void UnregisterFilename(File fd);
+}  // namespace file_info
+
+void MyFileInit();
+void MyFileEnd();
 
 #endif /* MYSYS_PRIV_INCLUDED */
