@@ -1,4 +1,4 @@
-// Copyright (c) 2007, 2021, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2007, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -34,21 +34,12 @@
 # include <windows.h>
 
 #define CONNECTION_TAB  1
-
-#if MFA_ENABLED
-#define MFA_TAB         2
+#define AUTH_TAB        2
 #define METADATA_TAB    3
 #define CURSORS_TAB     4
 #define DEBUG_TAB       5
 #define SSL_TAB         6
 #define MISC_TAB        7
-#else
-#define METADATA_TAB    2
-#define CURSORS_TAB     3
-#define DEBUG_TAB       4
-#define SSL_TAB         5
-#define MISC_TAB        6
-#endif
 
 #else
 # include <gtk/gtk.h>
@@ -78,10 +69,23 @@
 #include "installer.h"
 #include "unicode_transcode.h"
 #include <sql.h>
+#include <vector>
+#include <string>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
+  /**
+    This is the function implemented by the platform-specific code.
+
+    @return TRUE if user pressed OK, FALSE if cancelled or closed
+  */
+  int ShowOdbcParamsDialog(DataSource* params, HWND ParentWnd, BOOL isPrompt);
+
+#ifdef __cplusplus
+}
+#endif
+
 
 /* Utility functions. */
 void ShowDiagnostics(SQLRETURN nReturn, SQLSMALLINT nHandleType, SQLHANDLE h);
@@ -93,10 +97,10 @@ void Disconnect(SQLHDBC hDbc, SQLHENV hEnv);
 #define MYODBC_DB_NAME_MAX 255
 
 /* Callbacks */
-SQLWCHAR *mytest(HWND hwnd, DataSource *params);
+SQLWSTRING mytest(HWND hwnd, DataSource *params);
 BOOL mytestaccept(HWND hwnd, DataSource *params);
-LIST *mygetdatabases(HWND hwnd, DataSource *params);
-LIST *mygetcharsets(HWND hwnd, DataSource *params);
+std::vector<SQLWSTRING> mygetdatabases(HWND hwnd, DataSource *params);
+std::vector<SQLWSTRING> mygetcharsets(HWND hwnd, DataSource *params);
 
 /* Sync data functions */
 void syncData(HWND hwnd, DataSource *params);
@@ -106,15 +110,9 @@ void syncTabs(HWND hwnd, DataSource *params);
 void FillParameters(HWND hwnd, DataSource *params);
 
 #define _W(string) wchar_t_as_sqlwchar((wchar_t*)string, (SQLWCHAR*)tmpbuf, \
-                                          sizeof(string) / sizeof(wchar_t))
+                                        wcslen(string))
 
 
-/**
-  This is the function implemented by the platform-specific code.
-
-  @return TRUE if user pressed OK, FALSE if cancelled or closed
-*/
-int ShowOdbcParamsDialog(DataSource *params, HWND ParentWnd, BOOL isPrompt);
 
 #ifdef _WIN32
 
@@ -274,8 +272,4 @@ void setUnsignedFieldData(gchar *widget_name, unsigned int param);
 #define SET_BOOL_TAB(framenum, name) \
   SET_CHECKED_TAB(framenum, name, params->name)
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif
