@@ -1,4 +1,4 @@
-// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -509,7 +509,7 @@ DECLARE_TEST(t_bug30983)
 DECLARE_TEST(t_driverconnect_outstring)
 {
   HDBC hdbc1;
-  SQLCHAR conn[512], conn_out[512], exp_out[512];
+  SQLCHAR conn[512], conn_out[512];
   SQLSMALLINT conn_out_len, exp_conn_out_len;
 
   sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;CHARSET=utf8",
@@ -522,10 +522,16 @@ DECLARE_TEST(t_driverconnect_outstring)
 
   ok_env(henv, SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc1));
 
-  ok_con(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, sizeof(conn), conn_out,
+  ok_con(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, SQL_NTS, conn_out,
                                  sizeof(conn_out), &conn_out_len,
                                  SQL_DRIVER_NOPROMPT));
 
+  is_num(conn_out_len, strlen((char*)conn_out));
+
+  /*
+  TODO: enable when driver builds the connection string via options
+
+  SQLCHAR exp_out[512];
   sprintf((char *)exp_out, "DSN=%s;UID=%s", mydsn, myuid);
   if (mypwd && *mypwd)
   {
@@ -546,31 +552,42 @@ DECLARE_TEST(t_driverconnect_outstring)
 
   printMessage("Output connection string: %s", conn_out);
   printMessage("Expected output   string: %s", exp_out);
-  /* save proper length for later tests */
-  exp_conn_out_len= conn_out_len;
-  is_num(conn_out_len, strlen((char *)conn_out));
-  /* TODO
+  // save proper length for later tests
   is_str(conn_out, exp_out, strlen((char *)conn_out));
   */
+  exp_conn_out_len = conn_out_len;
   ok_con(hdbc1, SQLDisconnect(hdbc1));
 
   /* test truncation */
   conn_out_len= 999;
   expect_dbc(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, SQL_NTS, conn_out,
-                                     10 * sizeof(SQLWCHAR), &conn_out_len,
+                                     10, &conn_out_len,
                                      SQL_DRIVER_NOPROMPT),
              SQL_SUCCESS_WITH_INFO);
-  is_num(conn_out_len, 9);
+
+#ifndef USE_IODBC
+  // iODBC sets the output length not larger than the buffer size
+  // even when the driver reports otherwise.
+  is(conn_out_len > 10);
+#endif
+
   is_num(check_sqlstate_ex(hdbc1, SQL_HANDLE_DBC, "01004"), OK);
   ok_con(hdbc1, SQLDisconnect(hdbc1));
 
   /* test truncation on boundary */
   conn_out_len= 999;
   expect_dbc(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, SQL_NTS, conn_out,
-                                     exp_conn_out_len * sizeof(SQLWCHAR),
+                                     exp_conn_out_len,
                                      &conn_out_len, SQL_DRIVER_NOPROMPT),
              SQL_SUCCESS_WITH_INFO);
+#ifndef USE_IODBC
+  is_num(conn_out_len, exp_conn_out_len);
+#else
+  // iODBC sets the output length reserving one byte for
+  // the termination character.
   is_num(conn_out_len, exp_conn_out_len - 1);
+#endif
+
   is_num(check_sqlstate_ex(hdbc1, SQL_HANDLE_DBC, "01004"), OK);
   ok_con(hdbc1, SQLDisconnect(hdbc1));
 
@@ -1166,14 +1183,28 @@ DECLARE_TEST(t_tls_opts)
 
   char *opts[VERSION_COUNT] = { "NO_TLS_1_3=1;", "NO_TLS_1_2=1;" };
 
-  // Info about versions actually supported by server
+  // Info about versions actually supported by server and client
 
   unsigned supported_versions = 0;
 
   for (unsigned i = 0; i < VERSION_COUNT; ++i)
-    if (strstr(buf, ver[i]) != NULL)
-      supported_versions |= (1u << i);
+  {
+      if (strstr(buf, ver[i]) != NULL)
+      {
+        char connstr[512] = "SOCKET=;SSLMODE=REQUIRED;TLS-VERSIONS=";
+        strncat(connstr, ver[i], sizeof(connstr) - strlen(connstr));
+        printf("Connection options: %s\n", connstr);
 
+        // The version is marked as supported only when client
+        // supports it as well as the server.
+        if (OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+          NULL, NULL, NULL, connstr))
+        {
+          supported_versions |= (1u << i);
+        }
+        free_basic_handles(&henv1, &hdbc1, &hstmt1);
+      }
+  }
 
   // Test disabling each of supported versions, the last iteration is
   // testing a scenario where nothing is disabled.
@@ -1268,6 +1299,7 @@ DECLARE_TEST(t_tls_opts)
 
     is(FAIL == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
       NULL, NULL, NULL, connstr));
+    free_basic_handles(&henv1, &hdbc1, &hstmt1);
   }
 
   return OK;
@@ -1405,7 +1437,237 @@ DECLARE_TEST(t_ssl_align)
   return OK;
 }
 
+const char* make_tls_str(const char *str, char *out, char *expected, char ssl_disabled)
+{
+  if (ssl_disabled)
+  {
+    sprintf(out, "SSLMODE=DISABLED;TLS-VERSIONS=%s", str);
+    *expected = 0;
+    return str;
+  }
+
+  char ver[16];
+  int num;
+  do
+  {
+    ver[0] = 0;
+    num = 0;
+    // Skip a leading comma
+    if (*str == ',')
+      ++str;
+
+    while (str && *str && *str != ',')
+    {
+      ver[num] = *str;
+      ++num;
+      ++str;
+    }
+    ver[num] = 0;
+    // Skip TLSv1 (len < 6) and TLSv1.1
+  } while (*str && (strlen((const char*)ver) < 6 ||
+                    strncmp((const char*)ver, "TLSv1.1", 7) == 0));
+
+  sprintf(out, "SSLMODE=REQUIRED;TLS-VERSIONS=%s;NO_TLS_%c_%c=1",
+          ver, ver[num - 3], ver[num - 1]);
+  memcpy(expected, ver, num + 1);
+
+  if (*str == ',')
+    return ++str;
+
+  return NULL;
+}
+
+DECLARE_TEST(t_tls_versions)
+{
+  ok_sql(hstmt, "SHOW VARIABLES LIKE 'tls_version'");
+  ok_stmt(hstmt, SQLFetch(hstmt));
+  char tls_versions_list[255] = { 0 };
+
+  my_fetch_str(hstmt, tls_versions_list, 2);
+  SQLFetch(hstmt);
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+
+  char ssl_disabled = 1;
+  char con_str[255];
+  char exp_result[16];
+  const char *list = tls_versions_list;
+  while((list = make_tls_str(list, con_str, exp_result, ssl_disabled)))
+  {
+    SQLCHAR buf[1024] = { 0 };
+    ssl_disabled = 0;
+    int connect_res = alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+                                          NULL, NULL, NULL, con_str);
+    if (connect_res == SQL_SUCCESS)
+    {
+      ok_sql(hstmt1, "SHOW STATUS LIKE 'Ssl_version'");
+
+      ok_stmt(hstmt1, SQLFetch(hstmt1));
+      ok_stmt(hstmt1, SQLGetData(hstmt1, 2, SQL_C_CHAR, buf, sizeof(buf), NULL));
+      is_str(exp_result, buf, strlen(exp_result));
+    }
+    else
+    {
+      is(OK == check_errmsg_ex(hdbc1, SQL_HANDLE_DBC,
+        "SSL connection error: No valid TLS version available"))
+    }
+
+    free_basic_handles(&henv1, &hdbc1, &hstmt1);
+  }
+
+  return OK;
+}
+
+DECLARE_TEST(t_bug107307)
+{
+  SQLCHAR buff[128];
+  SQLRETURN rc;
+  double value = 0;
+  SQLLEN ind = 0;
+
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, NULL,
+    NULL, NULL, "NO_CACHE=1");
+
+  ok_sql(hstmt1, "DROP TABLE IF EXISTS t_bug107307");
+  ok_sql(hstmt1, "CREATE TABLE t_bug107307 (num0 DOUBLE)");
+
+  double expected[] = {12.3, -12.3, 15.7, -15.7, 3.5, -3.5, 0, 0, 10};
+  ok_sql(hstmt1, "INSERT INTO t_bug107307 VALUES (12.3), (-12.3), (15.7),"
+    "(-15.7), (3.5), (-3.5), (0), (NULL), (10)");
+
+  ok_stmt(hstmt1, SQLPrepare(hstmt1, "SELECT * FROM t_bug107307", SQL_NTS));
+  ok_stmt(hstmt, SQLExecute(hstmt1));
+
+  ok_stmt(hstmt1, SQLBindCol(hstmt1, 1, SQL_C_DOUBLE, (SQLPOINTER)&value, 0, &ind));
+
+  for (int i = 0; ; i++)
+  {
+    rc = SQLFetch(hstmt1);
+    if (rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO)
+    {
+      if (ind <= 0)
+      {
+        printf("Record %d: %s\n", i + 1, "NULL");
+      }
+      else
+      {
+        printf("Record %d: %f\n", i + 1, value);
+      }
+      is(value == expected[i]);
+    }
+    else
+    {
+      if (rc != SQL_NO_DATA)
+      {
+        printf("Error\n");
+      }
+      else
+      {
+        break;
+      }
+    }
+  }
+
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
+  ok_sql(hstmt1, "DROP TABLE IF EXISTS t_bug107307");
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+  return OK;
+}
+
+/*
+  WL #14880 - New ODBC connection options to specify CRL and
+  CRL Path for SSL
+*/
+DECLARE_TEST(t_ssl_crl)
+{
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  char *conn_strs[][3] = {
+    // Connection string, return value (1 - error, 0 - success), Encryption
+    {"ssl-mode=REQUIRED;ssl-crl=non-existing.pem", "\1", "\0"},
+    {"ssl-mode=REQUIRED;ssl-crlpath=invalid-path", "\0", "\1"},
+    {"ssl-mode=DISABLED;ssl-crl=non-existing.pem", "\0", "\0"},
+    {"ssl-mode=DISABLED;ssl-crlpath=invalid-path", "\0", "\0"}
+  };
+
+  for (int i = 0; i < 4; ++i)
+  {
+    SQLCHAR buf[1024] = { 0 };
+    int connect_res = alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+      NULL, NULL, NULL, conn_strs[i][0]);
+
+    is_num(connect_res, conn_strs[i][1][0]);
+
+    if (connect_res == SQL_SUCCESS)
+    {
+      // If connected check if encryption is used.
+      ok_sql(hstmt1, "SHOW STATUS LIKE 'Ssl_version'");
+      ok_stmt(hstmt1, SQLFetch(hstmt1));
+      ok_stmt(hstmt1, SQLGetData(hstmt1, 2, SQL_C_CHAR, buf, sizeof(buf), NULL));
+      if(conn_strs[i][2][0])
+      {
+        // Encryption is used
+        is(strlen(buf) > 0);
+      }
+      else
+      {
+        // Encryption is not used
+        is(strlen(buf) == 0);
+      }
+    }
+    else
+    {
+      // If not connected check the error message
+      is(OK == check_errmsg_ex(hdbc1, SQL_HANDLE_DBC,
+        "SSL connection error: SSL_CTX_set_default_verify_paths failed"));
+    }
+
+    free_basic_handles(&henv1, &hdbc1, &hstmt1);
+  }
+
+  return OK;
+}
+
+/*
+  Bug #34786939 - Adding ODBC ANSI 64-bit System DSN connection causes
+  error 01004
+*/
+DECLARE_TEST(t_bug34786939_out_trunc)
+{
+  SQLHDBC hdbc1 = NULL;
+  ok_env(henv, SQLAllocConnect(henv, &hdbc1));
+  SQLCHAR *connstr = make_conn_str(NULL, NULL, NULL, NULL, NULL, 0);
+  SQLSMALLINT out_str_len = -1;
+
+  // Give NULL for out string, but non-NULL for out string length.
+  SQLRETURN rc = SQLDriverConnect(hdbc1, NULL, connstr, SQL_NTS,
+    NULL, 0, &out_str_len, SQL_DRIVER_NOPROMPT);
+  // It must not return anything except SQL_SUCCESS.
+  is_num(rc, SQL_SUCCESS);
+
+  SQLDisconnect(hdbc1);
+
+  SQLCHAR out_str[10] = { 0 };
+
+  // Give a very small buffer out string and non-NULL for out string length.
+  rc = SQLDriverConnect(hdbc1, NULL, connstr, SQL_NTS,
+    out_str, 10, &out_str_len, SQL_DRIVER_NOPROMPT);
+  // It must not return anything except SQL_SUCCESS_WITH_INFO.
+  is_num(rc, SQL_SUCCESS_WITH_INFO);
+
+  SQLDisconnect(hdbc1);
+  SQLFreeConnect(hdbc1);
+
+  return OK;
+}
+
 BEGIN_TESTS
+  ADD_TEST(t_driverconnect_outstring)
+  ADD_TEST(t_bug34786939_out_trunc)
+  ADD_TEST(t_ssl_crl)
+  ADD_TEST(t_bug107307)
+  ADD_TEST(t_tls_versions)
   ADD_TEST(t_tls_opts)
   ADD_TEST(t_ssl_mode)
   ADD_TEST(my_basics)
@@ -1414,15 +1676,14 @@ BEGIN_TESTS
   ADD_TEST(t_nativesql)
 #ifndef NO_DRIVERMANAGER
   ADD_TEST(t_reconnect)
-  // ADD_TEST(t_bug19823) TODO: make sure it runs with the up-to-date UnixODBC
+  ADD_TEST(t_bug19823)
 #endif
   ADD_TEST(charset_utf8)
-  // ADD_TEST(charset_gbk) TODO: Fix
+  // ADD_TEST(charset_gbk) TODO: fix
   ADD_TEST(t_bug7445)
   ADD_TEST(t_bug30774)
   ADD_TEST(t_bug30840)
   ADD_TEST(t_bug30983)
-  ADD_TEST(t_driverconnect_outstring)
   ADD_TEST(setnames)
   ADD_TEST(setnames_conn)
   ADD_TEST(sqlcancel)
