@@ -1,4 +1,4 @@
-// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -497,14 +497,10 @@ DECLARE_TEST(t_bug16920750)
 
 
 BEGIN_TESTS
-  // ADD_TEST(my_foreign_keys)
+  ADD_TEST(my_foreign_keys)
   ADD_TEST(my_no_keys)
   ADD_TEST(t_bug16920750)
 END_TESTS
 
-myoption &= ~(1 << 30);
-RUN_TESTS_ONCE
-myoption |= (1 << 30);
-testname_suffix= "_no_i_s";
 RUN_TESTS
 
