@@ -1,4 +1,4 @@
-// Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2020, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -26,6 +26,7 @@
 // along with this program; if not, write to the Free Software Foundation, Inc.,
 // 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
+#include <algorithm>
 #include <string>
 #include <vector>
 #include <fstream>
@@ -36,7 +37,7 @@
 #include <vector>
 #include <stdlib.h>
 
-#include "odbctap.h"
+#include "odbc_util.h"
 
 #ifdef _WIN32
 #pragma warning (disable : 4996)
@@ -634,14 +635,12 @@ int run_func_tests(test_params &par)
     return OK;
   };
 
-  for (int i = 0; i < 2; ++i)
   {
     std::string connstr = "NO_CATALOG=";
     connstr.append(par.no_catalog ? "1" : "0");
     connstr.append(";NO_SCHEMA=");
     connstr.append(par.no_schema ? "1" : "0");
 
-    connstr.append( i ? ";NO_I_S=1" : ";NO_I_S=0");
     std::cout << "Connectin parameters [ " << connstr << " ]" << std::endl <<
                  "CATALOG: [ " << (par.catalog_name ? par.catalog_name : "NULL") <<
                  " ] SCHEMA: [ " << (par.schema_name ? par.schema_name : "NULL")
@@ -843,88 +842,273 @@ DECLARE_TEST(t_wl14490)
   return OK;
 }
 
-#define ok_dsn(C) if (!C) { \
-  std::cout << "Call failed: " << #C << " in line " << __LINE__ << ":" << \
-   __FILE__ << std::endl; \
-  return FAIL; \
-}
-/*
-  Deprecate NO_I_S connection option
-*/
-DECLARE_TEST(t_wl14586)
+DECLARE_TEST(t_wl15114)
 {
-  SQLHDBC hdbc1;
-  const char *dsn_name = "wl14586";
-  SQLCHAR drv_info[1024];
-  std::string drv_nobrackets;
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
 
-  if (mydriver[0] == '{' && strlen((const char*)mydriver) > 2)
-    drv_nobrackets = std::string((const char*)(mydriver + 1),
-                                 strlen((const char*)mydriver) - 2);
-  else
-    drv_nobrackets = (const char*)mydriver;
+  // The list of ciphers even with TLSv1.2 still contains the TLSv1.3
+  // ciphersuites and currently we do not have a connection option that
+  // selects ciphersuites. So this test is limited to at most TLSv1.2
+  // but when the missing option is added the test should be extended to
+  // cover ciphersuites as well.
+  std::vector<std::string> suites = {
+    "TLS_AES_128_GCM_SHA256",
+    "TLS_AES_256_GCM_SHA384",
+    "TLS_CHACHA20_POLY1305_SHA256",
+    "TLS_AES_128_CCM_SHA256",
+    "TLS_AES_128_CCM_8_SHA256"
+  };
 
-  ok_env(henv, SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc1));
+  std::vector<std::string> approved_ciphers = {
+    "ECDHE-RSA-AES256-GCM-SHA384",
+    "DHE-RSA-AES128-GCM-SHA256",
+    "DHE-RSA-AES256-GCM-SHA384",
+    "ECDHE-RSA-AES128-SHA256",
+    "ECDHE-RSA-AES256-SHA384",
+    "DHE-RSA-AES256-SHA256",
+    "DHE-RSA-AES128-SHA256",
+    "ECDHE-RSA-AES128-SHA",
+    "ECDHE-RSA-AES256-SHA",
+    "DHE-RSA-AES128-SHA",
+    "DHE-RSA-AES256-SHA",
+    "AES128-GCM-SHA256",
+    "AES256-GCM-SHA384",
+    "AES128-SHA256",
+    "CAMELLIA256-SHA",
+    "CAMELLIA128-SHA"
+  };
 
-  SQLCHAR *connstr = make_conn_str((SQLCHAR*)USE_DRIVER,
-                                nullptr, nullptr, nullptr,
-                                (const SQLCHAR*)"NO_I_S=1", 0);
+  std::vector<std::string> ciphers;
 
-  SQLRETURN rc = SQLDriverConnect(hdbc1, NULL, connstr, SQL_NTS, nullptr, 0,
-                                  nullptr, SQL_DRIVER_NOPROMPT);
-  // Driver must report the option deprecation
-  is(rc == SQL_SUCCESS_WITH_INFO);
-  print_diag(rc, SQL_HANDLE_DBC, hdbc1, "Expected Info inSQLDriverConnect():",
-             __FILE__, __LINE__);
-  ok_con(hdbc1, SQLDisconnect(hdbc1));
+  // Connect first time to get the list of available ciphers
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+    NULL, NULL, NULL, (SQLCHAR*)"ssl-mode=REQUIRED;tls-versions=TLSv1.2;"));
 
-  ok_dsn(SQLGetPrivateProfileString((LPCSTR)drv_nobrackets.c_str(),
-                                    (LPCSTR)"DRIVER", (LPCSTR)"",
-                                    (LPSTR)drv_info, 1023, "odbcinst.ini"));
+  auto get_status_var = [](SQLHSTMT stmt, std::string var, std::string &result) {
+    std::string buf;
+    const size_t bufsize = 32768;
+    buf.reserve(bufsize);
+    const char *data = buf.data();
+    std::string query = "SHOW SESSION STATUS LIKE '" + var + "'";
 
-  SQLRemoveDSNFromIni((LPCSTR)dsn_name);
+    ok_stmt(stmt, SQLExecDirect(stmt, (SQLCHAR*)query.c_str(), SQL_NTS));
+    ok_stmt(stmt, SQLFetch(stmt));
+    ok_stmt(stmt, SQLGetData(stmt, 2, SQL_C_CHAR, (SQLPOINTER)data, bufsize, NULL));
+    is(SQL_NO_DATA == SQLFetch(stmt));
+    ok_stmt(stmt, SQLFreeStmt(stmt, SQL_CLOSE));
+    result = data;
+    return SQL_SUCCESS;
+  };
 
-  if (!SQLWriteDSNToIni((LPCSTR)dsn_name, (LPCSTR)drv_nobrackets.c_str()))
+  std::string str_cipher_list;
+  is(SQL_SUCCESS == get_status_var(hstmt1, "Ssl_cipher_list", str_cipher_list));
+
+  // Break the result string into individual cipher names
+  size_t pos = 0, prev = 0;
+  while ((pos = str_cipher_list.find(':', prev)) != std::string::npos)
   {
-    std::cout << "Could not create a new DSN named " << dsn_name << std::endl;
+    const std::string item = str_cipher_list.substr(prev, pos - prev);
+    if (std::find(suites.begin(), suites.end(), item) == suites.end() &&
+      std::find(approved_ciphers.begin(),
+                approved_ciphers.end(), item) != approved_ciphers.end())
+    {
+      // Only add cipher names, but not suites
+      ciphers.push_back(item);
+    }
+    prev = pos + 1;
+  }
+
+  if (ciphers.size() < 3)
+  {
+    free_basic_handles(&henv1, &hdbc1, &hstmt1);
+    std::cout << "Server does not have enough ciphers to test the functionality\n";
     return FAIL;
   }
 
-  char port[16] = { 0 };
-  if (myport)
-    snprintf(port, sizeof(port), "%d", myport);
+  // Get the default cipher name used when the cipher is not specified
+  std::string str_cipher;
+  is(SQL_SUCCESS == get_status_var(hstmt1, "Ssl_cipher", str_cipher));
 
-  std::vector<std::vector<const char*>> params = {
-    {"SERVER",      (const char*)myserver},
-    {"USER",        (const char*)myuid   },
-    {"PWD",         (const char*)mypwd   },
-    {"DB",          (const char*)mydb    },
-    {"SOCKET",      (const char*)mysock  },
-    {"PORT",        (const char*)port    },
-    {"NO_I_S",      (const char*)"1"     }
-  };
+  srand(time(0));
+  std::string new_str_ciphers[2];
+  int idx = 0;
 
-  for (auto p : params)
+  // Get 1st cipher, which is not the same as used in the connection
+  do
   {
-    if (p[1] && *p[1])
-      ok_dsn(SQLWritePrivateProfileString((LPCSTR)dsn_name, p[0], p[1], "odbc.ini"));
+    idx = rand() % ciphers.size();
+  } while (str_cipher.compare(ciphers[idx]) == 0);
+  new_str_ciphers[0] = ciphers[idx];
+
+  // Get 2st cipher, which is not the same as used in the connection
+  do
+  {
+    idx = rand() % ciphers.size();
+  } while (str_cipher.compare(ciphers[idx]) == 0 ||
+    new_str_ciphers[0].compare(ciphers[idx]) == 0);
+
+  new_str_ciphers[1] = ciphers[idx];
+  std::cout << "Selected ciphers: " << new_str_ciphers[0] <<
+    ", " << new_str_ciphers[1] << std::endl;
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
+
+  // Check that when ssl-mode=DISABLED the encryption is not implicitly used
+  for (std::string mode :{ "REQUIRED", "DISABLED" })
+  {
+    // Check that the option names aliases work
+    for (std::string name : { "ssl-cipher", "SSLCIPHER" })
+    {
+      std::string new_opts = std::string("ssl-mode=" + mode +
+        ";tls-versions=TLSv1.2;") + name + "=" +  new_str_ciphers[0] +
+        ":" + new_str_ciphers[1];
+
+      is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+                                            NULL, NULL, NULL,
+                                            (SQLCHAR*)new_opts.c_str()));
+
+      str_cipher = "";
+      is(SQL_SUCCESS == get_status_var(hstmt1, "Ssl_cipher", str_cipher));
+      std::cout << "Current cipher is: " << str_cipher << std::endl;
+      if (mode.compare("REQUIRED") == 0)
+      {
+        // Must be one of previously selected ciphers
+        is(str_cipher.compare(new_str_ciphers[0]) == 0 ||
+           str_cipher.compare(new_str_ciphers[1]) == 0);
+      }
+      else
+      {
+        is(str_cipher.empty());
+      }
+
+      free_basic_handles(&henv1, &hdbc1, &hstmt1);
+    }
   }
 
-  rc = SQLConnect(hdbc1, (SQLCHAR*)dsn_name, SQL_NTS, nullptr, 0, nullptr, 0);
-  // Driver must report the option deprecation
-  is(rc == SQL_SUCCESS_WITH_INFO);
-  print_diag(rc, SQL_HANDLE_DBC, hdbc1, "Expected Info in SQLConnect():",
-             __FILE__, __LINE__);
-  ok_con(hdbc1, SQLDisconnect(hdbc1));
-  SQLRemoveDSNFromIni("wl14586");
   return OK;
 }
 
+/*
+  Bug #33986051 MySQL ODBC Connector hangs in SQLDriverConnect
+  if password contains '}'
+*/
+DECLARE_TEST(t_password_hang)
+{
+
+  std::vector<std::pair<std::string, std::string>> pwds = {
+    {"{some}password", "{{some}}password}"},
+    {"some{password", "{some{password}"},
+    {"{}some{password}", "{{}}some{password}}}"},
+    {"some}p}assword", "{some}}p}}assword}"},
+    {"}somepassword{", "{}}somepassword{}"},
+  };
+
+  try
+  {
+    for (auto p : pwds)
+    {
+      odbc::temp_user usr(hstmt, p.first);
+      odbc::xstring grant = "GRANT ALL PRIVILEGES ON " + odbc::xstring(mydb) +
+        ".* TO " + usr.username;
+      odbc::sql(hstmt, grant);
+      odbc::connection con(nullptr, usr.name,  p.second, nullptr);
+    }
+  }
+  ENDCATCH;
+}
+
+/*
+  Testing the ability of the driver to set the connection collation
+  (and character set) independent to the result character set, which
+  must always be utf8mb4.
+*/
+DECLARE_TEST(t_collation_set)
+{
+  /*
+    The binary is casted to different strings upper and lower case.
+    The comparison must be '1' for case insensitive collations and '0'
+    for case sensitive collations.
+  */
+  odbc::xstring select = "select cast(0xF2E5EAF1F2 as CHAR) = cast(0xD2C5CAD1D2 as CHAR)";
+  try
+  {
+    {
+      /*
+        Setting a connection collation belonging to a single byte charset.
+        The driver should still be able to get SQLWCHAR emoji string as normal.
+      */
+      odbc::connection con(nullptr, nullptr, nullptr, nullptr,
+        "INITSTMT=SET collation_connection=cp1251_ukrainian_ci;CHARSET=utf8mb4");
+      odbc::HSTMT hstmt(con);
+
+      odbc::table tab(hstmt, "test_data", "col1 char(8) character set utf8mb4");
+      tab.insert("(0xf09f988a)");
+
+      ok_stmt(hstmt, SQLPrepareW(hstmt, W(L"SELECT * FROM test_data"), SQL_NTS));
+      odbc::stmt_execute(hstmt);
+      ok_stmt(hstmt, SQLFetch(hstmt));
+
+      SQLLEN len = 0;
+      if (unicode_driver == 1)
+      {
+        SQLWCHAR wbuff[4] = { 0 };
+        ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_WCHAR, wbuff, sizeof(wbuff), &len));
+        SQLWCHAR smile[2] = { 0xD83D, 0xDE0A };
+        is_num(smile[0], wbuff[0]);
+        is_num(smile[1], wbuff[1]);
+      }
+      else
+      {
+        // ANSI driver cannot return SQLWCHAR data correctly
+        // because the ODBC Driver Manager wraps calls using
+        // SQL_CHAR type. So, each ANSI char will be converted
+        // to SQLWCHAR, which is incorrect.
+        // Therefore, verify that the initial data is returned
+        // (0xf09f988a)
+        SQLCHAR buff[8] = { 0 };
+        ok_stmt(hstmt, SQLGetData(hstmt, 1, SQL_CHAR, buff, sizeof(buff), &len));
+        is_num(0xf0, buff[0]);
+        is_num(0x9f, buff[1]);
+        is_num(0x98, buff[2]);
+        is_num(0x8a, buff[3]);
+      }
+
+      expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+      ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+
+      /*
+        Run the query with the comparison.
+        The result should be 1 - the strings are equal.
+      */
+      odbc::sql(hstmt, select);
+      ok_stmt(hstmt, SQLFetch(hstmt));
+      is_num(1, my_fetch_int(hstmt, 1));
+      expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+    }
+
+    {
+      /*
+        Setting the binary collation, so the same data comparison
+        inside the same query will return a negative result.
+      */
+      odbc::connection con(nullptr, nullptr, nullptr, nullptr,
+        "INITSTMT=SET collation_connection=cp1251_bin");
+      odbc::HSTMT hstmt(con);
+
+      odbc::sql(hstmt, select);
+      ok_stmt(hstmt, SQLFetch(hstmt));
+      is_num(0, my_fetch_int(hstmt, 1));
+      expect_stmt(hstmt, SQLFetch(hstmt), SQL_NO_DATA_FOUND);
+    }
+  }
+  ENDCATCH;
+}
 
 BEGIN_TESTS
+  ADD_TEST(t_collation_set)
+  ADD_TEST(t_password_hang)
+  ADD_TEST(t_wl15114)
   ADD_TEST(t_wl13883)
   ADD_TEST(t_wl14490)
-  ADD_TEST(t_wl14586)
   ADD_TEST(t_wl14362)
 END_TESTS
 
