@@ -1,4 +1,4 @@
-// Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2003, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -1293,9 +1293,7 @@ BEGIN_TESTS
   ADD_TEST(t_prep_catalog)
   ADD_TEST(t_sps)
   ADD_TEST(t_prepare)
-#ifndef USE_IODBC
   ADD_TEST(t_prepare1)
-#endif
   ADD_TEST(tmysql_bindcol)
   ADD_TEST(tmysql_bindparam)
   ADD_TEST(t_acc_update)
@@ -1304,7 +1302,7 @@ BEGIN_TESTS
   ADD_TEST(t_bug67702)
   ADD_TEST(t_bug68243)
   ADD_TEST(t_bug67920)
-  // ADD_TODO(t_bug31667091) Re-enable when server fixes sql_select_limit
+  ADD_TODO(t_bug31667091)
 END_TESTS
 
 
