@@ -1,30 +1,30 @@
-// Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved. 
-// 
-// This program is free software; you can redistribute it and/or modify 
-// it under the terms of the GNU General Public License, version 2.0, as 
-// published by the Free Software Foundation. 
-// 
-// This program is also distributed with certain software (including 
-// but not limited to OpenSSL) that is licensed under separate terms, 
-// as designated in a particular file or component or in included license 
-// documentation. The authors of MySQL hereby grant you an 
-// additional permission to link the program and your derivative works 
-// with the separately licensed software that they have included with 
-// MySQL. 
-// 
-// Without limiting anything contained in the foregoing, this file, 
-// which is part of <MySQL Product>, is also subject to the 
-// Universal FOSS Exception, version 1.0, a copy of which can be found at 
-// http://oss.oracle.com/licenses/universal-foss-exception. 
-// 
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of 
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
-// See the GNU General Public License, version 2.0, for more details. 
-// 
-// You should have received a copy of the GNU General Public License 
-// along with this program; if not, write to the Free Software Foundation, Inc., 
-// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA 
+// Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License, version 2.0, as
+// published by the Free Software Foundation.
+//
+// This program is also distributed with certain software (including
+// but not limited to OpenSSL) that is licensed under separate terms,
+// as designated in a particular file or component or in included license
+// documentation. The authors of MySQL hereby grant you an
+// additional permission to link the program and your derivative works
+// with the separately licensed software that they have included with
+// MySQL.
+//
+// Without limiting anything contained in the foregoing, this file,
+// which is part of <MySQL Product>, is also subject to the
+// Universal FOSS Exception, version 1.0, a copy of which can be found at
+// http://oss.oracle.com/licenses/universal-foss-exception.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+// See the GNU General Public License, version 2.0, for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 
 #include "odbctap.h"
@@ -156,14 +156,14 @@ DECLARE_TEST(t_bug24131)
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
   ok_stmt(hstmt, SQLPrepare(hstmt, (SQLCHAR *)"show create table bug24131", SQL_NTS));
 
-  ok_stmt(hstmt, SQLDescribeCol(hstmt, 2, buff, sizeof(buff), NULL, NULL,
-                                &colSize, NULL, NULL));
-
   ok_stmt(hstmt, SQLBindCol(hstmt,2,SQL_C_BINARY, buff, 1024, &boundLen));
 
   /* Note: buff has '2.0', but len is still 0! */
   ok_stmt(hstmt, SQLExecute(hstmt));
 
+  ok_stmt(hstmt, SQLDescribeCol(hstmt, 2, buff, sizeof(buff), NULL, NULL,
+                                &colSize, NULL, NULL));
+
   ok_stmt(hstmt, SQLExtendedFetch(hstmt, SQL_FETCH_NEXT, 1, &count, &status));
 
   if (sizeof(SQLLEN) == 4)
@@ -723,11 +723,8 @@ DECLARE_TEST(t_prefetch)
     is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
                                         NULL, NULL, NULL, "MULTI_STATEMENTS=1"));
 
-    ok_stmt(hstmt1, SQLPrepare(hstmt1, "select* from b_prefecth;\
-                                        select * from b_prefecth where i < 7; ",
-                              SQL_NTS));
-
-    ok_stmt(hstmt1, SQLExecute(hstmt1));
+    ok_sql(hstmt1, "select* from b_prefecth;\
+                   select * from b_prefecth where i < 7; ");
 
     is_num(7, myrowcount(hstmt1));
 
@@ -1224,6 +1221,7 @@ DECLARE_TEST(t_bug17311065)
   is_str(sqlstate, "07009", 5);
 
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
+  ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_RESET_PARAMS));
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug17311065");
 
   return OK;
@@ -1426,26 +1424,24 @@ DECLARE_TEST(t_bug28098219)
 }
 
 BEGIN_TESTS
+  // ADD_TEST(t_bug11766437) TODO: fix Solaris Sparc
   ADD_TEST(t_bug32420)
   ADD_TEST(t_bug34575)
   ADD_TEST(t_bug24131)
   ADD_TEST(t_bug36069)
   ADD_TEST(t_bug41942)
   ADD_TEST(t_bug39644)
-  // ADD_TEST(t_bug32821) TODO: Fix
+  // ADD_TEST(t_bug32821) TODO: Fix when SQL_INTEGER is a parameter for BIT column
   ADD_TEST(t_bug34271)
   ADD_TEST(t_bug32684)
   ADD_TEST(t_bug55024)
   ADD_TEST(t_bug56677)
   ADD_TEST(t_desccol_before_exec)
   ADD_TEST(t_bug62657)
-#ifndef USE_IODBC
   ADD_TEST(t_row_status)
-#endif
   ADD_TEST(t_prefetch)
   ADD_TEST(t_bug17386788)
   ADD_TOFIX(t_outparams)
-  // ADD_TEST(t_bug11766437) TODO: Fix
   // ADD_TEST(t_varbookmark) TODO: Fix
 #ifdef _UNIX_
   // ADD_TEST(t_bookmark) TODO: Fix
