@@ -1,4 +1,4 @@
-// Copyright (c) 2021, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2021, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -32,6 +32,8 @@
 #include <string>
 #include <memory>
 #include <cstdio>
+#include <fstream>
+#include <iostream>
 
 #define X(s) odbc::xstring(s)
 
@@ -346,6 +348,7 @@ struct table
   xstring database = nullptr;
   xstring table_name;
   xstring qualifier;
+  xstring table_extra = nullptr;
 
   void drop()
   {
@@ -354,7 +357,7 @@ struct table
 
   void create(xstring def)
   {
-    sql(hstmt, "CREATE TABLE `" + qualifier + "`(" + def + ")");
+    sql(hstmt, "CREATE TABLE `" + qualifier + "`(" + def + ")" + table_extra);
   }
 
   void insert(xstring vals)
@@ -362,8 +365,9 @@ struct table
     sql(hstmt, "INSERT INTO `" + qualifier + "` VALUES " + vals);
   }
 
-  table(SQLHSTMT stmt, xstring db, xstring tab, xstring def) : hstmt(stmt),
-      database(db), table_name(tab)
+  table(SQLHSTMT stmt, xstring db, xstring tab, xstring def,
+        xstring extra = nullptr) : hstmt(stmt),
+      database(db), table_name(tab), table_extra(extra)
   {
     qualifier = (!database.empty() ? database + "`.`" : "" ) + table_name;
     drop();
@@ -817,10 +821,10 @@ struct temp_user
   SQLHSTMT hstmt = nullptr;
   odbc::xstring username;  // full username@host
   odbc::xstring name;      // username without host
-  odbc::xstring pass = "temp_pass";
+  odbc::xstring pass;
   odbc::xbuf buf;
 
-  temp_user(SQLHSTMT stmt) : hstmt(stmt), buf(1024)
+  temp_user(SQLHSTMT stmt, odbc::xstring pwd) : hstmt(stmt), buf(1024), pass(pwd)
   {
     odbc::select_one_str(hstmt, "SELECT USER()", buf, 1);
     username = "temp_user_" + odbc::xstring(buf);
@@ -830,6 +834,9 @@ struct temp_user
     name = username.substr(0, username.find('@', 0));
   }
 
+  temp_user(SQLHSTMT stmt) : temp_user(stmt, "temp_pass")
+  { }
+
   ~temp_user()
   {
     odbc::sql(hstmt, "DROP USER " + username, true);
