@@ -1,4 +1,4 @@
-// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2007, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -38,6 +38,8 @@
  *    Add to the configuration GUIs
  *
  */
+
+#include <vector>
 #include "stringutil.h"
 #include "installer.h"
 
@@ -189,7 +191,6 @@ static SQLWCHAR W_DFLT_BIGINT_BIND_STR[]=
   {'D','F','L','T','_','B','I','G','I','N','T','_','B','I','N','D','_','S','T','R',0};
 static SQLWCHAR W_CLIENT_INTERACTIVE[]=
   {'I','N','T','E','R','A','C','T','I','V','E',0};
-static SQLWCHAR W_NO_I_S[]= {'N','O','_','I','_','S',0};
 static SQLWCHAR W_PREFETCH[]= {'P','R','E','F','E','T','C','H',0};
 static SQLWCHAR W_NO_SSPS[]= {'N','O','_','S','S','P','S',0};
 static SQLWCHAR W_CAN_HANDLE_EXP_PWD[]=
@@ -219,6 +220,15 @@ static SQLWCHAR W_LOAD_DATA_LOCAL_DIR[] =
 { 'L', 'O', 'A', 'D', '_', 'D', 'A', 'T', 'A', '_', 'L', 'O', 'C', 'A', 'L', '_', 'D', 'I', 'R', 0 };
 static SQLWCHAR W_OCI_CONFIG_FILE[] =
 { 'O', 'C', 'I', '_', 'C', 'O', 'N', 'F', 'I', 'G', '_', 'F', 'I', 'L', 'E', 0 };
+static SQLWCHAR W_AUTHENTICATION_KERBEROS_MODE[] =
+{ 'A','U','T','H','E','N','T','I','C','A','T','I','O','N','-',
+  'K','E','R','B','E','R','O','S','-','M','O','D','E', 0};
+static SQLWCHAR W_TLS_VERSIONS[] =
+{ 'T', 'L', 'S', '-', 'V', 'E', 'R', 'S', 'I', 'O', 'N', 'S', 0 };
+static SQLWCHAR W_SSL_CRL[] =
+{ 'S', 'S', 'L', '-', 'C', 'R', 'L', 0 };
+static SQLWCHAR W_SSL_CRLPATH[] =
+{ 'S', 'S', 'L', '-', 'C', 'R', 'L', 'P', 'A', 'T', 'H', 0};
 
 /* DS_PARAM */
 /* externally used strings */
@@ -252,13 +262,14 @@ SQLWCHAR *dsnparams[]= {W_DSN, W_DRIVER, W_DESCRIPTION, W_SERVER,
                         W_ZERO_DATE_TO_MIN, W_MIN_DATE_TO_ZERO,
                         W_MULTI_STATEMENTS, W_COLUMN_SIZE_S32,
                         W_NO_BINARY_RESULT, W_DFLT_BIGINT_BIND_STR,
-                        W_CLIENT_INTERACTIVE, W_NO_I_S, W_PREFETCH, W_NO_SSPS,
+                        W_CLIENT_INTERACTIVE, W_PREFETCH, W_NO_SSPS,
                         W_CAN_HANDLE_EXP_PWD, W_ENABLE_CLEARTEXT_PLUGIN,
                         W_GET_SERVER_PUBLIC_KEY, W_ENABLE_DNS_SRV, W_MULTI_HOST,
                         W_SAVEFILE, W_RSAKEY, W_PLUGIN_DIR, W_DEFAULT_AUTH,
                         W_NO_TLS_1_2, W_NO_TLS_1_3,
                         W_SSLMODE, W_NO_DATE_OVERFLOW, W_LOAD_DATA_LOCAL_DIR,
-                        W_OCI_CONFIG_FILE};
+                        W_OCI_CONFIG_FILE, W_AUTHENTICATION_KERBEROS_MODE,
+                        W_TLS_VERSIONS, W_SSL_CRL, W_SSL_CRLPATH};
 static const
 int dsnparamcnt= sizeof(dsnparams) / sizeof(SQLWCHAR *);
 /* DS_PARAM */
@@ -719,6 +730,10 @@ void ds_delete(DataSource *ds)
   x_free(ds->plugin_dir);
   x_free(ds->default_auth);
   x_free(ds->oci_config_file);
+  x_free(ds->authentication_kerberos_mode);
+  x_free(ds->tls_versions);
+  x_free(ds->ssl_crl);
+  x_free(ds->ssl_crlpath);
   x_free(ds->load_data_local_dir);
 
   x_free(ds->name8);
@@ -747,6 +762,10 @@ void ds_delete(DataSource *ds)
   x_free(ds->plugin_dir8);
   x_free(ds->default_auth8);
   x_free(ds->oci_config_file8);
+  x_free(ds->authentication_kerberos_mode8);
+  x_free(ds->tls_versions8);
+  x_free(ds->ssl_crl8);
+  x_free(ds->ssl_crlpath8);
   x_free(ds->load_data_local_dir8);
 
   x_free(ds);
@@ -767,7 +786,6 @@ int ds_set_strattr(SQLWCHAR **attr, const SQLWCHAR *val)
   return *attr || 0;
 }
 
-
 /*
  * Same as ds_set_strattr, but allows truncating the given string. If
  * charcount is 0 or SQL_NTS, it will act the same as ds_set_strattr.
@@ -786,7 +804,25 @@ int ds_set_strnattr(SQLWCHAR **attr, const SQLWCHAR *val, size_t charcount)
   }
 
   if (val && *val)
-    *attr= sqlwchardup(val, charcount);
+  {
+    SQLWCHAR *temp = sqlwchardup(val, charcount);
+    SQLWCHAR *pos = temp;
+    while (charcount)
+    {
+      *pos = *val;
+      if (charcount > 1 && ((*val == '}' && *(val + 1) == '}')))
+      {
+        ++val;
+        --charcount;
+      }
+
+      ++pos;
+      ++val;
+      --charcount;
+    }
+    *pos = 0; // Terminate the string
+    *attr= temp;
+  }
   else
     *attr= NULL;
   return *attr || 0;
@@ -946,8 +982,6 @@ void ds_map_param(DataSource *ds, const SQLWCHAR *param,
     *booldest= &ds->handle_binary_as_char;
   else if (!sqlwcharcasecmp(W_DFLT_BIGINT_BIND_STR, param))
     *booldest= &ds->default_bigint_bind_str;
-  else if (!sqlwcharcasecmp(W_NO_I_S, param))
-    *booldest= &ds->no_information_schema;
   else if (!sqlwcharcasecmp(W_NO_SSPS, param))
     *booldest= &ds->no_ssps;
   else if (!sqlwcharcasecmp(W_CAN_HANDLE_EXP_PWD, param))
@@ -976,6 +1010,14 @@ void ds_map_param(DataSource *ds, const SQLWCHAR *param,
     *strdest= &ds->load_data_local_dir;
   else if (!sqlwcharcasecmp(W_OCI_CONFIG_FILE, param))
     *strdest= &ds->oci_config_file;
+  else if (!sqlwcharcasecmp(W_AUTHENTICATION_KERBEROS_MODE, param))
+    *strdest= &ds->authentication_kerberos_mode;
+  else if (!sqlwcharcasecmp(W_TLS_VERSIONS, param))
+    *strdest= &ds->tls_versions;
+  else if (!sqlwcharcasecmp(W_SSL_CRL, param))
+    *strdest = &ds->ssl_crl;
+  else if (!sqlwcharcasecmp(W_SSL_CRLPATH, param))
+    *strdest = &ds->ssl_crlpath;
 
   /* DS_PARAM */
 }
@@ -1136,8 +1178,26 @@ int ds_from_kvpair(DataSource *ds, const SQLWCHAR *attrs, SQLWCHAR delim)
     /* remove leading and trailing spaces on value */
     while (*(++split) == ' ');
 
+    auto find_bracket_end = [](const SQLWCHAR *wstr) {
+      const SQLWCHAR *e = wstr;
+      do
+      {
+        e = sqlwcharchr(e, '}');
+        if (e == nullptr || *(e + 1) == 0)
+          break;
+
+        // Found escape }}, skip to the next bracket
+        if (*(e + 1) == '}')
+          e += 2;
+        else
+          break;
+
+      } while (*e);
+      return e;
+    };
+
     /* check for an "escaped" value */
-    if ((*split == '{' && (end= sqlwcharchr(attrs, '}')) == NULL) ||
+    if ((*split == '{' && (end = find_bracket_end(attrs)) == NULL) ||
         /* or a delimited value */
         (*split != '{' && (end= sqlwcharchr(attrs, delim)) == NULL))
       /* otherwise, take the rest of the string */
@@ -1267,18 +1327,22 @@ int ds_to_kvpair(DataSource *ds, SQLWCHAR *attrs, size_t attrslen,
 
 
 /*
- * Calculate the length of serializing this DataSource to a string
- * including null terminator. Given in characters.
+ * Copy data source details into an attribute string. Use attrslen
+ * to limit the number of characters placed into the string.
+ *
+ * Return -1 for an error or truncation, otherwise the number of
+ * characters written.
  */
-size_t ds_to_kvpair_len(DataSource *ds)
+int ds_to_kvpair(DataSource *ds, SQLWSTRING &attrs, SQLWCHAR delim)
 {
-  size_t len= 0;
   int i;
   SQLWCHAR **strval;
   unsigned int *intval;
   BOOL *boolval;
   SQLWCHAR numbuf[21];
 
+  attrs.clear();
+
   for (i= 0; i < dsnparamcnt; ++i)
   {
     ds_map_param(ds, dsnparams[i], &strval, &intval, &boolval);
@@ -1289,32 +1353,37 @@ size_t ds_to_kvpair_len(DataSource *ds)
 
     if (strval && *strval && **strval)
     {
-      len+= sqlwcharlen(dsnparams[i]);
-      len+= sqlwcharlen(*strval);
+      attrs.append(dsnparams[i]);
+      attrs.append({(SQLWCHAR)'='});
       if (value_needs_escaped(*strval))
-        len += 2; /* for escape braces */
-      len+= 2; /* for = and delimiter */
+      {
+        attrs.append({(SQLWCHAR)'{'});
+        attrs.append(escape_brackets(*strval, false));
+        attrs.append({(SQLWCHAR)'}'});
+      }
+      else
+        attrs.append(escape_brackets(*strval, false));
+
+      attrs.append({delim});
     }
+    /* only write out int values if they're non-zero */
     else if (intval && *intval)
     {
-      len+= sqlwcharlen(dsnparams[i]);
+      attrs.append(dsnparams[i]);
+      attrs.append({(SQLWCHAR)'='});
       sqlwcharfromul(numbuf, *intval);
-      len+= sqlwcharlen(numbuf);
-      len+= 2; /* for = and delimiter */
+      attrs.append(escape_brackets(numbuf, false));
+      attrs.append({delim});
     }
     else if (boolval && *boolval)
     {
-      len+= sqlwcharlen(dsnparams[i]);
-      len+= 3; /* for = and delimiter and '1' */
+      attrs.append(dsnparams[i]);
+      attrs.append({(SQLWCHAR)'=', (SQLWCHAR)'1'});
+      attrs.append({delim});
     }
   }
 
-  /*
-     delimiter is always counted at the end, so we don't add one
-     for the null terminator
-   */
-
-  return len;
+  return attrs.length();
 }
 
 
@@ -1415,7 +1484,8 @@ int ds_add(DataSource *ds)
   if (ds_add_strprop(ds->name, W_DESCRIPTION, ds->description)) goto error;
   if (ds_add_strprop(ds->name, W_SERVER     , ds->server     )) goto error;
   if (ds_add_strprop(ds->name, W_UID        , ds->uid        )) goto error;
-  if (ds_add_strprop(ds->name, W_PWD        , ds->pwd        )) goto error;
+  if (ds_add_strprop(ds->name, W_PWD        ,
+    ds->pwd ? escape_brackets(ds->pwd, false).c_str() : nullptr)) goto error;
 #if MFA_ENABLED
   if (ds_add_strprop(ds->name, W_PWD1       , ds->pwd1       )) goto error;
   if (ds_add_strprop(ds->name, W_PWD2       , ds->pwd2       )) goto error;
@@ -1470,7 +1540,6 @@ int ds_add(DataSource *ds)
   if (ds_add_intprop(ds->name, W_COLUMN_SIZE_S32, ds->limit_column_size)) goto error;
   if (ds_add_intprop(ds->name, W_NO_BINARY_RESULT, ds->handle_binary_as_char)) goto error;
   if (ds_add_intprop(ds->name, W_DFLT_BIGINT_BIND_STR, ds->default_bigint_bind_str)) goto error;
-  if (ds_add_intprop(ds->name, W_NO_I_S, ds->no_information_schema)) goto error;
   if (ds_add_intprop(ds->name, W_NO_SSPS, ds->no_ssps)) goto error;
   if (ds_add_intprop(ds->name, W_CAN_HANDLE_EXP_PWD, ds->can_handle_exp_pwd)) goto error;
   if (ds_add_intprop(ds->name, W_ENABLE_CLEARTEXT_PLUGIN, ds->enable_cleartext_plugin)) goto error;
@@ -1485,7 +1554,11 @@ int ds_add(DataSource *ds)
   if (ds_add_intprop(ds->name, W_ENABLE_LOCAL_INFILE, ds->enable_local_infile)) goto error;
   if (ds_add_strprop(ds->name, W_LOAD_DATA_LOCAL_DIR, ds->load_data_local_dir)) goto error;
   if (ds_add_strprop(ds->name, W_OCI_CONFIG_FILE, ds->oci_config_file)) goto error;
- /* DS_PARAM */
+  if (ds_add_strprop(ds->name, W_AUTHENTICATION_KERBEROS_MODE, ds->authentication_kerberos_mode)) goto error;
+  if (ds_add_strprop(ds->name, W_TLS_VERSIONS, ds->tls_versions)) goto error;
+  if (ds_add_strprop(ds->name, W_SSL_CRL, ds->ssl_crl)) goto error;
+  if (ds_add_strprop(ds->name, W_SSL_CRLPATH, ds->ssl_crlpath)) goto error;
+  /* DS_PARAM */
 
   rc= 0;
 
@@ -1575,7 +1648,6 @@ void ds_set_options(DataSource *ds, ulong options)
   ds->allow_multiple_statements=            (options & FLAG_MULTI_STATEMENTS) > 0;
   ds->limit_column_size=                    (options & FLAG_COLUMN_SIZE_S32) > 0;
   ds->handle_binary_as_char=                (options & FLAG_NO_BINARY_RESULT) > 0;
-  ds->no_information_schema=                (options & FLAG_NO_INFORMATION_SCHEMA) > 0;
   ds->default_bigint_bind_str=              (options & FLAG_DFLT_BIGINT_BIND_STR) > 0;
 }
 
@@ -1623,8 +1695,6 @@ ulong ds_get_options(DataSource *ds)
     options|= FLAG_LOG_QUERY;
   if (ds->dont_cache_result)
     options|= FLAG_NO_CACHE;
-  if (ds->no_information_schema)
-    options|= FLAG_NO_INFORMATION_SCHEMA;
   if (ds->force_use_of_forward_only_cursors)
     options|= FLAG_FORWARD_CURSOR;
   if (ds->auto_reconnect)
