@@ -1,4 +1,4 @@
-// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2007, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -98,6 +98,10 @@ typedef struct {
   SQLWCHAR *default_auth;
   SQLWCHAR *load_data_local_dir;
   SQLWCHAR *oci_config_file;
+  SQLWCHAR *authentication_kerberos_mode;
+  SQLWCHAR *tls_versions;
+  SQLWCHAR *ssl_crl;
+  SQLWCHAR *ssl_crlpath;
 
   bool has_port;
   unsigned int port;
@@ -132,6 +136,10 @@ typedef struct {
   SQLCHAR *default_auth8;
   SQLCHAR *load_data_local_dir8;
   SQLCHAR *oci_config_file8;
+  SQLCHAR *authentication_kerberos_mode8;
+  SQLCHAR *tls_versions8;
+  SQLCHAR* ssl_crl8;
+  SQLCHAR* ssl_crlpath8;
 
   /*  */
   BOOL return_matching_rows;
@@ -169,7 +177,6 @@ typedef struct {
   BOOL default_bigint_bind_str;
   /* debug */
   BOOL save_queries;
-  BOOL no_information_schema;
   /* SSL */
   unsigned int sslverify;
   unsigned int cursor_prefetch_number;
@@ -205,9 +212,6 @@ int ds_set_strattr(SQLWCHAR **attr, const SQLWCHAR *val);
 int ds_set_strnattr(SQLWCHAR **attr, const SQLWCHAR *val, size_t charcount);
 int ds_lookup(DataSource *ds);
 int ds_from_kvpair(DataSource *ds, const SQLWCHAR *attrs, SQLWCHAR delim);
-int ds_to_kvpair(DataSource *ds, SQLWCHAR *attrs, size_t attrslen,
-                 SQLWCHAR delim);
-size_t ds_to_kvpair_len(DataSource *ds);
 int ds_add(DataSource *ds);
 int ds_exists(SQLWCHAR *name);
 char *ds_get_utf8attr(SQLWCHAR *attrw, SQLCHAR **attr8);
@@ -258,8 +262,6 @@ extern const SQLWCHAR W_INVALID_ATTR_STR[];
 /*
   Use SHOW TABLE STATUS instead Information_Schema DB for table metadata
 */
-#define FLAG_NO_INFORMATION_SCHEMA (1 << 30)
-
 #define ODBC_SSL_MODE_DISABLED           "DISABLED"
 #define ODBC_SSL_MODE_PREFERRED          "PREFERRED"
 #define ODBC_SSL_MODE_REQUIRED           "REQUIRED"
@@ -272,6 +274,8 @@ extern const SQLWCHAR W_INVALID_ATTR_STR[];
 #ifdef __cplusplus
 }
 #endif
+typedef std::basic_string<SQLWCHAR, std::char_traits<SQLWCHAR>, std::allocator<SQLWCHAR>> SQLWSTRING;
+int ds_to_kvpair(DataSource *ds, SQLWSTRING &attrs, SQLWCHAR delim);
 
 #endif /* _INSTALLER_H */
 
