@@ -1,4 +1,4 @@
-// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2007, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -35,10 +35,63 @@
 
 #include "stringutil.h"
 
-
 CHARSET_INFO *utf8_charset_info = NULL;
 CHARSET_INFO *utf16_charset_info = NULL;
 
+const char *transport_charset = "utf8mb4";
+
+// Scientific floating point notation:
+// the value is represented always with only one digit before the decimal point,
+// followed by the decimal point and as many decimal digits as the precision.
+// Finally, this notation always includes an exponential part consisting on
+// the letter e followed by an optional sign and three exponential digits.
+//
+// https://cplusplus.com/reference/ios/scientific/
+static inline bool is_valid_float_char(char c)
+{
+  return ('0' <= c && c <= '9') ||
+    c == 'e' || c == 'E' ||
+    c == '+' || c == '-';
+}
+
+
+/**
+  Replace a locale dependent decimal point by C decimal dot
+
+  NOTE: Recommended for use only for scientific exponential
+        representation of floating point numbers
+        such as (-1.2345678E+9).
+
+  @param[in, out] buffer containing the string value of float/double
+*/
+void delocalize_radix(char* buffer)
+{
+  // Fast check:  if the buffer has a normal decimal point, assume no
+  // translation is needed.
+  if (strchr(buffer, '.') != nullptr) return;
+
+  // Find the first unknown character.
+  while (is_valid_float_char(*buffer)) ++buffer;
+
+  if (*buffer == '\0') {
+    // No radix character found.
+    return;
+  }
+
+  // We are now pointing at the locale-specific radix character.  Replace it
+  // with '.'.
+  *buffer = '.';
+  ++buffer;
+
+  if (!is_valid_float_char(*buffer) && *buffer != '\0') {
+    // It appears the radix was a multi-byte character.  We need to remove the
+    // extra bytes.
+    char* target = buffer;
+    do { ++buffer; } while (!is_valid_float_char(*buffer) && *buffer != '\0');
+    memmove(target, buffer, strlen(buffer) + 1);
+  }
+}
+
 
 /**
   Duplicate a SQLCHAR in the specified character set as a SQLWCHAR.
@@ -464,7 +517,7 @@ SQLINTEGER sqlwchar_as_sqlchar_buf(CHARSET_INFO *charset_info,
   if (!str || len == 0)
     return 0;
 
-  str_end= str + len;
+  str_end = str + myodbc_min(len, out_bytes);
 
   for (i= 0; str < str_end; )
   {
@@ -490,7 +543,9 @@ SQLINTEGER sqlwchar_as_sqlchar_buf(CHARSET_INFO *charset_info,
                          &used_chars, errors);
   }
 
-  out[i]= '\0';
+  // NULL-terminate only if we have space in out buffer
+  if (i < out_bytes)
+    out[i]= '\0';
 
   return i;
 }
@@ -1072,12 +1127,26 @@ struct CHARSET_COLLATION_INFO charset_collation_info[] = {
   {307, "utf8mb4", "utf8mb4_ru_0900_as_cs", 4},
   {308, "utf8mb4", "utf8mb4_zh_0900_as_cs", 4},
   {309, "utf8mb4", "utf8mb4_0900_bin", 4},
+  {310, "utf8mb4", "utf8mb4_nb_0900_ai_ci", 4},
+  {311, "utf8mb4", "utf8mb4_nb_0900_as_cs", 4},
+  {312, "utf8mb4", "utf8mb4_nn_0900_ai_ci", 4},
+  {313, "utf8mb4", "utf8mb4_nn_0900_as_cs", 4},
+  {314, "utf8mb4", "utf8mb4_sr_latn_0900_ai_ci", 4},
+  {315, "utf8mb4", "utf8mb4_sr_latn_0900_as_cs", 4},
+  {316, "utf8mb4", "utf8mb4_bs_0900_ai_ci", 4},
+  {317, "utf8mb4", "utf8mb4_bs_0900_as_cs", 4},
+  {318, "utf8mb4", "utf8mb4_bg_0900_ai_ci", 4},
+  {319, "utf8mb4", "utf8mb4_bg_0900_as_cs", 4},
+  {320, "utf8mb4", "utf8mb4_gl_0900_ai_ci", 4},
+  {321, "utf8mb4", "utf8mb4_gl_0900_as_cs", 4},
+  {322, "utf8mb4", "utf8mb4_mn_cyrl_0900_ai_ci", 4},
+  {323, "utf8mb4", "utf8mb4_mn_cyrl_0900_as_cs", 4}
 };
 
 
 unsigned int get_charset_maxlen(unsigned int num)
 {
-  if (num < 310)
+  if (num < 324)
     return charset_collation_info[num].maxlen;
 
   return 0;
@@ -1292,12 +1361,13 @@ def:
 SQLWCHAR *wchar_t_as_sqlwchar(wchar_t *from, SQLWCHAR *to, size_t len)
 {
   /* The function deals mostly with short strings */
-  if (len > 1024)
-  len= 1024;
+  if (len > 1023)
+    len= 1023;
 
   if (sizeof(wchar_t) == sizeof(SQLWCHAR))
   {
     memcpy(to, from, len * sizeof(wchar_t));
+    to[len] = 0;
     return to;
   }
   else
@@ -1633,7 +1703,32 @@ bool myodbc_append_os_quoted_std(std::string &str, const char *append, ...) {
   return false;
 }
 
+/*
+ * Escape curly brackets.
+ */
+SQLWSTRING escape_brackets(const SQLWCHAR* val, bool add_start_end)
+{
+  SQLWSTRING src = val;
+  if (!add_start_end &&
+      (src.empty() || src.find((SQLWCHAR)'}') == SQLWSTRING::npos)
+    )
+    return src;
 
+  SQLWSTRING temp;
+  if (add_start_end)
+    temp = { (SQLWCHAR)'{' };
+  // Reserver extra in case we have to escape every bracket
+  temp.reserve(src.length() * 2);
 
-
+  for (SQLWCHAR c : src)
+  {
+    if (c == (SQLWCHAR)'}')
+      temp.append({(SQLWCHAR)'}', (SQLWCHAR)'}'});
+    else
+      temp.append(&c, 1);
+  }
+  if (add_start_end)
+    temp.append({(SQLWCHAR)'}'});
+  return temp;
+}
 #endif
\ No newline at end of file
