@@ -1,4 +1,4 @@
-// Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
+// Copyright (c) 2007, 2022, Oracle and/or its affiliates. All rights reserved.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License, version 2.0, as
@@ -59,6 +59,7 @@ extern "C" {
 
 extern CHARSET_INFO *utf8_charset_info;
 extern CHARSET_INFO *utf16_charset_info;
+extern const char* transport_charset;
 
 struct CHARSET_COLLATION_INFO
 {
@@ -136,9 +137,14 @@ void myodbc_dynstr_free(DYNAMIC_STRING *str);
 bool myodbc_dynstr_realloc(DYNAMIC_STRING *str, size_t additional_size);
 
 unsigned int get_charset_maxlen(unsigned int num);
+void delocalize_radix(char* buffer);
+
 #ifdef __cplusplus
 }
 #endif
 
+typedef std::basic_string<SQLWCHAR, std::char_traits<SQLWCHAR>, std::allocator<SQLWCHAR>> SQLWSTRING;
+SQLWSTRING escape_brackets(const SQLWCHAR* val, bool add_start_end);
+
 #endif /* _STRINGUTIL_H */
 
